@@ -3,7 +3,7 @@ Parsing, annotating, and morphing queries in the Astronomical Data
 Query Language.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -15,14 +15,11 @@ from gavo.adql.annotations import annotate, optimize
 
 from gavo.adql.common import *
 
-from gavo.adql.tree import (
-	getTreeBuildingGrammar, registerNode)
-
-from gavo.adql.nodes import flatten
+from gavo.adql.nodes import flatten, registerNode, getTreeBuildingGrammar
 
 from gavo.adql.grammar import (
 	getADQLGrammar as getRawGrammar, 
-	allReservedWords,
+	ALL_RESERVED_WORDS,
 	ParseException, ParseSyntaxException)
 
 from gavo.adql.morphpg import (
