@@ -2,25 +2,27 @@
 Node classes and factories used in ADQL tree processing.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
 
 import fnmatch
+import functools
+import itertools
 import re
 import weakref
-
-import pyparsing
+from functools import reduce
 
 from gavo import stc
 from gavo import utils
 from gavo.adql import common
 from gavo.adql import fieldinfo
 from gavo.adql import fieldinfos
+from gavo.adql import grammar
 from gavo.stc import tapstc
-from functools import reduce
+from gavo.utils import parsetricks
 
 
 ################ Various helpers
@@ -74,7 +76,7 @@ def flatten(arg):
 		return arg
 	elif isinstance(arg, (int, float)):
 		return repr(arg)
-	elif isinstance(arg, pyparsing.ParseResults):
+	elif isinstance(arg, parsetricks.ParseResults):
 		return " ".join(flatten(c) for c in arg)
 #	elif arg is None: import pdb;pdb.Pdb(nosigint=True).set_trace()
 	else:
@@ -1379,7 +1381,8 @@ class SelectList(ADQLNode):
 			# Will be filled in by query, we don't have the from clause here.
 			selectFields = None  #noflake: locals returned
 		else:
-			selectFields = list(_parseResult.get("fieldSel")) #noflake: locals returned
+			selectFields = list(itertools.chain(#noflake: locals returned
+				*_parseResult.get("fieldSel"))) 
 		return locals()
 	
 	def flatten(self):
@@ -1556,7 +1559,7 @@ class GenericValueExpression(CombiningFINode, TransparentMixin):
 	It is returned by the makeValueExpression factory below to collect
 	stray children.
 	"""
-	type = "valueExpression"
+	type = "genericValueExpression"
 	collapsible = True
 
 	def _combineFieldInfos(self):
@@ -1708,8 +1711,9 @@ class TimestampFunction(FunctionNode):
 			ucd, stc = subordinates[0].fieldInfo.ucd, subordinates[0].fieldInfo.stc
 		else:
 			ucd, stc = None, None
+		userData, tainted = collectUserData(subordinates)
 		self.fieldInfo = fieldinfo.FieldInfo("timestamp", "",
-			ucd=ucd, stc=stc, userData=subordinates)
+			ucd=ucd, stc=stc, userData=userData, tainted=tainted)
 
 
 class InUnitFunction(FieldInfoedNode):
@@ -1805,6 +1809,29 @@ class CastSpecification(FieldInfoedNode, TransparentMixin):
 		else:
 			self.fieldInfo = fieldinfo.FieldInfo(self.newType, "", "")
 
+	def flatten(self):
+		if self.children[-2].lower().endswith("char ( * )"):
+			# postgres, surprisingly, can't cast to char(*), so we just hack
+			# it to text (the endswith is for national char)
+			self.children[-2] = "TEXT"
+		return FieldInfoedNode.flatten(self)
+
+
+class CoalesceExpression(FieldInfoedNode, TransparentMixin):
+	type = "coalesceExpression"
+
+	# The trouble as to inference here is that of course anything
+	# might happen per-row, as people could stick in whatever.
+	# But we assume well-meaning people and thus just take the annotation
+	# from the first argument (which is supposed to be the "normal case").
+	def addFieldInfo(self, context):
+		subordinates = self._getInfoChildren()
+		if subordinates:
+			self.fieldInfo = subordinates[0].fieldInfo.change(tainted=True)
+		else:
+			self.fieldInfo = None
+
+
 
 ###################### Geometry and stuff that needs morphing into real SQL
 
@@ -2269,3 +2296,41 @@ class Area(FunctionNode):
 		self.fieldInfo = fieldinfo.FieldInfo(type="double precision",
 			unit="deg**2", ucd="phys.angSize", 
 			userData=collectUserData(self._getInfoChildren())[0])
+
+
+_ADDITIONAL_NODES = []
+def registerNode(node):
+	"""registers a node class or a symbolAction from a module other than node.
+
+	This is a bit of magic -- some module can call this to register a node
+	class that is then bound to some parse action as if it were in nodes.
+
+	I'd expect this to be messy in the presence of chaotic imports (when
+	classes are not necessarily singletons and a single module can be
+	imported more than once.  For now, I ignore this potential bomb.
+	"""
+	# adding a node probably changes the grammar bindings, so we
+	# need to clear the cached grammar
+	getTreeBuildingGrammar.cache_clear()
+	_ADDITIONAL_NODES.append(node)
+
+
+def getNodeClasses():
+	"""returns a list of node classes (and standalone parse actions) available
+	for tree building.
+
+	This is what needs to be passed to adql.getADQLGrammarCopy to get
+	a proper parse tree out of the parser.
+	"""
+	res = []
+	for item in itertools.chain(globals().values(), _ADDITIONAL_NODES):
+		if isinstance(item, type) and issubclass(item, ADQLNode):
+			res.append(item)
+		if hasattr(item, "parseActionFor"):
+			res.append(item)
+	return res
+
+
+@functools.lru_cache(1)
+def getTreeBuildingGrammar():
+	return grammar.getADQLGrammarCopy(getNodeClasses())
