@@ -5,19 +5,19 @@ system.
 See the userFunction docstring on how to use these.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
 
+from gavo import base
 from gavo import stc
 from gavo import utils
 from gavo.adql import common
 from gavo.adql import fieldinfo
 from gavo.adql import morphhelpers
 from gavo.adql import nodes
-from gavo.adql import tree
 
 
 UFUNC_REGISTRY = {}
@@ -36,8 +36,13 @@ def getStringLiteral(node, description="Argument"):
 	return node.value
 
 
-def userFunction(name, signature, doc, returntype="double precision", 
-		unit="", ucd=""):
+def userFunction(name, 
+		signature, 
+		doc, 
+		returntype="double precision", 
+		unit="", 
+		ucd="",
+		polymorphism=()):
 	"""a decorator adding some metadata to python functions to make
 	them suitable as ADQL user defined functions.
 
@@ -55,6 +60,10 @@ def userFunction(name, signature, doc, returntype="double precision",
 	case, they'll be passed the (annotated) arguments, and whatever
 	they return will be the unit/ucd.
 
+	polymorphism, if given, must be a sequence of pairs of 
+	(signature, documentation) for alternative signatures of the
+	UDF.
+
 	The python function receives an array of arguments; this will in
 	general be ADQL expression trees.  It must return either 
 	
@@ -78,6 +87,9 @@ def userFunction(name, signature, doc, returntype="double precision",
 		f.adqlUDF_returntype = returntype
 		f.adqlUDF_unit = unit
 		f.adqlUDF_ucd = ucd
+		f.adqlUDF_additionalSignatures = [
+				{"signature": f.adqlUDF_name+signature.strip(), "doc": doc}
+			for signature, doc in polymorphism]
 		UFUNC_REGISTRY[f.adqlUDF_name.upper()] = f
 		return f
 	return deco
@@ -122,7 +134,7 @@ def _match(args):
 	gavo_hasword returns 1 if needle shows up in haystack, 0 otherwise.  This
 	is for "google-like"-searches in text-like fields.  In word, you can
 	actually employ a fairly complex query language; see
-	http://www.postgresql.org/docs/8.3/static/textsearch.html
+	https://www.postgresql.org/docs/current/textsearch.html
 	for details.
 	""",
 	"integer")
@@ -550,6 +562,175 @@ _makeBooleanizer("gavo_vocmatch",
 	"%(1)s IN %(2)s")
 
 
+# TODO: move the basic spectral coversion stuff to base.unitconv
+# (and then fix WIRR's dbadapter.py, too).
+
+def toStdUnit(fromUnit):
+	"""returns a pair of (standard unit, factor) to turn fromUnit into
+	one of J, Hz, or m.
+
+	Unconvertable units raise a UfuncError.
+	"""
+	for stdUnit in ["J", "Hz", "m"]:
+		try:
+			 factor = base.computeConversionFactor(fromUnit, stdUnit)
+		except base.IncompatibleUnits:
+			continue
+		return stdUnit, factor
+	
+	raise common.UfuncError(
+		f"specconv: {fromUnit} is not a spectral unit understood here")
+
+
+SPEC_CONVERSION = {
+	("J", "m"): "h*c/(({expr})*{f})",
+	("J", "Hz"): "({expr})*{f}/h",
+	("J", "J"): "({expr})*{f}",
+	("Hz", "m"): "c/({expr})/{f}",
+	("Hz", "Hz"): "{f}*({expr})",
+	("Hz", "J"): "h*{f}*({expr})",
+	("m", "m"): "{f}*({expr})",
+	("m", "Hz"): "c/({expr})/{f}",
+	("m", "J"): "h*c/({expr})/{f}",}
+
+
+def getSpecExpr(fromUnit, toUnit):
+	"""returns an expression that turns a value in fromUnit to one in toUnit.
+
+	The value in fromUnit is represented by a {} so you can just .format
+	the expression it is in into the resulting string.
+	"""
+	fromStd, fromFactor = toStdUnit(fromUnit)
+	toStd, toFactor = toStdUnit(toUnit)
+	convExpr = SPEC_CONVERSION[fromStd, toStd
+			].replace("h", str(base.PLANCK_H)
+			).replace("c", str(base.LIGHT_C))
+	return "{}/{}".format(
+		convExpr.format(f=fromFactor, expr="{}"),
+		toFactor)
+
+
+class _SpecconvNode(nodes.FieldInfoedNode):
+	"""A node left by the two-argument gavo_specconv.
+
+	It is constructed with the target unit and the expression to
+	convert.  During annotation, it will mutate its children so they
+	are in the target unit.
+	"""
+	_a_targetUnit = None
+	_a_forceUnit = None
+	_a_expr = None
+
+	def _getFromUnit(self, context):
+		"""returns the unit we transform from.
+
+		This needs the context because it might produce warnings.
+		"""
+		fromUnit = None
+		try:
+			fromUnit = self.expr.fieldInfo.unit
+			if not fromUnit:
+				raise AttributeError("Empty from Unit")
+		except AttributeError:
+			pass # handle missing fromUnit below
+
+		if self.forceUnit:
+			if fromUnit:
+				context.warnings.append("Warning in specconv: overriding"
+					f" unit {fromUnit} with {self.forceUnit}")
+			fromUnit = self.forceUnit
+
+		if not fromUnit:
+			raise common.UfuncError("specconv: Cannot infer unit of first"
+				" argument.  Perhaps use the three-argument specconv?")
+		
+		return fromUnit
+
+	def _replaceExpr(self, sentinelName, newExpr):
+		"""replaces self.expr with newExpr, where a (fake) column
+		reference named sentinelName is replaced by the previous expression.
+		"""
+		def replaceTarget(nodes):
+			for ind, child in enumerate(nodes):
+				if getattr(child, "name", None)==sentinelName:
+					nodes[ind] = self.expr
+					return True
+				if hasattr(child, "children"):
+					if replaceTarget(child.children):
+						return True
+
+		if replaceTarget(newExpr.children):
+			self.expr = newExpr
+		else:
+			raise common.UfuncError("specconv: Internal Error: No sentinel found.")
+
+	def addFieldInfo(self, context):
+		nodes.FieldInfoedNode.addFieldInfo(self, context)
+
+		fromUnit = self._getFromUnit(context)
+
+		exprLit = getSpecExpr(fromUnit, self.targetUnit
+			).format("REPLACE_ME")
+		exprNode = nodes.getTreeBuildingGrammar()[0][
+				"userDefinedFunctionParam"
+			].parseString(exprLit, parseAll=True)[0]
+		
+		self._replaceExpr("REPLACE_ME", exprNode)
+
+		self.fieldInfo = self.fieldInfo.change(unit=self.targetUnit)
+	
+	def flatten(self):
+		return self.expr.flatten()
+
+
+@userFunction("gavo_specconv",
+	"(expr DOUBLE PRECISION, dest_unit TEXT) -> DOUBLE PRECISION",
+	"""returns the spectral value expr converted to dest_unit.
+
+	expr has to be in either energy, wavelength, or frequency, and dest_unit
+	must be a VOUnit giving another spectral unit (e.g., MHz, keV, nm, or
+	Angstrom). This is intended to let users express spectral constraints
+	in their preferred unit independently of the choice of unit in the 
+	database.  Examples::
+
+		gavo_specconv(obscore.em_min, "keV") > 300
+		gavo_specconv(obscore.em_max, "MHz") > 30
+		gavo_specconv(spectral_start, "Angstrom") > 4000
+	
+	There is a variant of gavo_specconv accepting expr's unit in a third
+	argument.
+	""",
+	polymorphism=[(
+		"(expr NUMERIC, expr_unit TEXT dest_unit TEXT) -> NUMERIC",
+		"""returns expr assumed to be in expr_unit expressed in dest_unit.
+
+		This is a variant of the two-argument gavo_specconv for when
+		the unit of expr is not known to the ADQL translator, either because
+		it because it is a literal or because it does not look like
+		a spectral unit.  Examples::
+
+			gavo_specconv(656, 'nm', 'J') BETWEEN spectral_start AND spectral_end
+			gavo_specconv(arccos(phi)*incidence, 'Hz', 'eV')
+		
+		Clearly, overriding known units is likely to yield bad results;
+		the translator therefore warns if an existing unit is overridden
+		with a different unit.""")])
+def _gavo_specconv(args):
+	if len(args)==2:
+		newNode = _SpecconvNode(expr=args[0], 
+			targetUnit=getStringLiteral(args[1]))
+
+	elif len(args)==3:
+		newNode = _SpecconvNode(expr=args[0], 
+			targetUnit=getStringLiteral(args[2]),
+			forceUnit=getStringLiteral(args[1]))
+	
+	else:
+		raise common.UfuncError("gavo_specconv takes two or three arguments")
+
+	raise nodes.ReplaceNode(newNode)
+	
+
 class UserFunction(nodes.FunctionNode):
 	"""A node processing user defined functions.
 
@@ -589,4 +770,4 @@ class UserFunction(nodes.FunctionNode):
 			ucd)
 
 
-tree.registerNode(UserFunction)
+nodes.registerNode(UserFunction)
