@@ -3,7 +3,7 @@ Basic code for defining objects in the data center: Structures, their
 attributes, fundamental VO conventions.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -25,7 +25,8 @@ from gavo.base.complexattrs import *
 
 from gavo.base.config import(
 	get as getConfig, set as setConfig,
-	getDBProfile)
+	getDBProfile, 
+	makeFallbackMeta as _makeFallbackMeta)
 
 from gavo.base.events import EventDispatcher
 
@@ -52,6 +53,9 @@ from gavo.base.typedmeta import (
 	InfoItem as MetaInfoItem,
 )
 
+# now that typed meta is registered, pull in the fallback metadata
+_makeFallbackMeta()
+
 from gavo.base.metavalidation import MetaValidationError, validateStructure
 
 from gavo.base.observer import ObserverBase, listensTo
@@ -73,7 +77,8 @@ from gavo.base.sqlsupport import (getDBConnection,
 	getTableConn, getAdminConn, getUntrustedConn,
 	getWritableTableConn, getWritableAdminConn, getWritableUntrustedConn,
 	setDBMeta, getDBMeta,
-	NUMERIC_TYPES, ORDERED_TYPES)
+	NUMERIC_TYPES, ORDERED_TYPES,
+	NonBlockingQuery)
 
 from gavo.base.structure import (Structure, ParseableStructure, 
 	DataContent, makeStruct, RestrictionMixin)
