@@ -19,7 +19,7 @@ macros within the loop are expanded not within some FEED element but
 within the RD.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -298,6 +298,11 @@ class Prune(ActiveTag, structure.Structure):
 	
 	This only works if the items to be matched are true XML attributes
 	(i.e., not written as children).
+
+	For instance, the following will filter out all elements with a name
+	of VERB from the stream::
+
+		<PRUNE name="VERB"/>
 	"""
 	name_ = "PRUNE"
 	
@@ -476,7 +481,15 @@ class ReplayBase(ActiveTag, structure.Structure, macros.StandardMacroMixin):
 		evTarget.setRoot(destination)
 
 		self._ensureEditsDict()
-		self._replayTo(events, evTarget, ctx)
+		try:
+			previousTarget = getattr(ctx, "replayTarget", None)
+			ctx.replayTarget = destination
+			self._replayTo(events, evTarget, ctx)
+		finally:
+			if previousTarget:
+				ctx.replayTarget = previousTarget
+			else:
+				delattr(ctx, "replayTarget")
 
 
 class DelayedReplayBase(ReplayBase, GhostMixin):
@@ -528,6 +541,12 @@ class ReplayedEventsWithFreeAttributesBase(DelayedReplayBase):
 		try:
 			return DelayedReplayBase.getAttribute(self, name)
 		except common.StructureError: # no "real" attribute, it's a macro def
+
+			if len(name)==1:
+				raise common.StructureError("DaCHS does not support one-character"
+					f' macro names.  Hence, you cannot use "{name}" as a {self.name_}'
+					" attribute.")
+
 			def m():
 				return getattr(self, name)
 			setattr(self, "macro_"+name.strip(), m)
@@ -569,9 +588,9 @@ class GeneratorAttribute(attrdef.UnicodeAttribute):
 		src = utils.fixIndentation(
 			getattr(instance, self.name_), 
 			"  ", governingLine=1)
-		src = "def makeRows():\n"+src+"\n"
+		src = "def makeRows(context):\n"+src+"\n"
 		instance.iterRowsFromCode = utils.compileFunction(
-			src, "makeRows", useGlobals={"context": ctx})
+			src, "makeRows", useGlobals={"definitionContext": ctx})
 
 
 class Loop(ReplayedEventsWithFreeAttributesBase):
@@ -590,7 +609,8 @@ class Loop(ReplayedEventsWithFreeAttributesBase):
 	_codeItems = GeneratorAttribute("codeItems", default=None,
 		description="A python generator body that yields dictionaries"
 		" that are then used as loop items.  You can access the parse context"
-		" as the context variable in these code snippets.", strip=False)
+		" as the context variable in these code snippets.  context.replayTarget"
+		" is the struct the loop is feeding to.", strip=False)
 
 	def maybeExpand(self, val):
 		if "\\" in val:
@@ -618,16 +638,16 @@ class Loop(ReplayedEventsWithFreeAttributesBase):
 
 		return csv.DictReader(StringIO(src), skipinitialspace=True)
 
-	def _makeRowIteratorFromCode(self):
+	def _makeRowIteratorFromCode(self, ctx):
 		if self.codeItems is None:
 			return None
-		return self.iterRowsFromCode()
+		return self.iterRowsFromCode(ctx)
 
-	def _getRowIterator(self):
+	def _getRowIterator(self, ctx):
 		rowIterators = [ri for ri in [
 			self._makeRowIteratorFromListItems(),
 			self._makeRowIteratorFromCSV(),
-			self._makeRowIteratorFromCode()] if ri]
+			self._makeRowIteratorFromCode(ctx)] if ri]
 		if len(rowIterators)!=1:
 				raise common.StructureError("Must give exactly one data source in"
 					" LOOP")
@@ -635,7 +655,7 @@ class Loop(ReplayedEventsWithFreeAttributesBase):
 			
 	def completeElement(self, ctx):
 		self._completeElementNext(Loop, ctx)
-		for row in self._getRowIterator():
+		for row in self._getRowIterator(ctx):
 			for name, value in row.items():
 				if value:
 					value = value.strip()
