@@ -2,7 +2,7 @@
 Definition of DC config options and their management including I/O.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -13,7 +13,6 @@ import os
 import re
 import shlex
 import sys
-import warnings
 
 from gavo import utils
 from gavo.base import attrdef
@@ -217,6 +216,8 @@ class ProfileParser(object):
 			self._raiseError("unexpected end of file (missing line feed?)")
 		if profileName:
 			self.profile.profileName = profileName
+
+		stream.close()
 		return self.profile
 
 	def _raiseError(self, msg):
@@ -280,7 +281,6 @@ class ProfileParser(object):
 		self.stateFun = self.stateFun(token)
 
 
-
 class Configuration(fancyconfig.Configuration):
 	"""A container for settings.
 
@@ -341,8 +341,6 @@ _config = Configuration(
 				" of space if your users do large queries"),
 		EnumeratedConfigItem("logLevel", options=["info", "warning",
 			"debug", "error"], description="How much should be logged?"),
-		StringConfigItem("operator", description=
-			"Deprecated and ignored.  maintainerAddress instead."),
 		StringConfigItem("platform", description="Platform string (can be"
 			" empty if inputsDir is only accessed by identical machines)"),
 		StringConfigItem("gavoGroup", description="Name of the unix group that"
@@ -366,6 +364,17 @@ _config = Configuration(
 				" this could be the same as contact.email; in practice, it is"
 				" shown in more technical circumstances, so it's adviable"
 				" to have a narrower distribution here."),
+		SetConfigItem("future", default="",
+			description="A set of strings naming experimental features"
+				" to enable in the server.  You only want to touch this"
+				" if you are working on proving proposed improvements."),
+		SetConfigItem("rdblacklist", default="",
+			description="File names of blacklisted RDs.  These are matched"
+				" against the ends of file names, so be careful you do not"
+				" over-block; in general, you should use something like"
+				" '/resname/q.rd' or so.  You can also blacklist RD ids"
+				" (which are compared literally). Separate multiple entries"
+				" by commas."),
 		),
 
 	Section('web', 'Settings related to serving content to the web.',
@@ -408,7 +417,8 @@ _config = Configuration(
 		StringConfigItem("sitename", "Unnamed data center",
 			"A short name for your site"),
 		IntConfigItem("sqlTimeout", "15",
-			"Default timeout for db queries via the web"),
+			"Default timeout for non-TAP database queries (which can in"
+			" general be overridden by users setting _TIMEOUT)."),
 		WebRelativeConfigItem("previewCache", "previewcache",
 			"Webdir-relative directory to store cached previews in"),
 		WebRelativeConfigItem("favicon", "None",
@@ -464,6 +474,10 @@ _config = Configuration(
 			" renders the root.html template.  On single-service installations,"
 			" you could direct this to that single service, or you could"
 			" use the ADQL query form."),
+		BooleanConfigItem("ignore-uir-header", "False",
+			"Do not honour clients' upgrade-insecure-requests headers if"
+			" https is enabled.  Outside of development or testing environments,"
+			" there is generally little reason to use this.")
 	),
 
 	Section('adql', "(ignored, only left for backward compatibility)",
@@ -473,8 +487,8 @@ _config = Configuration(
 	),
 
 	Section('async', "Settings concerning TAP, UWS, and friends",
-		IntConfigItem("defaultExecTimeSync", "60", "Default timeout"
-			" for synchronous UWS jobs, in seconds"),
+		IntConfigItem("defaultExecTimeSync", "60", "Timeout"
+			" for synchronous TAP/UWS jobs, in seconds."),
 		IntConfigItem("defaultExecTime", "3600", "Default timeout"
 			" for UWS jobs, in seconds"),
 		IntConfigItem("maxTAPRunning", "2", "Maximum number of"
@@ -590,25 +604,27 @@ if "GAVO_INPUTSDIR" in os.environ:
 
 get = _config.get
 set = _config.set
+getitem = _config.getitem
+
 getDBProfile = _config.getDBProfile
 
 
 def makeFallbackMeta(reload=False):
 	"""fills meta.configMeta with items from $configDir/defaultmeta.txt.
+
+	This is called from the module __init__ once typed meta from base
+	is in (which means that any typed meta defined in other places won't
+	become magic in fallback; regrettably, we can't hold off loading fallback
+	meta much longer).
 	"""
 	srcPath = os.path.join(get("configDir"), "defaultmeta.txt")
 	if not os.path.exists(srcPath):
-		# python warning rather than event interface since this is very early
-		# init.
-		warnings.warn("%s does not exist, registry interface  will be broken"%
-			srcPath)
+		# this will be made a bit later in dachs init
 		return
 	with open(srcPath, "rb") as f:
 		content = f.read().decode("utf-8", "ignore")
 	meta.parseMetaStream(meta.configMeta, content, clearItems=reload)
 
-makeFallbackMeta()
-
 
 def getRSTReference(underlineChar="-"):
 	"""returns a ReStructuredText reference of configuration items.
