@@ -5,7 +5,7 @@ between them.
 Basically all of this should be taken over by stc and astropysics.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -17,6 +17,7 @@ from math import sin, cos, pi #noflake: exported names
 import re
 
 import numpy
+from numpy import linalg
 
 from gavo import utils
 from gavo.utils import DEG
@@ -138,10 +139,18 @@ def _calcFootprintMonkeypatch(self, hdr=None, undistort=True):
 	"""
 	naxis1, naxis2 = self._monkey_naxis_lengths
 	corners = [[1,1],[1,naxis2], [naxis1,naxis2], [naxis1, 1]]
+
+	# we want our polygons to have a counterclockwise winding direction,
+	# and until I think about this a bit more deeply or get a sufficiently
+	# capable library, I just revert my sense if the determinant of the
+	# CD matrix is negative.  That's gut feeling, not proven right.
+	if linalg.det(self.pixel_scale_matrix)<0:
+		corners.reverse()
+
 	if undistort:
 		return self.all_pix2world(corners, 1)
 	else:
-		return self.wcs_pix2world(corners,1)
+		return self.wcs_pix2world(corners, 1)
 
 
 def _monkeypatchWCS(wcsObj, naxis, wcsFields):
