@@ -2,7 +2,7 @@
 A macro mechanism primarily for string replacement in resource descriptors.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -12,11 +12,6 @@ import datetime
 import re
 import urllib.request, urllib.parse, urllib.error
 
-from pyparsing import (
-	ZeroOrMore, Forward,
-	Regex, Suppress,
-	Literal)
-
 
 from gavo import utils
 from gavo.base import attrdef
@@ -26,6 +21,11 @@ from gavo.base import config
 from gavo.base import meta
 from gavo.base import osinter
 from gavo.base import structure
+from gavo.utils.parsetricks import (
+	ZeroOrMore, Forward,
+	Regex, Suppress,
+	Literal, pyparsingWhitechars)
+
 
 
 class MacroError(common.StructureError):
@@ -85,7 +85,7 @@ class MacroExpander(object):
 		return utils.pyparseTransform(self._macroGrammar, aString)
 
 	def _getMacroGrammar(self, debug=False):
-		with utils.pyparsingWhitechars(" \t"):
+		with pyparsingWhitechars(" \t"):
 			macro = Forward()
 			quoteEscape = (Literal("\\{").addParseAction(lambda *args: "{") | 
 				Literal("\\}").addParseAction(lambda *args: "}"))
