@@ -31,7 +31,7 @@ use META_CLASSES_FOR_KEYS (but don't change that directory except
 through @meta.forKeys).
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -506,6 +506,25 @@ class MetaMixin(object):
 			ex.carrier = self
 			raise
 		return default
+	
+	def hasMeta(self, atom, propagate=False):
+		"""returns true if this meta carrier has an atom metadatum.
+
+		This will *not* parse atom, so you cannot (yet) pass in hierarchical
+		meta names.
+
+		With propagate, it will also ask the meta parents.
+
+		In user code, it is generally preferable to just call getMeta
+		with raiseOnFail.  This is mainly a shortcut for internal code.
+		"""
+		if atom in self.meta_:
+			return True
+		if propagate:
+			parent = self.getMetaParent()
+			if parent:
+				return parent.hasMeta(atom, True)
+		return False
 
 	def _iterMeta(self, atoms):
 		mine, others = atoms[0], atoms[1:]
@@ -1175,8 +1194,6 @@ class ModelBasedBuilder(object):
 			def processContent(childContent, metaItem):
 				moreAttrs = {}
 				for argName, metaKey in attrs.items():
-					if metaKey=="sets":
-						import pdb;pdb.Pdb(nosigint=True).set_trace()
 					val = metaItem.getMeta(metaKey)
 					if val:
 						moreAttrs[argName] = val.getContent("text", 
