@@ -6,7 +6,7 @@ This is currently very postgres specific.  If we really wanted to
 support some other database, this would need massive refactoring.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -330,6 +330,8 @@ class GAVOConnection(psycopg2.extensions.connection):
 		in the select-list.  To fix this, you can pass in a caseFixer dict
 		that gives a properly cased version of lowercase names.
 
+		Timeout is in seconds.
+
 		Warning: this is an iterator, so unless you iterate over the result,
 		the query will not get executed.  Hence, for non-select statements
 		you will generally have to use conn.execute.
@@ -980,6 +982,74 @@ class AdhocQuerier(QuerierMixin):
 		return self._cm.__exit__(*args)
 
 
+class NonBlockingQuery:
+	"""a query run in a pseudo-nonblocking way.
+
+	While psycopg2 can do proper async, that doesn't play well with 
+	about everything else DaCHS is doing so far.  So, here's a quick
+	way to allow long-running queries that users can still interrupt.
+	The ugly secret is that it's based on threads.
+
+	This should not be used within the server.  We might want to port
+	the async taprunner (which runs outside of the server) to using this, 
+	though.
+
+	To use it, construct it with conn, query and perhaps args and
+	use it as a context manager.
+
+	Wait for its result attribute to become non-None; this will then
+	be either a list of result rows or an Exception (which will also be
+	raised when exiting the context manager).
+
+	To abort a running query, call abort().
+	"""
+	def __init__(self, conn, query, args={}):
+		self.conn, self.query, self.args = conn, query, args
+		self.backendPID = list(self.conn.query("SELECT pg_backend_pid()"))[0]
+
+		# will be set only from the thread
+		self.result = None
+
+	def __enter__(self):
+		self.thread = threading.Thread(target=self._runQuery)
+		self.thread.setDaemon(True)
+		self.thread.start()
+		return self
+
+	def __exit__(self, *excInfo):
+		self.cleanup(1)
+		if excInfo==(None, None, None) and isinstance(self.result, Exception):
+			# this probably is a memory leak, which is one of the reasons
+			# this shouldn't be used in the server without more thought
+			raise self.result
+		return False
+
+	def _runQuery(self):
+		try:
+			self.result = list(self.conn.query(self.query, self.args))
+		except QueryCanceledError:
+			# assume this happened on user request
+			pass
+		except Exception as ex:
+			self.result = ex
+
+	def abort(self):
+		"""aborts the current query and reaps the thread.
+		"""
+		self.conn.cancel()
+		self.cleanup(1)
+
+	def cleanup(self, timeout=None):
+		"""tries to reap the thread (i.e., join it).
+
+		If the thread hasn't terminated within timeout seconds, a 
+		sqlsupport.Error is raised.
+		"""
+		self.thread.join(timeout=timeout)
+		if self.thread.is_alive():
+			raise Error("Could not join NonBlockingQuery")
+
+
 def setDBMeta(conn, key, value):
 	"""adds/overwrites (key, value) in the dc.metastore table within
 	conn.
@@ -1105,7 +1175,7 @@ class CustomConnectionPool(psycopg2.pool.ThreadedConnectionPool):
 		cls.knownPools = []
 
 	def _connect(self, key=None):
-		"""creates a new trustedquery connection and assigns it to
+		"""creates a new connection with our selected profile and assigns it to
 		key if not None.
 
 		This is an implementation detail of psycopg2's connection
@@ -1154,7 +1224,7 @@ def _cleanupAfterDBError(ex, conn, pool, poolLock):
 			"Disaster: %s while force-closing connection"%msg)
 
 
-def _makeConnectionManager(profileName, minConn=5, maxConn=20,
+def _makeConnectionManager(profileName, minConn=2, maxConn=20,
 		autocommitted=True):
 	"""returns a context manager for a connection pool for profileName
 	connections.
