@@ -6,7 +6,7 @@ of DaCHS tables is an ObjectAnnotation, the other Annotation classes
 (conceptually, all are key-value pairs) make up their inner structure.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -153,14 +153,17 @@ class AtomicAnnotation(AnnotationBase):
 					**attrs)
 			ctx.getEnclosingContainer()[
 				votable.serializeToParam(param, self.value)]
-			return V.CONSTANT(ref=param.id)
+			ctx.curDMAttr(ref=param.id)
+			return None
 
 		else:
 			# no additional metadata, just dump a literal string
 			# (technically, we should use ivoa: types.  Oh, bother, another
 			# type system, just what we need).
-			return V.LITERAL(dmtype="ivoa:string")[
-				utils.safe_str(self.value)]
+			ctx.curDMAttr(
+				dmtype="ivoa:string",
+				value=utils.safe_str(self.value))
+			return None
 
 
 	def asSIL(self, suppressType=False):
@@ -274,7 +277,10 @@ class _AttributeGroupAnnotation(AnnotationBase, _WithMapCopyMixin):
 			"\n  ".join(r.asSIL() for r in list(self.childRoles.values())))
 
 	def getVOT(self, ctx, instance):
-		"""helps getVOT.
+		"""inserts this annotation into a VOTable.
+
+		ctx is the votable serialisation context, instance is the annotated
+		DaCHS object.
 		"""
 		ctx.addVODMLPrefix(self.modelPrefix)
 		ctx.groupIdsInTree.add(id(self))
@@ -283,8 +289,10 @@ class _AttributeGroupAnnotation(AnnotationBase, _WithMapCopyMixin):
 				ID=ctx.getOrMakeIdFor(self))
 		
 		for role, ann in self.childRoles.items():
-			res[V.ATTRIBUTE(dmrole=completeVODMLId(ctx, role))[
-					ann.getVOT(ctx, instance)]]
+			with ctx.settingAttribute(
+					V.ATTRIBUTE(dmrole=completeVODMLId(ctx, role))) as attr:
+				attr[ann.getVOT(ctx, instance)]
+				res[attr]
 
 		return res
 
@@ -352,14 +360,17 @@ class CollectionAnnotation(AnnotationBase, _WithMapCopyMixin):
 			"\n  ".join(bodyItems))
 
 	def getVOT(self, ctx, instance):
-# So... it's unclear at this point what to do here -- I somehow feel
-# we should serialise collections into a table.  But then this would
-# entail one table each whenever an attribute is potentially sequence-valued,
-# and that doesn't seem right either.  So, we'll dump multiple things
-# into one attribute for now and see how far we get with than.
 		if self.type:
 			ctx.addVODMLPrefix(self.modelPrefix)
-		return [c.getVOT(ctx, instance) for c in self.children]
+
+		res = V.COLLECTION()
+		for c in self.children:
+			with ctx.settingAttribute(
+					V.ITEM()) as item:
+				item[c.getVOT(ctx, instance)]
+				res[item]
+
+		return res
 
 	
 def _test():
