@@ -2,7 +2,7 @@
 Output formats.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -12,4 +12,5 @@ Output formats.
 
 from gavo.formats.common import (formatData, getFormatted, getMIMEFor,
 	registerDataWriter, CannotSerializeIn, iterFormats, getWriterFor,
-	getLabelFor, getMIMEKey, guessMediaType, getExtensionFor)
+	getLabelFor, getMIMEKey, guessMediaType, getExtensionFor,
+	getAliasesFor, getTAPIdFor, getKeyFor)
