@@ -9,7 +9,7 @@ VOTable FIELDs and the corresponding values.
 You should access this module through formats.votable.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -83,6 +83,7 @@ class VOTableContext(utils.IdManagerMixin):
 		self.overflowElement = overflowElement
 		self._containerStack = []
 		self._tableStack = []
+		self._dmAttrStack = []
 		self._pushedRefs = {}
 
 		# state for VO-DML serialisation
@@ -184,6 +185,28 @@ class VOTableContext(utils.IdManagerMixin):
 		finally:
 			self._tableStack.pop()
 
+	@contextlib.contextmanager
+	def settingAttribute(self, dmAttr):
+		"""a context manager controlling the curDMAttr attribute.
+
+		This is used by the DM annotators that sometimes need to directly
+		manipulate the ATTRIBUTE element.
+		"""
+		self._dmAttrStack.append(dmAttr)
+		try:
+			yield dmAttr
+		finally:
+			self._dmAttrStack.pop()
+	
+	@property
+	def curDMAttr(self):
+		"""returns the V.ATTRIBUTE element currently worked on.
+
+		This will raise an IndexError if not ATTRIBUTE is being built at the
+		moment.
+		"""
+		return self._dmAttrStack[-1]
+
 	def pushRefFor(self, rdEl, refVal):
 		"""orders refVal to be set as ref on rdEl's VOTable representation
 		if such a thing is being serialised.
@@ -737,7 +760,7 @@ def _iterGroups(ctx, container, serManager):
 		yield votGroup
 
 
-def makeTable(ctx, table):
+def makeTable(ctx, table, isMeta=False):
 	"""returns a Table node for the table.Table instance table.
 	"""
 	sm = valuemappers.SerManager(table, mfRegistry=ctx.mfRegistry,
@@ -785,19 +808,25 @@ def makeTable(ctx, table):
 			_linkBuilder.build(table.tableDef),
 			]
 
+		if isMeta:
+			# we take the "should not contain data" from the spec as "will not".
+			# That is, we will simply drop any data on resources declared as meta
+			return result
 
-		return votable.DelayedTable(result,
-			sm.getMappedTuples(),
-			tableEncoders[ctx.tablecoding],
-			overflowElement=ctx.overflowElement)
+		else:
+			return votable.DelayedTable(result,
+				sm.getMappedTuples(),
+				tableEncoders[ctx.tablecoding],
+				overflowElement=ctx.overflowElement)
 
 
 def _makeResource(ctx, data):
 	"""returns a Resource node for the rsc.Data instance data.
 	"""
+	resType = base.getMetaText(data, "_type")
 	res = V.RESOURCE()
 	with ctx.activeContainer(res):
-		res(type=base.getMetaText(data, "_type"),
+		res(type=resType,
 				utype=base.getMetaText(data, "utype"))[
 			_iterResourceMeta(ctx, data), [
 				_makeVOTParam(ctx, param) for param in data.iterParams()],
@@ -805,7 +834,7 @@ def _makeResource(ctx, data):
 			]
 		for table in data:
 			with ctx.buildingFromTable(table):
-				res[makeTable(ctx, table)]
+				res[makeTable(ctx, table, isMeta=resType=="meta")]
 		res[ctx.overflowElement]
 	return res
 
@@ -911,17 +940,22 @@ def format(data, outputFile, **ctxargs):
 	return writeAsVOTable(data, outputFile, VOTableContext(**ctxargs))
 
 common.registerDataWriter("votable", format, 
-	base.votableType, "Default VOTable", ".vot")
+	base.votableType, "Default VOTable", ".vot",
+	tapId="ivo://ivoa.net/std/TAPRegExt#output-votable-binary")
 common.registerDataWriter("votableb2", functools.partial(
 	format, tablecoding="binary2"),
 	"application/x-votable+xml;serialization=BINARY2",
 	"Binary2 VOTable",
-	".votb2")
+	".votb2",
+	"votable/b2",
+	tapId="ivo://ivoa.net/std/TAPRegExt#output-votable-binary2")
 common.registerDataWriter("votabletd", functools.partial(
 	format, tablecoding="td"), 
 	"application/x-votable+xml;serialization=TABLEDATA", "Tabledata VOTable",
 	".vottd",
-	"text/xml")
+	"text/xml",
+	"votable/td",
+	tapId="ivo://ivoa.net/std/TAPRegExt#output-votable-td")
 common.registerDataWriter("votabletd1.1", functools.partial(
 	format, tablecoding="td", version=(1,1)), 
 	"application/x-votable+xml;serialization=TABLEDATA;version=1.1", 
