@@ -2,7 +2,7 @@
 A grammar reading from (fixed-record) binary files.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -11,12 +11,12 @@ A grammar reading from (fixed-record) binary files.
 import re
 import struct
 
-import pyparsing
 
 from gavo import base
 from gavo import utils
 from gavo.grammars.common import Grammar, FileRowIterator
 from gavo.utils import misctricks
+from gavo.utils import parsetricks
 
 
 class BinaryRowIterator(FileRowIterator):
@@ -53,17 +53,17 @@ class BinaryRowIterator(FileRowIterator):
 
 
 def _getFieldsGrammar():
-	with utils.pyparsingWhitechars(" \n\t\r"):
-		identifier = pyparsing.Regex(utils.identifierPattern.pattern[:-1]
+	with parsetricks.pyparsingWhitechars(" \n\t\r"):
+		identifier = parsetricks.Regex(utils.identifierPattern.pattern[:-1]
 			).setName("identifier")
-		formatCode = pyparsing.Regex("\d+s|[bBhHiIqQfd]"
+		formatCode = parsetricks.Regex("\d+s|[bBhHiIqQfd]"
 			).setName("fieldSpec")
 		field = ( identifier("identifier")
-			+ pyparsing.Suppress(pyparsing.Literal("("))
+			+ parsetricks.Suppress(parsetricks.Literal("("))
 			+ formatCode("formatCode")
-			+ pyparsing.Suppress(pyparsing.Literal(")"))).setParseAction(
+			+ parsetricks.Suppress(parsetricks.Literal(")"))).setParseAction(
 				lambda s, p, t: dict(t))
-		return pyparsing.OneOrMore(field)+pyparsing.StringEnd()
+		return parsetricks.OneOrMore(field)+parsetricks.StringEnd()
 
 		
 class BinaryRecordDef(base.Structure):
@@ -109,7 +109,7 @@ class BinaryRecordDef(base.Structure):
 	def completeElement(self, ctx):
 		try:
 			parsedFields = utils.pyparseString(self._fieldsGrammar, self.content_)
-		except pyparsing.ParseBaseException as ex:
+		except parsetricks.ParseBaseException as ex:
 			raise base.ui.logOldExc(base.LiteralParseError("binaryRecordDef", 
 				re.sub("\s+", " ", self.content_),
 				pos=str(ex.loc), hint="The parser said: '%s'"%str(ex)))
