@@ -2,7 +2,7 @@
 A (quite trivial) grammar that iterates over lists of dicts.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -16,6 +16,7 @@ class ListIterator(RowIterator):
 	def __init__(self, *args, **kwargs):
 		RowIterator.__init__(self, *args, **kwargs)
 		self.recNo = 0
+		self.notify = self.grammar.notify
 		if self.grammar.asPars:
 			self.sourceRow = self.sourceToken[0]
 
@@ -43,3 +44,9 @@ class DictlistGrammar(Grammar):
 
 	_asPars = base.BooleanAttribute("asPars", default=False, description=
 		"Just return the first item of the list as parameters row and exit?")
+	_notify = base.BooleanAttribute("notify", default=False,
+		description="Enable notification of begin/end of processing (as"
+			" for other grammars; dictlistGrammars are usually internal,"
+			" where notification is unwelcome).  Note that the -M option"
+			" of gavo imp will only work if you set it to true.", copyable=True)
+
