@@ -9,7 +9,7 @@ the rest of the DC software.
 Currently, only one kind of DirectGrammar is supported: C boosters.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -64,19 +64,27 @@ class CBooster(object):
 		getResource("resources/src/boosterskel.h", "boosterskel.h")
 		shutil.copyfile(self.srcName, os.path.join(wd, "func.c"))
 
-		# XXX TODO: take this from the embedding data's make;
-		# DirectGrammars can't be outside of a data element any more anyway.
-		mat = re.search("(?m)^#define QUERY_N_PARS\s+(\d+)", 
-			open(self.srcName, "r", encoding="utf-8").read())
-		if not mat:
-			raise base.ReportableError("Booster function doesn't define QUERY_N_PARS")
-		query_n_pars = mat.group(1)
+		dd = self.dataDesc
+		if len(dd.makes)!=1:
+			raise base.ReportableError(
+				"Direct grammars can only work on single-make DDs")
+		query_n_pars = len(dd.makes[0].table.columns)
 
 		f = open(os.path.join(wd, "Makefile"), "w")
 
-		if self.dataDesc.grammar.type=="fits":
+		if dd.grammar.type=="fits":
 			f.write("LDFLAGS += -lcfitsio\n")
 
+		if dd.grammar.type=="hdf5vaex":
+			flags = subprocess.check_output(
+				["pkg-config", "--cflags", "hdf5"]).strip().decode("ascii")
+			if flags:
+				f.write(f"CFLAGS += {flags}\n")
+			flags = subprocess.check_output(
+				["pkg-config", "--libs", "hdf5"]).strip().decode("ascii")
+			if flags:
+				f.write(f"LDFLAGS += {flags}\n")
+
 		f.write("LDFLAGS += -lm\n"
 			"CFLAGS += -Wall -DINPUT_LINE_MAX=%d -DQUERY_N_PARS=%s\n"%(
 				self.recordSize, query_n_pars))
@@ -147,15 +155,24 @@ class CBooster(object):
 class DirectGrammar(base.Structure, base.RestrictionMixin):
 	"""A user-defined external grammar.
 
-	See the `separate document`_ on user-defined code on more on direct grammars.
+	See the :dachsdoc:`booster.html` on user-defined code on more on direct
+	grammars.
 
 	You will almost always use these in connection with C code generated
 	by ``dachs mkboost``.
 
-	.. _separate document: :dachsdoc:booster.html
+	Properties:
+
+	* dataset: For HDF5 boosters, the HDF path to the table to be
+	  imported (in vaex, not including "columns").  Certain, hopefully
+	  sensible, defaults apply depending on the HDF5 dialect.
+	* chunkSize: In HDF5 boosters, now many rows to pull into
+	  memory at one time (defaults to 5000).
 	"""
 	name_ = "directGrammar"
 
+	notify = True
+
 	_cbooster = rscdef.ResdirRelativeAttribute("cBooster", 
 		default=base.Undefined,
 		description="resdir-relative path to the booster C source.",
@@ -191,7 +208,7 @@ class DirectGrammar(base.Structure, base.RestrictionMixin):
 		copyable=True)
 
 	_type = base.EnumeratedUnicodeAttribute("type", default="col", 
-		validValues=["col", "bin", "fits", "split"],
+		validValues=["col", "bin", "fits", "split", "hdf5vaex"],
 		description="Make code for a booster parsing by column indices (col),"
 			" by splitting along separators (split), by reading fixed-length"
 			" binary records (bin), for from FITS binary tables (fits).",
@@ -207,10 +224,13 @@ class DirectGrammar(base.Structure, base.RestrictionMixin):
 
 	_mapKeys = base.StructAttribute("mapKeys", childFactory=common.MapKeys,
 		default=None, copyable=True, 
-		description="For a FITS booster, map DB table column names"
-			" to FITS column names (e.g., if the FITS table name flx is to"
-			" end up in the DB column flux, say flux:flx).")
-	
+		description="For certain input formats (FITS, HDF5), map input"
+			" column names to database column name.  For instance, to generate"
+			" code for a FITS column flx to end up in the DB column flux,"
+			" say flux:flx).")
+
+	_properties = base.PropertyAttribute()
+
 	_rd = rscdef.RDAttribute()
 
 	isDispatching = False
@@ -222,9 +242,9 @@ class DirectGrammar(base.Structure, base.RestrictionMixin):
 				raise base.StructureError("DirectGrammars reading from binary need"
 					" a recordSize attribute")
 		if self.mapKeys is not None:
-			if self.type!="fits":
-				raise base.StructureError("mapKeys is only allowed for FITS"
-					" boosters.")
+			if self.type not in {"fits", "hdf5vaex"}:
+				raise base.StructureError("mapKeys is not allowed for this"
+					" kind of booster.")
 
 	def onElementComplete(self):
 		if self.type=="fits":
@@ -250,11 +270,19 @@ class DirectGrammar(base.Structure, base.RestrictionMixin):
 					self.parent.id, len(makes)))
 
 		def copyIn(data):
-			list(data.tables.values())[0].copyIn(booster.getOutput(sourceToken))
-			if booster.getStatus():
-				raise base.SourceParseError(
-					"Booster returned error signature",
-					source=sourceToken)
+			if self.notify:
+				base.ui.notifyNewSource(sourceToken)
+			try:
+				source = booster.getOutput(sourceToken)
+				list(data.tables.values())[0].copyIn(source)
+				source.close()
+				if booster.getStatus():
+					raise base.SourceParseError(
+						"Booster returned error signature",
+						source=sourceToken)
+			finally:
+				base.ui.notifySourceFinished()
+
 		return copyIn
 
 
@@ -343,6 +371,7 @@ def _getMakeMacro(item):
 			"char": "MAKE_CHAR_NULL",
 			"bytea": "MAKE_BYTE",
 			"text": "MAKE_TEXT",
+			"bigint": "MAKE_BIGINT",
 		}[item.type]
 	except KeyError:
 		# not a simple case; this could be a place for doing arrays and such
@@ -352,7 +381,8 @@ def _getMakeMacro(item):
 class _CodeGenerator(object):
 	"""a base class for code generators.
 
-	You must at least override getItemParser.
+	You will at least have to override getItemParser and getFooter
+	if this is to do anything useful.
 	"""
 	def __init__(self, grammar, tableDef):
 		self.grammar, self.tableDef = grammar, tableDef
@@ -540,8 +570,9 @@ class BinCodeGenerator(_CodeGenerator):
 			+COMMON_MAIN_FOOT)
 
 
-class FITSCodeGenerator(_CodeGenerator):
-	"""A code generator for reading from FITS binary tables.
+class _NumpyMetaCodeGenerator(_CodeGenerator):
+	"""A base class for code generators built using numpy-compatible
+	metadata.
 	"""
 	fitsTypes = {
 		"B": ("TBYTE", "char"),
@@ -552,6 +583,26 @@ class FITSCodeGenerator(_CodeGenerator):
 		"E": ("TFLOAT", "float"),
 		"L": ("TLOGICAL", "unsigned char"),
 		"D": ("TDOUBLE", "double")}
+	numpyTypes = {
+		"byte": "char",
+		"int8": "char",
+		"short": "short",
+		"int16": "short",
+		"cint": "int",
+		"int32": "int",
+		"int64": "int64_t",
+		"ubyte": "unsigned char",
+		"uint8": "unsigned char",
+		"ushort": "unsigned short",
+		"uint16": "unsigned char",
+		"uintc": "unsigned int",
+		"ulonglong": "unsigned long long",
+		"uint64": "uint64_t",
+		"single": "float",
+		"float32": "float",
+		"double": "double",
+		"float64": "double",
+	}
 	makers = {
 		"bigint": "MAKE_BIGINT",
 		"smallint": "MAKE_SHORT",
@@ -562,22 +613,42 @@ class FITSCodeGenerator(_CodeGenerator):
 		"double precision": "MAKE_DOUBLE",
 	}
 	
+	def _parseFITSFormat(self, format, colName):
+		"""returns length and typecode for the supported FITS table types.
+
+		All others raise errors.
+		"""
+		mat = re.match("(\d*)(.)$", format)
+		if not mat:
+			raise base.ReportableError("FITS type code '%s' of %s not handled"
+				" by gavo mkboost; add handling if you can."%(format, colName))
+		if not mat.group(2) in self.fitsTypes:
+			raise base.ReportableError("FITS type '%s' of %s not handled"
+				" by gavo mkboost; add handling if you can."%(
+					mat.group(2), colName))
+		return int(mat.group(1) or "1"), mat.group(2)
+
+
+class FITSCodeGenerator(_NumpyMetaCodeGenerator):
+	"""A code generator for reading from FITS binary tables.
+	"""
 	def __init__(self, grammar, tableDef):
 		from gavo.utils import pyfits
 		_CodeGenerator.__init__(self, grammar, tableDef)
-		# now fetch the first source to figure out its schema
+		self.forExtension = grammar.extension
+
+		# now pull the table schema from the first source.
 		if self.grammar.parent.sources is None:
 			raise base.StructureError("Cannot make FITS bintable booster without"
 				" a sources element on the embedding data.")
-
-		self.forExtension = grammar.extension
-
 		try:
-			self.fitsTable = pyfits.open(
-				next(self.grammar.parent.sources.iterSources()))[self.forExtension]
+			hdus = pyfits.open(
+				next(self.grammar.parent.sources.iterSources()))
+			self.fitsTable = hdus[self.forExtension]
 		except StopIteration:
-			raise base.StructureError("Buliding a FITS bintable booster requires"
+			raise base.StructureError("Building a FITS bintable booster requires"
 				" at least one matching source.")
+		hdus.close()
 		
 		self._computeMatches()
 	
@@ -780,6 +851,23 @@ FITSColDesc COL_DESCS[%(nCols)d] = %(colDescs)s;
 		+COMMON_MAIN_FOOT)
 
 
+def HDF5vaexCodeGenerator(*args, **kwargs):
+	"""a factory function for an HDF5 code generator.
+
+	This is built like this because directhdf5 imports this module and 
+	we need to avoid circular imports.  If this kind of construct becomes
+	more common, we should pull basic code generation stuff into a
+	boostercommon module.  Meanwhile, this is a convenient place to
+	produce a useful error message for when h5py is not available.
+	"""
+	try:
+		from gavo.grammars import directhdf5
+	except ImportError:
+		raise base.ReportableError("HDF5 boosters can only be built if h5py"
+			" (Debian: python3-h5py) is installed.")
+	return directhdf5.HDF5vaexCodeGenerator(*args, **kwargs)
+	
+
 def getCodeGen(grammar, tableDef):
 	"""returns the code generator suitable for making code for grammar.
 	"""
@@ -788,6 +876,7 @@ def getCodeGen(grammar, tableDef):
 		"split": SplitCodeGenerator,
 		"fits": FITSCodeGenerator,
 		"col": ColCodeGenerator,
+		"hdf5vaex": HDF5vaexCodeGenerator,
 	}[grammar.type](grammar, tableDef)
 
 
