@@ -2,7 +2,7 @@
 A grammar taking rows from a FITS table.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -13,6 +13,7 @@ from gavo.grammars import common
 
 from gavo.utils import pyfits
 
+
 class FITSTableIterator(common.RowIterator):
 	"""The row iterator for FITSTableGrammars.
 	"""
@@ -20,7 +21,14 @@ class FITSTableIterator(common.RowIterator):
 		hdus = pyfits.open(self.sourceToken)
 		fitsTable = hdus[self.grammar.hdu].data
 		names = [n for n in fitsTable.dtype.names]
+
+		if self.grammar.lowerKeys:
+			names = [n.lower() for n in names]
+
 		for row in fitsTable:
+			if self.grammar.nanIsNULL:
+				row = [v if v==v else None for v in row]
+
 			res = dict(list(zip(names, row)))
 			yield res
 
@@ -39,6 +47,11 @@ class FITSTableGrammar(common.Grammar):
 
 	_hduIndex = base.IntAttribute("hdu", default=1,
 		description="Take the data from this extension (primary=0)."
-			" Tabular data typically resides in the first extension.")
+		" Tabular data typically resides in the first extension.")
+	_lowerKeys = base.BooleanAttribute("lowerKeys", default=False,
+		description="Convert column names to lower case when building"
+		" the rawdict.")
+	_mapNaN = base.BooleanAttribute("nanIsNULL", default=False,
+		description="Map NaN floats to NULL when building the rawdict.")
 
 	rowIterator = FITSTableIterator
