@@ -6,7 +6,7 @@ testhelpers, too.  Unit test modules should not be forced to import
 this.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -167,35 +167,38 @@ XS = QNamer("http://www.w3.org/2001/XMLSchema")
 
 
 VO_SCHEMATA = [
-		"Characterisation-v1.11.xsd",
-		"ConeSearch-v1.1.xsd",
-		"DataModel-v1.0.xsd",
-		"DocRegExt-v1.0.xsd",
+		"b2find.xsd",
+		"Characterisation.xsd",
+		"Colstats.xsd",    # remove once it's in VODataService
+		"ConeSearch.xsd",
+		"DaCHS.xsd",
+		"DataModel.xsd",
+		"DocRegExt.xsd",
 		"oai_dc.xsd",
 		"OAI-PMH.xsd",
-		"RegistryInterface-v1.0.xsd",
-		"SIA-v1.2.xsd",
-		"SLAP-v1.1.xsd",
-		"SSA-v1.3.xsd",
-		"StandardsRegExt-1.0.xsd",
-		"stc-v1.30.xsd",
+		"RegistryInterface.xsd",
+		"SIA.xsd",
+		"SLAP.xsd",
+		"SSA.xsd",
+		"StandardsRegExt.xsd",
+		"stc.xsd",
 		"stc-v1.20.xsd",
 		"coords-v1.20.xsd",
 		"region-v1.20.xsd",
-		"TAPRegExt-v1.0.xsd",
-		"UWS-v1.1.xsd",
-		"VODataService-v1.2.xsd",
-		"VOEvent-1.0.xsd",
-		"VOEventRegExt-v2.0.xsd",
-		"VORegistry-1.1.xsd",
-		"VOResource-v1.1.xsd",
-		"VOSIAvailability-v1.0.xsd",
-		"VOSICapabilities-v1.0.xsd",
-		"VOSITables-v1.0.xsd",
+		"TAPRegExt.xsd",
+		"UWS.xsd",
+		"VODataService.xsd",
+		"VOEvent.xsd",
+		"VOEventRegExt.xsd",
+		"VORegistry.xsd",
+		"VOResource.xsd",
+		"VOSIAvailability.xsd",
+		"VOSICapabilities.xsd",
+		"VOSITables.xsd",
 		"VOTable-1.1.xsd",
 		"VOTable-1.2.xsd",
-		"VOTable-1.4.xsd",
-		"vo-dml-v1.0.xsd",
+		"VOTable.xsd",
+		"vo-dml.xsd",
 		"xlink.xsd",
 		"XMLSchema.xsd",
 		"xml.xsd",]
