@@ -4,7 +4,7 @@ The datalink core and its numerous helper classes.
 More on this in "Datalink Cores" in the reference documentation.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -24,6 +24,7 @@ from gavo import svcs
 from gavo import utils
 from gavo.formal import nevowc
 from gavo.formats import votablewrite
+from gavo.protocols import dlasync
 from gavo.protocols import products
 from gavo.protocols import soda
 from gavo.protocols.soda import (FormatNow, DeliverNow, DatalinkFault,
@@ -38,69 +39,13 @@ from twisted.web import static
 MS = base.makeStruct
 
 
-class ProductDescriptor(object):
-	"""An encapsulation of information about some "product" (i.e., file).
-
-	This is basically equivalent to a line in the product table; the
-	arguments of the constructor are all available as same-named attributes.
-
-	It also has an attribute data defaulting to None.  DataGenerators
-	set it, DataFilters potentially change it.
-
-	If you inherit from this method and you have a way to guess the
-	size of what the descriptor describes, override the estimateSize()
-	method.  The default will return a file size if accessPath points
-	to an existing file, None otherwise.
+class DatalinkDescriptorMixin:
+	"""some generally useful methods that various code
+	expectes datalink descriptors to have.
 	"""
 	data = None
 	forSemantics = DEFAULT_SEMANTICS
 
-	def __init__(self, pubDID, accref, accessPath, mime, 
-			owner=None, embargo=None, sourceTable=None, datalink=None,
-			preview=None, preview_mime=None):
-		self.pubDID = pubDID
-		self.accref, self.accessPath, self.mime = accref, accessPath, mime
-		self.owner, self.embargo, self.sourceTable = owner, embargo, sourceTable
-		self.preview, self.previewMime = preview, preview_mime
-
-	@classmethod
-	def fromAccref(cls, pubDID, accref, accrefPrefix=None, **moreKWs):
-		"""returns a product descriptor for an access reference.
-
-		If an accrefPrefix is passed in, an AuthenticationFault (for want
-		of something better fitting) is returned when the accref doesn't
-		start with accrefPrefix.
-		"""
-		if accrefPrefix and not accref.startswith(accrefPrefix):
-			return DatalinkFault.AuthenticationFault(pubDID,
-				"This Datalink service not available"
-				" with this pubDID", semantics="#this")
-
-		kwargs = products.RAccref(accref).productsRow
-		kwargs.update(moreKWs)
-		return cls(pubDID, **kwargs)
-
-	@contextlib.contextmanager
-	def currentSemantics(self, sem):
-		"""sets the default semantics for links and faults generated in
-		the controlled block to sem.
-		"""
-		oldSemantics = self.forSemantics
-		self.forSemantics = sem
-		try:
-			yield
-		finally:
-			self.forSemantics = oldSemantics
-
-	def estimateSize(self):
-		if isinstance(self.accessPath, str):
-			candPath = os.path.join(base.getConfig("inputsDir"), self.accessPath)
-			try:
-				return os.path.getsize(candPath)
-			except:
-				# fall through to returning None
-				pass
-
 	def makeLink(self, url, **kwargs):
 		"""returns a LinkDef for this descriptor for url.
 
@@ -133,6 +78,68 @@ class ProductDescriptor(object):
 		return LinkDef.fromFile(localPath, description, semantics,
 			service=service, contentType=None, suppressMissing=suppressMissing)
 
+	@contextlib.contextmanager
+	def currentSemantics(self, sem):
+		"""sets the default semantics for links and faults generated in
+		the controlled block to sem.
+		"""
+		oldSemantics = self.forSemantics
+		self.forSemantics = sem
+		try:
+			yield
+		finally:
+			self.forSemantics = oldSemantics
+
+
+class ProductDescriptor(DatalinkDescriptorMixin):
+	"""An encapsulation of information about some "product" (i.e., file).
+
+	This is basically equivalent to a line in the product table; the
+	arguments of the constructor are all available as same-named attributes.
+
+	It also has an attribute data defaulting to None.  DataGenerators
+	set it, DataFilters potentially change it.
+
+	If you inherit from this method and you have a way to guess the
+	size of what the descriptor describes, override the estimateSize()
+	method.  The default will return a file size if accessPath points
+	to an existing file, None otherwise.
+	"""
+
+	def __init__(self, pubDID, accref, accessPath, mime, 
+			owner=None, embargo=None, sourceTable=None, datalink=None,
+			preview=None, preview_mime=None):
+		self.pubDID = pubDID
+		self.accref, self.accessPath, self.mime = accref, accessPath, mime
+		self.owner, self.embargo, self.sourceTable = owner, embargo, sourceTable
+		self.preview, self.previewMime = preview, preview_mime
+
+	@classmethod
+	def fromAccref(cls, pubDID, accref, accrefPrefix=None, **moreKWs):
+		"""returns a product descriptor for an access reference.
+
+		If an accrefPrefix is passed in, an AuthenticationFault (for want
+		of something better fitting) is returned when the accref doesn't
+		start with accrefPrefix.
+		"""
+		if accrefPrefix and not accref.startswith(accrefPrefix):
+			return DatalinkFault.AuthenticationFault(pubDID,
+				"This Datalink service not available"
+				" with this pubDID", semantics="#this")
+
+		kwargs = products.RAccref(accref).productsRow
+		kwargs.update(moreKWs)
+		return cls(pubDID, **kwargs)
+
+	def estimateSize(self):
+		if isinstance(self.accessPath, str):
+			candPath = os.path.join(base.getConfig("inputsDir"), self.accessPath)
+			try:
+				return os.path.getsize(candPath)
+			except:
+				# fall through to returning None
+				pass
+
 
 class FITSProductDescriptor(ProductDescriptor):
 	"""A SODA descriptor for FITS files.
@@ -259,16 +266,16 @@ class DescriptorGenerator(rscdef.ProcApp):
 
 	The following names are available to the code:
 
-	  - pubDID -- the pubDID to be resolved
-	  - args -- all the arguments that came in from the web
-	    (these should not ususally be necessary for making the descriptor
-	    and are completely unparsed at this point)
-	  - FITSProductDescriptor -- the base class of FITS product descriptors
-	  - DLFITSProductDescriptor -- the same, just for when the product table
-	    has a datalink.
-	  - ProductDescriptor -- a base class for your own custom descriptors
-	  - DatalinkFault -- use this when flagging failures
-	  - soda -- contents of the soda module for convenience
+		- pubDID -- the pubDID to be resolved
+		- args -- all the arguments that came in from the web
+			(these should not ususally be necessary for making the descriptor
+			and are completely unparsed at this point)
+		- FITSProductDescriptor -- the base class of FITS product descriptors
+		- DLFITSProductDescriptor -- the same, just for when the product table
+			has a datalink.
+		- ProductDescriptor -- a base class for your own custom descriptors
+		- DatalinkFault -- use this when flagging failures
+		- soda -- contents of the soda module for convenience
 	
 	If you made your pubDID using the ``getStandardPubDID`` rowmaker function,
 	and you need no additional logic within the descriptor,
@@ -452,8 +459,14 @@ class _ServiceDescriptor(object):
 		for ik in self.inputKeys:
 			param = ctx.addID(ik,
 				votablewrite.makeFieldFromColumn(ctx, V.PARAM, ik))
+
 			if linkIdTo and ik.name=="ID":
 				param = param(ref=linkIdTo)
+		
+			if ik.hasProperty("defaultForForm"):
+				param[
+					V.LINK(action="rdf", content_role="#pre-set", 
+						value=str(ik.getProperty("defaultForForm")))]
 			inputParams[param]
 
 		return res
@@ -475,15 +488,15 @@ class MetaMaker(rscdef.ProcApp):
 	LinkDef without qualification, and there's the MS function to build
 	structures.  Hence, a metaMaker returning an InputKey could look like this::
 
-	  <metaMaker>
-	    <code>
-	      yield MS(InputKey, name="format", type="text",
-	        description="Output format desired",
-	          values=MS(Values,
-	          options=[MS(Option, content_=descriptor.mime),
-	                MS(Option, content_="text/plain")]))
-	    </code>
-	  </metaMaker>
+		<metaMaker>
+			<code>
+				yield MS(InputKey, name="format", type="text",
+					description="Output format desired",
+						values=MS(Values,
+						options=[MS(Option, content_=descriptor.mime),
+									MS(Option, content_="text/plain")]))
+			</code>
+		</metaMaker>
 
 	(of course, you should give more metadata -- ucds, better description,
 	etc) in production).
@@ -500,15 +513,15 @@ class MetaMaker(rscdef.ProcApp):
 	and you will usually create links from the descriptor so the pubDID will
 	be right automatically::
 
-	  <metaMaker semantics="#flat">
-	      <code>
-	         yield descriptor.makeLink(
-	           "http://example.org/flats/master-flat.fits",
-	           contentType="image/fits",
-	           description="The master flat for this epoch",
-	           contentLength=2048x2048*2)
-	      </code>
-	   </metaMaker>
+		<metaMaker semantics="#flat">
+				<code>
+					 yield descriptor.makeLink(
+						 "http://example.org/flats/master-flat.fits",
+						 contentType="image/fits",
+						 description="The master flat for this epoch",
+						 contentLength=2048x2048*2)
+				</code>
+		 </metaMaker>
 
 	It's ok to yield None; this will suppress a Datalink and is convenient
 	when some component further down figures out that a link doesn't exist
@@ -571,20 +584,21 @@ class DataFunction(rscdef.ProcApp):
 	data attribute; this can be used to return arbitrary chunks of data).
 
 	The following names are available to the code:
-	  - descriptor -- whatever the DescriptorGenerator returned
-	  - args -- all the arguments that came in from the web.
+		- descriptor -- whatever the DescriptorGenerator returned
+		- args -- all the arguments that came in from the web.
 	
 	In addition to the usual names available to ProcApps, data functions have:
-	  - FormatNow -- exception to raise to go directly to the formatter
-	  - DeliverNow -- exception to raise to skip all further formatting
-	    and just deliver what's currently in descriptor.data
-	  - File(path, type) -- if you just want to return a file on disk, pass 
-	  	its path and media type to File and assign the result to 
-	  	descriptor.data.  
-	  - TemporaryFile(path,type) -- as File, but the disk file is 
-	    unlinked after use
-	  - makeData -- the rsc.makeData function
-	  - soda -- the protocols.soda module
+		- FormatNow -- exception to raise to go directly to the formatter
+		- DeliverNow -- exception to raise to skip all further formatting
+			and just deliver what's currently in descriptor.data
+		- File(path, type) -- if you just want to return a file on disk, pass 
+			its path and media type to File and assign the result to 
+			descriptor.data.  
+		- TemporaryFile(path,type) -- as File, but the disk file is 
+			unlinked after use
+		- makeData -- the rsc.makeData function
+		- soda -- the protocols.soda module
+		- RemoteProduct(url) -- 301-redirects to URL when rendered
 	"""
 	name_ = "dataFunction"
 	requiredType = "dataFunction"
@@ -597,6 +611,7 @@ class DataFunction(rscdef.ProcApp):
 		"TemporaryFile": _TemporaryFile, 
 		"makeData": rsc.makeData, 
 		"soda": soda,
+		"DataFromURL": products.RemoteProduct.fromURL,
 	}
 
 
@@ -615,17 +630,17 @@ class DataFormatter(rscdef.ProcApp):
 	periods of time.
 
 	The following names are available to the code:
-	  - descriptor -- whatever the DescriptorGenerator returned
-	  - args -- all the arguments that came in from the web.
+		- descriptor -- whatever the DescriptorGenerator returned
+		- args -- all the arguments that came in from the web.
 	
 	In addition to the usual names available to ProcApps, data formatters have:
-	  - Page -- base class for resources with renderHTTP methods.
-	  - IRequest -- the nevow interface to make Request objects with.
-	  - File(path, type) -- if you just want to return a file on disk, pass 
-	    its path and media type to File and return the result. 
-	  - TemporaryFile(path, type) -- as File, but the disk file is unlinked 
-	    after use
-	  - soda -- the protocols.soda module
+		- Page -- base class for resources with renderHTTP methods.
+		- IRequest -- the nevow interface to make Request objects with.
+		- File(path, type) -- if you just want to return a file on disk, pass 
+			its path and media type to File and return the result. 
+		- TemporaryFile(path, type) -- as File, but the disk file is unlinked 
+			after use
+		- soda -- the protocols.soda module
 	"""
 	name_ = "dataFormatter"
 	requiredType = "dataFormatter"
@@ -855,6 +870,8 @@ class DatalinkCore(DatalinkCoreBase):
 
 	overflowed = False
 
+	workerSystem = dlasync.DL_WORKER
+
 	# the core will be specially and non-cacheably adapted for these
 	# renderers (ssap.xml is in here for legacy getData):
 	datalinkAdaptingRenderers = frozenset([
@@ -897,9 +914,12 @@ class DatalinkCore(DatalinkCoreBase):
 				errors.extend(lerrs)
 	
 				# ssap expects the first renderer here to be dlget, so don't
-				# remove it or move it back.
+				# remove it or move it back; also: only declare services
+				# if they accept more than just ID; with just ID, it's obviously
+				# just a bytestream generator for #this rather than a proper
+				# service.
 				for rendName in ["dlget", "dlasync"]:
-					if rendName in allowedForSvc:
+					if len(inputKeys)>1 and rendName in allowedForSvc:
 						endpoints.append(
 							_ServiceDescriptor(
 								descriptor.pubDID, 
@@ -1073,7 +1093,7 @@ class DatalinkCore(DatalinkCoreBase):
 			else:
 				destMime = self.datalinkType
 
-			destMime = str(inputTable.getParam("RESPONSEFORMAT") or destMime)
+			destMime = str(inputTable.getParam("responseformat") or destMime)
 			if destMime=="votable":
 				destMime = self.datalinkType
 				
