@@ -5,7 +5,7 @@ This includes both some high-level functions and rudimentary parsers
 that can serve as bases for more specialized parsers.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -16,12 +16,11 @@ import hashlib
 import os
 import pickle
 import re
-import urllib.request, urllib.parse, urllib.error
+import urllib.parse
 from xml import sax
 from xml.sax import saxutils
 
 from gavo import base
-from gavo import svcs
 from gavo import utils
 
 
@@ -92,7 +91,7 @@ class CanonicalPrefixes(object):
 		try:
 			return self._reverseRegistry[ns]
 		except KeyError:
-			raise svcs.NotFoundError(ns, "XML namespace",
+			raise base.NotFoundError(ns, "XML namespace",
 				"registry of XML namespaces.")
 
 	def haveNS(self, ns):
@@ -266,15 +265,16 @@ class OAIRecordsParser(sax.ContentHandler, OAIErrorMixin):
 	def startPrefixMapping(self, prefix, uri):
 		self.prefixMap.setdefault(prefix, []).append(uri)
 
-		# Here, we make sure we find a globally unique prefix for every
-		# namespace URI.  canonicalPrefixes makes sure this unique prefix
-		# is persistent and later available to the OAI interface
-		if not self.canonicalPrefixes.haveNS(uri):
-			self.canonicalPrefixes.registerPrefixOrMakeUp(prefix, uri)
-		
-		canonPrefix = self.canonicalPrefixes.getPrefixForNS(uri)
-		if prefix!=canonPrefix or prefix in self.prefixesToTranslate:
-			self.prefixesToTranslate.setdefault(prefix, []).append(canonPrefix)
+		if prefix:
+			# Here, we make sure we find a globally unique prefix for every
+			# namespace URI.  canonicalPrefixes makes sure this unique prefix
+			# is persistent and later available to the OAI interface
+			if not self.canonicalPrefixes.haveNS(uri):
+				self.canonicalPrefixes.registerPrefixOrMakeUp(prefix, uri)
+			
+			canonPrefix = self.canonicalPrefixes.getPrefixForNS(uri)
+			if prefix!=canonPrefix or prefix in self.prefixesToTranslate:
+				self.prefixesToTranslate.setdefault(prefix, []).append(canonPrefix)
 	
 	def endPrefixMapping(self, prefix):
 		self.prefixMap[prefix].pop()
@@ -675,7 +675,7 @@ def _addCanonicalNSDecls(xmlLiteral):
 	"""
 	prefixesUsed = set()
 	for elementContent in re.finditer("<[^>]+>", xmlLiteral):
-		prefixesUsed |= set(re.findall("([a-zA-Z_]\w*):[a-zA-Z_]", 
+		prefixesUsed |= set(re.findall("([a-zA-Z_-][\w-]*):[a-zA-Z_]", 
 			elementContent.group()))
 
 	cp = getCanonicalPrefixes()
