@@ -23,7 +23,7 @@ it if we don't, and for now it doesn't seem we'll support a different framework
 in the forseeable future.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -448,6 +448,12 @@ class RemoteProduct(ProductBase):
 		if REMOTE_URL_PATTERN.match(rAccref.productsRow["accessPath"]):
 			return cls(rAccref)
 
+	@classmethod
+	def fromURL(cls, url):
+		class rAccref:
+			productsRow = {"accessPath": url}
+		return cls(rAccref)
+
 	def iterData(self):
 		f = urllib.request.urlopen(self.pr["accessPath"])
 		while True:
@@ -458,7 +464,7 @@ class RemoteProduct(ProductBase):
 
 	def render(self, request):
 		raise svcs.WebRedirect(self.pr["accessPath"])
-
+	
 
 class RemotePreview(RemoteProduct):
 	"""A preview that's on a remote server.
