@@ -2,7 +2,7 @@
 IVOA cone search: Helper functions, a core, and misc.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -185,8 +185,8 @@ class SCSCore(svcs.DBCore):
 			return pars["RA"], pars["DEC"]
 		elif pars.get("hscs_pos") is not None:
 			try:
-				return parseHumanSpoint(pars["hscs_pos"])
-			except ValueError:
+				return parseHumanSpoint(pars["hscs_pos"], "hscs_pos")
+			except (ValueError, base.ValidationError):
 				# We do not want to fail for this fairly unimportant thing.  
 				# If the core actually needs the position, it should fail itself.
 				return None
