@@ -2,7 +2,7 @@
 TAP: schema maintenance, job/parameter definition incl. upload and UWS actions.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -12,88 +12,26 @@ import datetime
 import functools
 import os
 
-from pyparsing import ParseException
 from twisted.internet import threads
 
 from gavo import base
+from gavo import formats
 from gavo import rsc
 from gavo import svcs
 from gavo import utils
+from gavo.protocols import taprunner
 from gavo.protocols import uws
 from gavo.protocols import uwsactions
-from gavo.utils import codetricks
+from gavo.utils.parsetricks import ParseException
 
 
+TAP_VERSION = "1.1"
+
 RD_ID = "__system__/tap"
 
 # used in the computation of quote
 EST_TIME_PER_JOB = datetime.timedelta(minutes=10)
 
-# A mapping of values of TAP's RESPONSEFORMAT parameter to our 
-# formats.format codes, IANA media types and user-readable labels.
-# Used below (1st element of value tuple) and for registry purposes.
-FORMAT_CODES = {
-	base.votableType:
-		("votable", base.votableType, "VOTable, binary", 
-			"ivo://ivoa.net/std/TAPRegExt#output-votable-binary"),
-	"text/xml": 
-		("votable", "text/xml", "VOTable, binary",
-			"ivo://ivoa.net/std/TAPRegExt#output-votable-binary"),
-	"votable": 
-		("votable", base.votableType, "VOTable, binary",
-			"ivo://ivoa.net/std/TAPRegEXT#output-votable-binary"),
-	"application/x-votable+xml;serialization=binary2": 
-		("votableb2", "application/x-votable+xml;serialization=binary2", 
-			"VOTable, new binary", 
-			"ivo://ivoa.net/std/TAPRegExt#output-votable-binary2"),
-	"votable/b2": 
-		("votableb2", "application/x-votable+xml;serialization=binary2", 
-			"VOTable, new binary", 
-			"ivo://ivoa.net/std/TAPRegExt#output-votable-binary2"),
-	"vodml":
-		("vodml", "application/x-votable+xml;version=1.4",
-			"Experimental VOTable 1.4",
-			"http://dc.g-vo.org/output-vodml"),
-	"application/x-votable+xml;serialization=tabledata":
-		("votabletd", "application/x-votable+xml;serialization=tabledata", 
-			"VOTable, tabledata",
-			"ivo://ivoa.net/std/TAPRegEXT#output-votable-td"),
-	"votable/td":
-		("votabletd", "application/x-votable+xml;serialization=tabledata", 
-			"VOTable, tabledata",
-			"ivo://ivoa.net/std/TAPRegEXT#output-votable-td"),
-	"text/csv": 
-		("csv", "text/csv", "CSV without column labels", None),
-	"csv": ("csv_header", "text/csv;header=present", 
-			"CSV with column labels", None),
-	"text/csv;header=present": 
-		("csv_header", "text/csv;header=present",
-			"CSV with column labels", None),
-	"text/tab-separated-values": 
-		("tsv", "text/tab-separated-values", 
-			"Tab separated values", None),
-	"tsv": 
-		("tsv", "text/tab-separated-values", 
-			"Tab separated values", None),
-	"text/plain": 
-		("tsv", "text/plain",
-			"Tab separated values", None),
-	"application/fits": 
-		("fits", "application/fits", "FITS binary table", None),
-	"fits":
-		("fits", "application/fits", "FITS binary table", None),
-	"text/html": 
-		("html", "text/html", "HTML table", None),
-	"html": 
-		("html", "text/html", "HTML table", None),
-	"json": 
-		("json", "application/json", "JSON", None),
-	"application/json": 
-		("json", "application/json", "JSON", None),
-	"geojson":
-		("geojson", "application/geo-json", "GeoJSON", None),
-}
-
 
 def _R(**kws): return kws
 
@@ -131,7 +69,7 @@ class TAPError(uws.UWSError):
 def getSupportedLanguages():
 	"""returns a list of tuples for the supported languages.
 
-	This is tap.SUPPORTED_LANGUAGES in a format suitable for the
+	This is SUPPORTED_LANGUAGES in a format suitable for the
 	TAP capabilities element.
 
 	Each tuple returned is made up of 
@@ -144,20 +82,16 @@ def getSupportedLanguages():
 def getSupportedOutputFormats():
 	"""yields tuples for the supported output formats.
 
-	This is tap.OUTPUT_FORMATS in a format suitable for the
+	This is OUTPUT_FORMATS in a format suitable for the
 	TAP capabilities element.
 
 	Each tuple is made up of (mime, aliases, description, ivoId).
 	"""
-	codes, descrs, ivoIds = {}, {}, {}
-	for code, (_, outputMime, descr, ivoId) in FORMAT_CODES.items():
-		codes.setdefault(outputMime, set()).add(code)
-		descrs[outputMime] = descr
-		ivoIds[outputMime] = ivoId
-	for mime in codes:
-		# mime never is an alias of itself
-		codes[mime].discard(mime)
-		yield mime, codes[mime], descrs[mime], ivoIds[mime]
+	for key in formats.iterFormats():
+		outputMime = formats.getMIMEFor(key)
+		descr = formats.getLabelFor(key)
+		ivoid = formats.getTAPIdFor(key)
+		yield outputMime, formats.getAliasesFor(key), descr, ivoid
 
 
 ######################## maintaining TAP schema
@@ -245,7 +179,7 @@ class TAPTransitions(uws.ProcessBasedUWSTransitions):
 		uws.SimpleUWSTransitions.__init__(self, "TAP")
 
 	def getCommandLine(self, wjob):
-		return "gavo", ["gavo", "--ui", "stingy", "tap", "--", str(wjob.jobId)]
+		return "gavo", ["gavo", "--ui", "stingy", "taprun", "--", str(wjob.jobId)]
 
 	def queueJob(self, newState, wjob, ignored):
 		"""puts a job on the queue.
@@ -273,10 +207,10 @@ class TAPTransitions(uws.ProcessBasedUWSTransitions):
 
 @functools.lru_cache(1)
 def getUploadGrammar():
-	from pyparsing import (Word, ZeroOrMore, Suppress, StringEnd,
-		alphas, alphanums, CharsNotIn)
+	from gavo.utils.parsetricks import (Word, ZeroOrMore, Suppress, StringEnd,
+		alphas, alphanums, CharsNotIn, pyparsingWhitechars)
 	# Should we allow more tableNames?
-	with utils.pyparsingWhitechars(" \t"):
+	with pyparsingWhitechars(" \t"):
 		tableName = Word( alphas+"_", alphanums+"_" )
 		# What should we allow/forbid in terms of URIs?
 		uri = CharsNotIn(" ;,")
@@ -300,103 +234,72 @@ def parseUploadString(uploadString):
 				" i.e., basically only alphanumerics are allowed.")
 
 
-class LangParameter(uws.JobParameter):
-	@classmethod
-	def addPar(cls, name, value, job):
-		uws.JobParameter.updatePar(name, value, job)
-
-
-class MaxrecParameter(uws.JobParameter):
-	name = "MAXREC"
-	_serialize, _deserialize = staticmethod(uws.strOrEmpty), int
-
-
 class LocalFile(object):
 	"""A sentinel class representing a file within a job work directory
 	(as resulting from an upload).
 	"""
+	# TODO: unify with uws.LocalFile
 	def __init__(self, jobId, wd, fileName):
 		self.jobId, self.fileName = jobId, fileName
 		self.fullPath = os.path.join(wd, fileName)
 
 	def __str__(self):
-		# stringify to a URL for easy UPLOAD string generation.
-		# This smells of a bad idea.  If you change it, change UPLOAD.getParam.
-		return self.getURL()
+		# This is mainly for serialisation of the upload parameter in the
+		# UWS job.
+		return f"file://{self.fileName}"
 
 	def getURL(self):
 		"""returns the URL the file is retrievable under for the life time of
 		the job.
 		"""
-		return base.caches.getRD(RD_ID).getById("run").getURL("tap",
-			absolute=True)+"/async/%s/results/%s"%(
+		return base.caches.getRD(RD_ID).getById("run").getURL("async",
+			absolute=True)+"/%s/results/%s"%(
 				self.jobId,
 				self.fileName)
 
 
-class UploadParameter(uws.JobParameter):
-# the way this is specified, inline uploads are quite tricky. 
-# To obtain the data, we must access the request, which we don't have
-# here.  So, I just grab in from upstack (which of course is bound
-# to fail if we're not being called from within a proper web request).
-# It's not pretty, but then this kind of interdependency between
-# HTTP parameters sucks whatever you do.
-#
-# We assume uploads come in the request's special files dictionary.
-# This is created in taprender.TAPRenderer.gatherUploadFiles.
+def mangleUploads(request):
+	"""parses TAP-compliant UPLOAD specifications in request and return
+	something that the core can handle.
 
-	_deserialize, _serialize = utils.identity, utils.identity
+	This is being executed from the sync/async renderers and probably can't be
+	used anywhere else.
 
-	@classmethod
-	def addPar(cls, name, value, job):
-		if not value.strip():
-			return
-		newUploads = []
-		for tableName, upload in parseUploadString(value):
-			if upload.startswith("param:"):
-				newUploads.append(
-					(tableName, cls._saveUpload(job, upload[6:])))
-			else:
-				newUploads.append((tableName, upload))
-		newVal = job.parameters.get(name, [])+newUploads
-		uws.JobParameter.updatePar(name, newVal, job)
+	The underlying trouble is that the core cannot see the request any more,
+	and thus could not resolve param: uploads.
+	"""
+	uploadSpec = request.strargs.pop("upload", [])
+	uploadSpec = ";".join(uploadSpec)
+	if not uploadSpec: 
+		return
+	
+	parsed = []
+	for tableName, upload in parseUploadString(uploadSpec):
+		if upload.startswith("param:"):
+			try:
+				upload = request.fields[upload[6:]]
+				parsed.append(
+					(tableName, (upload.filename, upload.file)))
+			except KeyError:
+				raise base.ui.logOldExc(
+					base.ValidationError(f"No inline upload '{upload[6:]}' found", 
+						"UPLOAD"))
+			except AttributeError:
+				raise base.ui.logOldExc(
+					base.ValidationError(
+						f"Upload parameter references non-file '{upload[6:]}'",
+						"UPLOAD"))
 
-	@classmethod
-	def getPar(cls, name, job):
-		return ";".join("%s,%s"%p for p in job.parameters.get("upload", ""))
+		else:
+			parsed.append((tableName, upload))
 
-	@classmethod
-	def _saveUpload(cls, job, paramName):
-		try:
-			upload = codetricks.stealVar("request").fields[paramName]
-			uploadName = utils.defuseFileName(upload.filename)
-			uploadFile = upload.file
-		except KeyError:
-			raise base.ui.logOldExc(
-				base.ValidationError("No inline upload '%s' found"%paramName, 
-					"UPLOAD"))
-		except AttributeError:
-			raise base.ui.logOldExc(
-				base.ValidationError("Upload parameter references non-file '%s'"
-					%paramName, "UPLOAD"))
-
-		uploadName = utils.defuseFileName(uploadName)
-		with job.openFile(uploadName, "wb") as f:
-			f.write(uploadFile.read())
-		return LocalFile(job.jobId, job.getWD(), uploadName)
+	request.strargs["upload"] = parsed
 
 
 class TAPJob(uws.UWSJobWithWD):
 	_jobsTDId = "//tap#tapjobs"
 	_transitions = TAPTransitions()
 
-	_parameter_maxrec = MaxrecParameter
-	_parameter_lang = LangParameter
-	_parameter_upload = UploadParameter
-	_parameter_request = uws.JobParameter
-	_parameter_query = uws.JobParameter
-	_parameter_responseformat = uws.JobParameter
-
 	@property
 	def quote(self):
 		"""returns an estimation of the job completion.
@@ -413,7 +316,6 @@ class TAPJob(uws.UWSJobWithWD):
 		return datetime.datetime.utcnow()+nBefore*EST_TIME_PER_JOB
 
 
-
 #################### The TAP worker system
 
 class PlanAction(uwsactions.JobAction):
@@ -431,7 +333,7 @@ class PlanAction(uwsactions.JobAction):
 	def getPlan(self, job, request):
 		from gavo.protocols import taprunner
 		qTable = taprunner.getQTableFromJob(job.parameters,
-			job.jobId, "untrustedquery", 1)
+			job, "untrustedquery", 1)
 		request.setHeader("content-type", "text/plain;charset=utf-8")
 		return qTable, qTable.getPlan()
 
@@ -456,11 +358,19 @@ class TAPUWS(uws.UWSWithQueueing):
 		uws.UWSWithQueueing.__init__(self, TAPJob, uwsactions.JobActions(
 			PlanAction))
 
+	@functools.cached_property
+	def parameterGrammar(self):
+		"""A grammar to be used to process parameters in UWS calls.
+		"""
+		return base.makeStruct(
+				svcs.ContextGrammar,
+				inputTD=base.resolveCrossId("//tap#run").core.inputTable)
+
 	@property
 	def baseURL(self):
 		if self._baseURLCache is None:
 			self._baseURLCache = base.caches.getRD(
-				RD_ID).getById("run").getURL("tap")
+				RD_ID).getById("run").getURL("sync")[:-5]
 		return self._baseURLCache
 
 	def getURLForId(self, jobId):
@@ -475,9 +385,90 @@ WORKER_SYSTEM = TAPUWS()
 
 class TAPCore(svcs.Core):
 	"""A core for the TAP renderer.
-
-	Right now, this is a no-op and not used by the renderer.
-
-	This will change as we move to regularise the TAP system.
 	"""
 	name_ = "tapCore"
+	workerSystem = WORKER_SYSTEM
+
+	inputTableXML = f"""
+		<inputTable>
+			<inputKey name="request" type="text" required="True"
+				std="True" multiplicity="force-single"
+				description="Type of operation requested; this can be doQuery
+					or getCapabilities.  Preferably, don't pass it at all.">
+				<values default="doQuery"/>
+			</inputKey>
+			<inputKey name="lang" type="text" required="True"
+				std="True" multiplicity="force-single"
+				description="A name of a language that QUERY should be parsed as.
+					See capabilities for what you can pass in here.">
+			</inputKey>
+			<inputKey name="query" type="text" required="True"
+				std="True" multiplicity="force-single"
+				description="The query to be executed.">
+			</inputKey>
+			<inputKey name="version" type="text"
+				std="True"
+				description="Don't use this parameter.  You'll only break stuff.">
+				<values default="{TAP_VERSION}"/>
+				<preparse>
+					if input!="{TAP_VERSION}":
+						raise ValueError(
+							"Version mismatch; this service only supports"
+							" TAP version {TAP_VERSION}.")
+				</preparse>
+			</inputKey>
+			<inputKey name="format" type="text"
+				description="Deprecated alias of RESPONSEFORMAT"/>
+			<inputKey name="upload" type="raw"
+				description="A TAP-compliant upload; roughly: 
+					(tablename,source-uri), where source-uri can also have 
+					a param: scheme.">
+			</inputKey>
+			<FEED source="//pql#DALIPars">
+				<PRUNE name="VERB"/>
+			</FEED>
+		</inputTable>"""
+	
+	# The output table is ignored.
+
+	def run(self, service, inputTable, queryMeta):
+		jobId = WORKER_SYSTEM.getNewIdFromArgs(
+			{}, inputTable.getParamDict())
+
+		try:
+			taprunner.runSyncTAPJob(jobId, queryMeta)
+
+			job = WORKER_SYSTEM.getJob(jobId)
+			if job.phase==uws.COMPLETED:
+				# This is TAP, so there's exactly one result
+				res = job.getResults()[0]
+				name, type = res["resultName"], res["resultType"]
+				# hold on to the result fd so its inode is not lost when we delete
+				# the job.
+				f = open(os.path.join(job.getWD(), name), "rb")
+				return (f, type)
+			elif job.phase==uws.ERROR:
+				exc = job.error
+				raise base.Error(exc["msg"], hint=exc["hint"])
+			elif job.phase==uws.ABORTED:
+				raise uws.UWSError("Job was manually aborted.  For synchronous"
+					" jobs, this probably means the operators killed it.",
+					jobId)
+			else:
+				raise uws.UWSError("Internal error.  Invalid UWS phase.", jobId)
+		finally:
+			WORKER_SYSTEM.destroy(jobId)
+
+	def getRelevantTables(self):
+		tables = []
+
+		with base.getTableConn() as conn:
+			for tableName, in conn.query("SELECT table_name"
+					" FROM TAP_SCHEMA.tables"):
+				try:
+					tables.append(base.getTableDefForTable(conn, tableName))
+				except:
+					base.ui.notifyError("Failure trying to retrieve table definition"
+						" for table %s.  Please fix the corresponding RD."%tableName)
+
+		return [t for t in tables if t is not None and t.rd is not None]
