@@ -2,7 +2,7 @@
 Support classes for the universal worker service.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -10,6 +10,8 @@ Support classes for the universal worker service.
 
 import contextlib
 import datetime
+import functools
+import io
 import os
 import pickle 
 import shutil
@@ -41,19 +43,6 @@ EST_TIME_PER_JOB = datetime.timedelta(minutes=10)
 _DEFAULT_LOCK_TIMEOUT = 0.1
 
 
-__docformat__ = "restructuredtext en"
-
-
-def strOrEmpty(aVal):
-	"""returns a stringified version of aVal, except an empty string is returned
-	when aVal is None.
-	"""
-	if aVal is None:
-		return ""
-	else:
-		return str(aVal)
-
-
 class UWSError(base.Error):
 	"""UWS-related errors, mainly to communicate with web renderers.
 
@@ -97,6 +86,10 @@ class UWS(object):
 	The UWS then provides methods to access the jobs table,
 	create jobs and and deserialize jobs from the jobs table.
 
+	It also has a context grammar that parses the JCL parameters
+	from request.strargs.  In this base class, that parses no
+	parameters at all, and you will almost certainly have to override this.
+
 	We have a "statements cache" in here, where we used the UWS table
 	defintion to create query strings we can later pass to the database.
 	Don't worry about this any more.  Just write text queries when adding
@@ -119,6 +112,12 @@ class UWS(object):
 	joblistPreamble = ""
 	# raw XML to prepend to job documents
 	jobdocPreamble = ""
+	# the grammar to parse incoming parameters.  This must be idempotent,
+	# because before starting the core, the parameters already parsed
+	# will be filtered through the grammar.  That's necessary to catch
+	# incomplete parameter sets (which we can't do during individual
+	# posts).
+	parameterGrammar = base.makeStruct(svcs.ContextGrammar)
 
 	def __init__(self, jobClass, jobActions):
 		self.jobClass = jobClass
@@ -230,20 +229,23 @@ class UWS(object):
 				self._serializeProps(job.getProperties(), conn)
 		return job.jobId
 
-	def getNewIdFromRequest(self, request, service):
-		"""returns the id of a new TAP job created from request.
+	def getNewIdFromArgs(self, uwsArgs, paramDict, user=None):
+		"""returns a new UWS id for a job processing paramDict.
 
-		Request has to be a t.w request or similar, but preprocessed
-		by the renderer to have a uwsArgs attribute; see taprender for how this
-		is to be done.
+		paramDict need to be parsed already (typically, it's a 
+		CoreArgs.getParamDict() result) using the workerSystem's
+		parameterGrammar.
+
+		Pass user to restrict the job to that user.  No authentication
+		is done here, so make sure you actually authenticate user
+		(cf. getNewIdFromRequest for how this could be done).
 		"""
 		jobId = self.getNewJobId()
 		try:
 			with self.changeableJob(jobId) as wjob:
-				wjob.setParamsFromDict(request.uwsArgs)
-
-				if request.getAuthUser():
-					wjob.change(owner=request.getUser())
+				wjob.setParamsFromDict(paramDict)
+				if user is not None:
+					wjob.change(owner=user)
 		except:
 			# something went wrong while setting up the request; do away with it
 			# again
@@ -251,6 +253,20 @@ class UWS(object):
 			raise
 		return jobId
 
+	def getNewIdFromRequest(self, request):
+		"""returns the id of a new TAP job created from request.
+
+		Request has to be a t.w request or similar, but preprocessed
+		by prepareRequest below to have the parsed UWS parameters in uwsArgs.
+		"""
+		return self.getNewIdFromArgs(
+			request.uwsArgs,
+			self.parameterGrammar.parseStrargs(
+				request.strargs, ensureRequired=False),
+			request.getUser() 
+				if request.getAuthUser()
+				else None)
+
 	def _getJob(self, jobId, conn, writable=False):
 		"""helps getJob and getNewJob.
 		"""
@@ -458,12 +474,21 @@ class UWSWithQueueing(UWS):
 	_processQueueDirty = False
 	# How many jobs will the UWS (try to) run at the same time?
 	runcountGoal = 1
+	# how often should we do an extra pass over the queue to 
+	# see if we can run something?  (under normal circumstances,
+	# the queue is checked when a job goes out of executing
+	# more or less immediately; this is just a last-resort thing
+	# -- and it seeds the queue runner after a restart with UWS jobs.
+	queueCheckInterval = 60
 
 	def __init__(self, jobClass, actions):
 		# processQueue shouldn't strictly need a lock.  The lock mainly
 		# protects against running more unqueuers than necessary
 		self._processQueueLock = threading.Lock()
 		UWS.__init__(self, jobClass, actions)
+		cron.runEvery(-self.queueCheckInterval, 
+			"UWS %s timed queue processor"%str(self),
+			self._processQueue)
 
 	def _makeMoreStatements(self, statements, jobsTable):
 		UWS._makeMoreStatements(self, statements, jobsTable)
@@ -511,8 +536,8 @@ class UWSWithQueueing(UWS):
 	def _processQueue(self):
 		"""tries to take jobs from the queue.
 
-		This function is called from checkProcessQueue when we think
-		from EXECUTING so somewhere else.
+		This function is called from checkProcessQueue when we think we
+		are transitioning from EXECUTING to something else.
 
 		Currently, the jobs with the earliest destructionTime are processed
 		first.  That's, of course, completely ad-hoc.
@@ -545,8 +570,9 @@ class UWSWithQueueing(UWS):
 						# failures we want to make sure all jobs we think are executing
 						# actually are.  If they've silently died, we log that and
 						# push them to error.
-						# We only want to do that if we're the server -- any other
-						# process couldn't see the pids anyway.
+						# We only want to do that if we're the server; and it shouldn't
+						# really happen in normal operation, so we only run this
+						# if zombies might block proper jobs.
 						if base.IS_DACHS_SERVER:
 							self._ensureJobsAreRunning()
 				except Exception:
@@ -566,24 +592,29 @@ class UWSWithQueueing(UWS):
 					self.changeToPhase(jobId, "ERROR",
 						UWSError("EXECUTING job %s had no pid."%jobId, jobId))
 					base.ui.notifyError("Stillborn async slave %s"%jobId)
+
 				elif pid<2:
 					# these are jobs run within the server -- we can't do
 					# anything about them using process manipulation (if at all,
 					# they may be realised as threads).  Skip them here.
 					continue
+
 				else:
-					pass
-# We should be checking if the process is still running.  Alas,
-# there's serious syncing issues here that need to be investigated.
-# Let's rely on the slaves cleaning up behind themselves.
-#					try:
-#						os.waitpid(pid, os.WNOHANG)
-#					except os.error, ex: # child presumably is dead
-#						# the following doesn't hurt if the job has gone to COMPLETED
-#						# in the meantime -- we don't transition *from* COMPLETED.
-#						self.changeToPhase(jobId, "ERROR",
-#							uws.UWSError("EXECUTING job %s has silently died."%jobId, jobId))
-#						base.ui.notifyError("Zombie taprunner: %s"%jobId)
+					# Guard against jobs not cleaning up behind themselves;
+					# If they're kill -9-ed, segfault or anything else, they
+					# won't fix their phase.  We also can't be sure they are
+					# our children, as the server may have been restarted.
+					# Simple, somewhat ad-hoc solution: let's see if they're
+					# visible in /proc and push them to error otherwise.
+					# There are various ways in which this could go wrong
+					# (pid reuse, races), but it's still a lot better
+					# than nothing.
+					if not os.path.exists(f"/proc/{pid}"):
+						# the following doesn't hurt if the job has gone to COMPLETED
+						# in the meantime -- we don't transition *from* COMPLETED.
+						self.changeToPhase(jobId, "ERROR",
+							UWSError("EXECUTING job %s has silently died."%jobId, jobId))
+						base.ui.notifyError("Zombie UWS runner: %s"%jobId)
 
 	def changeToPhase(self, jobId, newPhase, input=None, timeout=10):
 		"""overridden here to hook in queue management.
@@ -607,153 +638,6 @@ class ParameterRef(object):
 		self.url = url
 
 
-class JobParameter(object):
-	"""A UWS job parameter.
-
-	Job parameters are (normally) elements of a job's parameters dictionary,
-	i.e., usually elements of the job control language.  "Magic" parameters
-	that allow automatic serialization or special effects on the job
-	can be defined using the _parameter_name class attributes of
-	UWSJobs.  They are employed when using the setSerializedPar and
-	getSerializedPar interface.
-
-	All methods of these are class or static methods since 
-	ProtocolParameters are never instanciated.
-
-	The default value setter enforces a maximum length of 50000 characters
-	for incoming strings.  This is a mild shield against accidental DoS
-	with, e.g., bad uploads in TAP.
-
-	The serialization thing really belongs to the user-facing interface.
-	However, since it's most convenient to have these parameters
-	in the UWSJob classes, the class is defined here.
-
-	Internal clients would read the parameters directly from the dictionary.
-
-	Methods you can override:
-
-		- addPar(value, job) -> None -- parse value and perform some action
-			(typically, set an attribute) on job.  The default implementation
-			puts a value into the parameters dictionary _deserialized.
-		- getPar(job) -> string -- infer a string representation of the
-			job parameter on job.  The default implementation gets
-			the value from the parameter from the parameters dictionary and
-			_serializes it.
-		- _deserialize(str) -> val -- turns something from an XML tree
-		  into a python value as usable by the worker
-		- _serialize(val) -> str -- yields some XML-embeddable serialization
-		  of val
-
-	The default implementation just dumps/gets name from the job's
-	parameters dict.  This is the behaviour for non-magic parameters
-	since (get|set)SerializedPar falls back to the base class.
-
-	CAUTION: Do *not* say job.parameters[bla] = "ab" -- your changes
-	will get lost because serialisation of the parameters dictionary must
-	be initiated manually.  Always manipulate the parameters dictionary
-	by using cls.updatePar(name, value, job) or a suitable
-	facade (job.setPar, job.setSerializedPar)
-	"""
-	_deserialize, _serialize = staticmethod(strOrEmpty), staticmethod(strOrEmpty)
-
-	@classmethod
-	def updatePar(cls, name, value, job):
-		"""enters name:value into job's parameter dict.
-
-		See the uws.JobParameter's docstring.
-		"""
-		# this is a bit magic because job.parameters is only re-encoded
-		# on demand
-		parameters = job.parameters
-		if isinstance(value, str):
-			value = cls._deserialize(value)
-		parameters[name] = value
-		# see our docstring
-		job.change(parameters=parameters)
-
-	@classmethod
-	def addPar(cls, name, value, job):
-		# this is a somewhat lame protection against file uploads
-		# gone haywire: ignore any values longer than 50k
-		if isinstance(value, str) and len(value)>50000:
-			base.ui.notifyWarning("UWS Parameter %s discarded as too long; first"
-				" bytes: %s"%(name, repr(value[:20])))
-		cls.updatePar(name, cls._deserialize(value), job)
-
-	@classmethod
-	def getPar(cls, name, job):
-		return cls._serialize(job.parameters.get(name))
-
-
-class UploadParameter(JobParameter):
-	"""A generic DALI-style upload facility.
-
-	We add this to all UWS job classes when their underlying services have
-	a file-typed input key.  It will contain some somewhat arbitrary string
-	that lets people guess what they uploaded.  TAP does this a bit
-	differently from useruws, which tries a somewhat rationalised approach.
-	"""
-	# the implementation is messy -- as for inline uploads, two parameters
-	# are involved (UPLOAD and the file parameter) and the normal UWS parameter
-	# interface only passes the parameter to be processed, we need to steal
-	# the request from upstack.   This, admittedly, is ugly, but then
-	# the UPLOAD design is botched, so I feel entitled to play it dirty
-	# rather than spoil my design.
-	@classmethod
-	def _deserialize(cls, value):
-		if value is None:
-			return []
-		return value.split("/")
-
-	@classmethod
-	def _serialize(cls, value):
-		if value is None:
-			return ""
-		return "/".join(value)
-
-	@classmethod
-	def addPar(cls, name, value, job):
-		if not value.strip():
-			return
-
-		for newFName in dali.writeUploadBytesTo(
-				utils.stealVar("request"), os.path.join(job.getWD())):
-			job.setPar(newFName, newFName)
-
-
-class FileParameter(JobParameter):
-	"""an uploaded file.
-
-	These are being created by posting to upload in the current design;
-	hence, we fail on an attempt to addPar those.  The serialisation
-	yields ParameterRefs.
-
-	Note that TAP uploads currently employ a different scheme since TAP
-	uploads don't match what DALI says.
-
-	The stored values are always URLs into our service, regardless of where
-	the upload came from.  For simplicity, we store the things in results.
-
-	TODO: We should preserve the media type of the upload where available.
-	"""
-	@classmethod
-	def _serialize(cls, value):
-		if value is None:
-			return ""
-		return ParameterRef(value)
-
-	@classmethod
-	def updatePar(cls, name, value, job):
-		# value is the file name (that happens to be the name of the input key;
-		# in DALI terms, it's what's in front of the comma.
-		JobParameter.updatePar(name, job.getURL()+"/results/"+value, job)
-
-	@classmethod
-	def addPar(self, name, value, job):
-		raise base.ValidationError("File parameters cannot be set by posting to"
-			" them.  Use DALI-style UPDATEs for them.", name)
-
-
 class UWSJobType(type):
 	"""The metaclass for UWS jobs.
 
@@ -781,6 +665,10 @@ class BaseUWSJob(object, metaclass=UWSJobType):
 	can read the properties as attributes.  UWSJobs also keep
 	a (weak) reference to the UWS that made them.
 
+	Make sure you don't confuse the UWS properties with the JCL parameters
+	(which are what the core receives).  the latter are in the
+	parameters property.
+
 	To alter uws properties, use the change method.  This will fail unless
 	the job was created giving writable=True.
 
@@ -805,15 +693,20 @@ class BaseUWSJob(object, metaclass=UWSJobType):
 	  id.  The default implementation does this, but if you want
 	  something in the file system, you probably don't want to
 	  use that.
-	- a method setParamsFromDict(argDict), which takes a dictionary 
-	  (in general the result of a contextgrammar) and sets the job parameters.
-	  This is only necessary if you need extra mappings between names and such.
 
-	For every piece of the job parameters, define
-	class attributes _parameters_<parname.lower()> with JobParameter
-	values saying how they are serialized and deserialized.   Only
-	parameters defined in this way are accepted and integrated
-	into the parameters dict.
+	The job parameters -- those eventually passed to the core
+	when the job runs -- are held in a dictionary parameters.  You
+	should in general not do any parsing of them at the UWS level, as
+	that's really the job of the service/core, as in the sync case.
+
+	However, certain job parameters need handling at the async level,
+	in particular UPLOAD.  In that case, define _preprocess_<parName>
+	methods receiving the value as parsed by the grammar and have to
+	return something that can be serialised to the database.
+
+	Note that when you assign to job.parameters yourself, these changes
+	will be ignored.  Always use setPar to modify job.parameters.
+	Reading job.parameters is fine.
 	
 	If you need to clean up before the job is torn down, redefine
 	the prepareForDestruction method.
@@ -932,78 +825,119 @@ class BaseUWSJob(object, metaclass=UWSJobType):
 		"""
 		return pickle.loads(utils.getDirtyBytes(value))
 
-	def _getParameterDef(self, parName):
-		"""returns the job/uws parameter definition for parName and the name 
-		the parameter will actually be stored as.
-
-		All these parameters are forced to be lower case (and thus
-		case-insensitive).  The actual storage name of the parameter is
-		still returned in case saner conventions may be forthcoming.
-		"""
-		parName = parName.lower()
-		name = "_parameter_"+parName
-		if hasattr(self, name):
-			return getattr(self, name), parName
-		return JobParameter, parName
-
-	def setSerializedPar(self, parName, parValue):
-		"""enters parName:parValue into self.parameters after deserializing it.
-
-		This is when input comes from text; use setPar for values already 
-		parsed.
-		"""
-		parDef, name = self._getParameterDef(parName)
-		parDef.addPar(name, parValue, self)
-
 	def setPar(self, parName, parValue):
 		"""enters parName:parValue into self.parameters.
 		"""
-		parDef, name = self._getParameterDef(parName)
-		parDef.updatePar(name, parValue, self)
+		parName = parName.lower()
+		if parValue is None and self.parameters.get(parName):
+			# don't nuke a parameter that already exists.
+			return
 
-	def getSerializedPar(self, parName):
-		"""returns self.parameters[parName] in text form.
+		if hasattr(self, "_preprocess_"+parName):
+			parValue = getattr(self, "_preprocess_"+parName)(parValue)
+		newPars = self.parameters.copy()
 
-		This is for use from a text-based interface.  Workers read from
-		parameters directly.
-		"""
-		parDef, name = self._getParameterDef(parName)
-		return parDef.getPar(name, self)
+		# We'll get DALI uploads as 2-tuples with a BytesIO in the
+		# second item.  Save that before serialisation.
+		# Perhaps we should re-use this for TAP (where the analogous
+		# thing happens in _preprocess_upload).
+		if isinstance(parValue, tuple):
+			try:
+				if isinstance(parValue[1], io.BytesIO):
+					parValue = (parName, str(self._saveUpload(parName, parValue[1])))
+			except IndexError:
+				# parValue is too short, hence it's no upload
+				pass
+
+		# we had better check the multiplicity of the inputKey to decide
+		# between extending and overwriting; but for now that's about
+		# equivalent.
+		if isinstance(newPars.get(parName), list):
+			newPars[parName].extend(parValue)
+		elif isinstance(newPars.get(parName), dict):
+			# that's a temporary hack for dlasync until we get to properly 
+			# review how to really do that kind of thing
+			newPars[parName].update(parName)
+		else:
+			newPars[parName] = parValue
+		self.change(parameters=newPars)
 
 	def iterSerializedPars(self):
 		"""iterates over the serialized versions of the parameters.
-		"""
-		for key in self.iterParameterNames():
-			yield key, self.getSerializedPar(key)
 
-	def iterParameterNames(self):
-		"""iterates over the names of the parameters declared for the job.
+		This is, really for uwsactions.getParametersElement, and uses
+		a secret handshake to let this create ByReference links.
 		"""
-		for n in dir(self):
-			if n.startswith("_parameter_"):
-				yield n[11:]
+		for key, value in self.parameters.items():
+			if (isinstance(value, list) 
+					and len(value)>0
+					and isinstance(value[0], tuple)
+					and len(value[0])==2 
+					and value[0][1].startswith("file://")):
+				# this quite certainly is one of our uploaded files
+				for upload in value:
+					yield key, ParameterRef(
+						LocalFile(self, self.getWD(), upload[1].split("/")[-1]
+						).getURL())
+
+			elif (isinstance(value, tuple)
+					and len(value)==2 
+					and value[1].startswith("file://")):
+				# same thing for DALI uploads
+					yield key, ParameterRef(
+						LocalFile(self, self.getWD(), value[1].split("/")[-1]
+						).getURL())
+
+			else:
+				yield key, "" if value is None else str(value)
 
 	def setParamsFromDict(self, argDict):
-		"""sets our parameters from a dictionary of string lists.
+		"""sets our parameters from a dictionary of parsed parameters.
 
-		self must be writable for this to work.
+		self must be writeable for this to work.
 
 		argDict should in general be the result of a contextgrammar,
 		but, really, any dict will do.
+
+		We force-lowercase everything here.  If that's a problem for your
+		protocol, you deserve no mercy.
 		"""
-		for key in self.iterParameterNames():
-			if key in argDict:
-				val = argDict[key]
-				# TODO: handling multiple arguments must be way better thought
-				# than just blank-joining them.  We *could* pickle lists, of
-				# course... Well, let's wait for when we have a good case.
-				if isinstance(val, list):
-					val = " ".join(val)
-				if not val:
-					# have some way to re-set a parameter?  Anyway, I need to
-					# ignore empty parameters or my XSLT form will break
-					continue
-				self.setSerializedPar(key, val)
+		for key, value in argDict.items():
+			self.setPar(key.lower(), value)
+	
+	def setParamsFromRawDict(self, strargs):
+		"""sets our parameters from a dictionary of string lists (i.e.,
+		requests.strargs).
+
+		The job must be writeable when you call this.
+
+		The arguments don't have to be complete by the parameterGrammar;
+		this also means that no defaults are inserted.  Use job.completeParams()
+		for that (it's typically called when queuing a job).
+		"""
+		self.setParamsFromDict(
+			self.uws.parameterGrammar.parseStrargs(strargs, ensureRequired=False))
+
+	def completeParams(self):
+		"""completes self's parameters from the worker's context grammar.
+	
+		Job must be writable for this to work.
+
+		This is necessary because in UWS operations, actions for
+		missing parameters are never executed; missing parameters
+		could still be provided later.
+
+		Worker systems hence must arrange for this to be called when queueing
+		a job; SimpleUWSTransitions.queueJob already does that.  This may raise 
+		an exception if parameters are missing.
+		"""
+		pars = self.parameters
+		for ik in self.uws.parameterGrammar.iterInputKeys():
+			key = ik.name.lower()
+			val = pars.get(key)
+			if val is None or val==[]:
+				pars[key] = ik.computeCoreArgValue([])
+		self.change(parameters=pars)
 
 	def getTransitionTo(self, newPhase):
 		"""returns the action prescribed to push self to newPhase.
@@ -1046,6 +980,7 @@ class BaseUWSJob(object, metaclass=UWSJobType):
 		a UWS job from the database.
 		"""
 		self._props = self.uws.getJob(self.jobId)._props
+		return self
 
 	def prepareForDestruction(self):
 		"""is called before the job's database row is torn down.
@@ -1077,6 +1012,26 @@ class BaseUWSJob(object, metaclass=UWSJobType):
 		self.update()
 
 
+class LocalFile(object):
+	"""A sentinel class representing a file within a job work directory
+	(as resulting from an upload).
+	"""
+	def __init__(self, jobId, wd, fileName):
+		self.job, self.fileName = weakref.proxy(jobId), fileName
+		self.fullPath = os.path.join(wd, fileName)
+
+	def __str__(self):
+		# This is mainly for serialisation of the upload parameter in the
+		# UWS job.
+		return f"file://{self.fileName}"
+
+	def getURL(self):
+		"""returns the URL the file is retrievable under for the life time of
+		the job.
+		"""
+		return self.job.getURL()+f"/results/{self.fileName}"
+
+
 class UWSJobWithWD(BaseUWSJob):
 	"""A UWS job with a working directory.
 
@@ -1152,6 +1107,32 @@ class UWSJobWithWD(BaseUWSJob):
 
 		self._saveResults(resTable)
 
+	def _preprocess_upload(self, uploadParam):
+		"""picks non-saved inline uploads from uploadParam and saves them.
+
+		This is done so we can serialise the uploads into the database; we
+		wouldn't want to do that with the uploads.
+
+		Note that this is called by setPar("upload") by virtue of its
+		magic name.
+		"""
+		if not uploadParam:
+			return None
+
+		processed = []
+		for destTable, target in uploadParam:
+			if isinstance(target, tuple):
+				target = self._saveUpload(destTable, target[1])
+			processed.append((destTable, str(target)))
+		return processed
+
+	def _saveUpload(self, localName, uploadFile):
+		"""helps preprocess_upload.
+		"""
+		with self.openFile(localName, "wb") as f:
+			f.write(uploadFile.read())
+		return LocalFile(self, self.getWD(), localName)
+
 	def fixTypeForResultName(self, resultName, mediaType):
 		"""sets the media type for result resultName.
 
@@ -1210,10 +1191,10 @@ class UWSJobWithWD(BaseUWSJob):
 	def openFile(self, name, mode="rb"):
 		"""returns an open file object for a file within the job's work directory.
 
-		No path parts are allowed on name.
+		This will only use the last path component, everything else is
+		discarded.
 		"""
-		if "/" in name:
-			raise ValueError("No path components allowed on job files.")
+		name = name.split("/")[-1]
 		return open(os.path.join(self.getWD(), name), mode)
 
 
@@ -1335,6 +1316,7 @@ class SimpleUWSTransitions(UWSTransitions):
 	def queueJob(self, newState, wjob, ignored):
 		"""puts a job on the queue.
 		"""
+		wjob.completeParams()
 		wjob.change(phase=QUEUED)
 
 	def markAborted(self, newState, wjob, ignored):
@@ -1420,10 +1402,8 @@ class _UWSBackendProtocol(protocol.ProcessProtocol):
 		try:
 			job = self.workerSystem.getJob(self.jobId)
 			if job.phase==QUEUED or job.phase==EXECUTING:
-				try:
-					raise UWSError("Job hung in %s"%job.phase, job.jobId)
-				except UWSError as ex:
-					self.workerSystem.changeToPhase(self.jobId, ERROR, ex)
+				self.workerSystem.changeToPhase(self.jobId, ERROR, 
+					UWSError("Job hung in %s"%job.phase, job.jobId))
 		except JobNotFound: # job already deleted
 			pass
 
@@ -1433,7 +1413,7 @@ class ProcessBasedUWSTransitions(SimpleUWSTransitions):
 
 	Inheriting classes must implement the getCommandLine(wjob) method --
 	it must return a command (suitable for reactor.spawnProcess and
-	os.execlp and a list of arguments suitable for reactor.spawnProcess.
+	os.execlp) and a list of arguments suitable for reactor.spawnProcess.
 
 	They must also implement some sort of queue management.  The the simplest
 	case, override queueJob and start the job from there (but set
@@ -1519,3 +1499,72 @@ class ProcessBasedUWSTransitions(SimpleUWSTransitions):
 			raise
 		except Exception as ex:
 			raise UWSError(None, ex)
+
+
+@functools.lru_cache(None)
+def _getUWSGrammar():
+	"""returns the context grammar for the standard UWS parameters.
+	"""
+	inputTD = base.parseFromString(svcs.InputTD,
+		"""<inputTable id="generic-uws-args">
+			<inputKey name="upload" type="file"
+				multiplicity="multiple"
+				description="A DALI-type upload string"/>
+			<inputKey name="runid" type="text"
+				description="Ignored by this service."/>
+			<inputKey name="action" type="text"
+				multiplicity="single"
+				description="On posts to the job URI, you can pass in DELETE here
+					instead of using the standards-compliant DELETE method.
+					Ignored otherwise."/>
+			<inputKey name="wait" type="text"
+				unit="s"
+				multiplicity="single"
+				description="On certain operations, pass a timeout here to
+					have the server delay its response until there is a change
+					to report."/>
+			<inputKey name="phase" type="text"
+				multiplicity="single"
+				description="A UWS PHASE; the interpretation is different across
+					the UWS operations."/>
+			<inputKey name="executionduration" type="text"
+				unit="s"
+				multiplicity="single"
+				description="When posting to the job resource, set the maximum
+					run time of the query to this."/>
+			<inputKey name="destruction" type="text"
+				multiplicity="single"
+				description="When posting to the job resource, adjust the
+					time at which the job will be thrown away to this."/>
+		</inputTable>""")
+	return base.makeStruct(svcs.ContextGrammar, inputTD=inputTD)
+
+
+def prepareRequest(request, service):
+	"""prepares request for processing UWS requests.
+
+	In UWS, the service in general doesn't run, and hence there's no
+	context grammar running.  Also, there are the UWS-specific parameters,
+	which the core shouldn't be seeing.
+
+	These latter ones are parsed into request.uwsArgs by this functions. This
+	is what drives uwsactions.  The function removes the corresponding keys from
+	strargs so the core in question is not confused by them.
+	"""
+	uwsGrammar = _getUWSGrammar()
+	request.uwsArgs = uwsGrammar.parseStrargs(request.strargs)
+	uwsKeys = [ik.name for ik in uwsGrammar.iterInputKeys()]
+
+	if service.core.name_=="tapCore":
+		from gavo.protocols import tap
+		tap.mangleUploads(request)
+		# TAP needs to retain the upload parameter (in contrast to DALI,
+		# where the keys already contain the uploads)
+		uwsKeys.remove("upload")
+	else:
+		dali.mangleUploads(request)
+
+	# now remove the UWS arguments so they don't confuse any actual cores.
+	for uwsInputName in uwsKeys:
+		if uwsInputName in request.strargs:
+			del request.strargs[uwsInputName]
