@@ -13,7 +13,7 @@ using common.getResType; this means the resType meta is tried first,
 using resob.resType as a fallback.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -115,12 +115,11 @@ _vrResourceBuilder = meta.ModelBasedBuilder([
 			(metaName, SF(VOR.relatedResource), (), {
 					"ivoId": "ivoId"})])
 			for metaName, vorTerm in [
-				("servedBy", "served-by"),
-				("serviceFor", "service-for"),
-				("derivedFrom", "dervived-from"),
-				("relatedTo", "related-to"),
-				("mirrorOf", "mirror-of"),
+				("servedBy", "IsServedBy"),
+				("serviceFor", "IsServiceFor"),
+				("derivedFrom", "IsDerivedFrom"),
 				("relatedTo", "related-to"),
+				("mirrorOf", "IsIdenticalTo"),
 				("uses", "Cites")]
 		] + [
 		# new-style relationship terms from VOResource 1.1
