@@ -7,7 +7,7 @@ Thus, in the module we mostly deal with publication objects.  If you
 need the service object, use publication.parent.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -20,9 +20,11 @@ from gavo import svcs
 from gavo import utils
 from gavo.base import meta
 from gavo.base import osinter
+from gavo.protocols import dali
+from gavo.protocols import tap
 from gavo.registry import tableset
 from gavo.registry.model import (
-	VOR, VOG, VS, SIA, SCS, SLAP, SSAP, TR, DOC)
+	VOR, VOG, VS, SIA, SCS, SLAP, SSAP, TR, DOC, DaFut)
 
 
 ###################### Helpers (TODO: Move to tableset, I guess)
@@ -50,9 +52,19 @@ def getInputParamFromColumn(column, rootElement=VS.param):
 def getInputParams(publication, service):
 	"""returns a sequence of vs:param elements for the input of service.
 	"""
+	inputParams = []
+	renderer = svcs.getRenderer(publication.render)
 	if hasattr(service, "getInputKeysFor"): # no params on published tables
-		return [getInputParamFromColumn(f) 
-			for f in service.getInputKeysFor(svcs.getRenderer(publication.render))]
+		for param in service.getInputKeysFor(renderer):
+			# there's similar code in vodal.DALRenderer._getMetadataData;
+			# If more mess like this turns up, we'll have to think about
+			# a mechanism for per-renderer, pre-service processing, parameter
+			# mangling.
+			if param.type=="file" and renderer.parameterStyle=="dali":
+				param = dali.getUploadKeyFor(param)
+			inputParams.append(getInputParamFromColumn(param))
+
+	return inputParams
 
 
 ####################### Interfaces
@@ -132,6 +144,27 @@ class VOSITMInterface(VOSIInterface):
 	renderer = "tableMetadata"
 
 
+class DALIInterface(InterfaceMaker):
+	interfaceClass = DaFut.daliInterface
+
+	def _makeInterface(self, publication):
+		# TODO: figure out where I'll get the endpoints in the end; going
+		# by publication's render attribute will make this extremely ugly
+		# over time.
+		res = InterfaceMaker._makeInterface(self, publication)(version="1.1")
+		if publication.render!="dali":
+			raise NotImplementedError("Don't know how to make DALIInterfaces"
+				" for anything but TAP yet.")
+	
+		return res[
+			DaFut.endpoint[TR.name["sync"]],
+			DaFut.endpoint[TR.name["async"],
+				DaFut.meta(
+					property="http://dc.g-vo.org/static/tap-plan")["dachs"]],
+			DaFut.endpoint[TR.name["tables"]],
+			DaFut.endpoint[TR.name["examples"]]]
+
+
 class InterfaceWithParams(InterfaceMaker):
 	"""An InterfaceMaker on a publication sporting input parameters.
 
@@ -200,11 +233,12 @@ class TAPInterface(InterfaceMaker):
 # for TAP, result type is tricky, and we don't have good metadata
 # on the accepted input parameters (QUERY, etc).  We should really
 # provide them when we have extensions...
-	renderer = "tap"
+	renderer = "dali"
 	interfaceClass = TR.interface
 
 	def _makeInterface(self, publication):
-		return InterfaceMaker._makeInterface(self, publication)(version="1.1")
+		return InterfaceMaker._makeInterface(self, publication)(
+			version=tap.TAP_VERSION)
 
 
 class ExamplesInterface(InterfaceMaker):
@@ -275,12 +309,6 @@ _getInterfaceMaker = utils.buildClassResolver(InterfaceMaker,
 	default=InterfaceWithParams())
 
 
-def getInterfaceElement(publication):
-	"""returns the appropriate interface definition for service and renderer.
-	"""
-	return _getInterfaceMaker(publication.render)(publication)
-	
-
 ####################### Capabilities
 
 
@@ -307,11 +335,23 @@ class CapabilityMaker(object):
 	capabilityClass = VOR.capability
 	auxiliaryId = None
 
+	def _getInterfaceElement(self, publication):
+		"""returns the appropriate interface definition(s) for service and 
+		renderer.
+		"""
+		if publication.auxiliary and publication.render=="dali":
+			# for now, we want to make ParamHTTP auxiliary interfaces
+			# for TAP, not the new-fangled DALI ones.
+			interfaceMaker = TAPInterface()
+		else:
+			interfaceMaker = _getInterfaceMaker(publication.render)
+		return interfaceMaker(publication)
+	
 	def _makeCapability(self, publication):
 		return self.capabilityClass[
 			VOR.description[base.getMetaText(publication, "description", 
 				propagate=False, macroPackage=publication.parent)],
-			getInterfaceElement(publication)]
+			self._getInterfaceElement(publication)]
 
 	def __call__(self, publication):
 		return self._makeCapability(publication)
@@ -409,9 +449,6 @@ class SSACapabilityMaker(CapabilityMaker):
 			SSAP.complianceLevel[
 				service.getMeta("ssap.complianceLevel", default="minimal")]]
 
-		for m in list(service.iterMeta("productType", propagate=True)) or ["spectrum"]:
-			cap[SSAP.productType[str(m)]]
-
 		cap[
 			SSAP.dataSource[service.getMeta("ssap.dataSource", raiseOnFail=True)],
 			SSAP.creationType[service.getMeta("ssap.creationType", 
@@ -423,7 +460,7 @@ class SSACapabilityMaker(CapabilityMaker):
 			SSAP.maxAperture["90"],
 			SSAP.testQuery[
 				SSAP.queryDataCmd[base.getMetaText(service, "ssap.testQuery", 
-					raiseOnFail=True)+"&REQUEST=queryData"]],
+					raiseOnFail=True)]],
 		]
 		return cap
 
@@ -449,10 +486,19 @@ _tapModelBuilder = meta.ModelBasedBuilder([
 		{"ivoId": "ivoId"})])
 
 class TAPCapabilityMaker(CapabilityMaker):
-	renderer = "tap"
+	renderer = "dali"
 	capabilityClass = TR.capability
 	auxiliaryId = "ivo://ivoa.net/std/TAP#aux"
 
+	def _getInterfaceElement(self, publication):
+		"""for TAP, we return both a 1.0 pseudo-ParamHTTP and a DALIInterface.
+
+		The DALIInterface part is experimental for now.
+		"""
+		yield TAPInterface()(publication)
+		if "dali-interface-in-tap" in base.getConfig("future"):
+			yield DALIInterface()(publication)
+
 	def _makeCapability(self, publication):
 		res = CapabilityMaker._makeCapability(self, publication)
 	
@@ -465,6 +511,13 @@ class TAPCapabilityMaker(CapabilityMaker):
 					for dmname, dmivoid in conn.query(
 						"select dmname, dmivorn from tap_schema.supportedmodels")]]
 
+			udfs = []
+			for udf in ufunctions.UFUNC_REGISTRY.values():
+				udfs.append({
+						"signature": udf.adqlUDF_signature, 
+						"doc": udf.adqlUDF_doc})
+				udfs.extend(udf.adqlUDF_additionalSignatures)
+
 			res[
 				# Once we support more than one language, we'll have to
 				# revisit this -- the optional features must then become
@@ -477,11 +530,9 @@ class TAPCapabilityMaker(CapabilityMaker):
 						TR.languageFeatures(
 							type="ivo://ivoa.net/std/TAPRegExt#features-udf")[
 							[TR.feature[
-								TR.form[udf.adqlUDF_signature],
-								TR.description[udf.adqlUDF_doc]]
-							for udf in sorted(
-									list(ufunctions.UFUNC_REGISTRY.values()),
-									key=lambda v:v.adqlUDF_signature)]],
+								TR.form[udf["signature"]],
+								TR.description[udf["doc"]]]
+							for udf in sorted(udfs, key=lambda v: v["signature"])]],
 
 						TR.languageFeatures(
 								type="ivo://ivoa.net/std/TAPRegExt#features-adqlgeo")[
@@ -537,6 +588,10 @@ class TAPCapabilityMaker(CapabilityMaker):
 									"  It either takes a string argument with an ASCII"
 									" MOC ('4/13 17-18 8/3002'), or an order and another"
 									" geometry."]],
+							TR.feature[
+								TR.form["COALESCE"],
+								TR.description["This is the standard SQL COALESCE"
+									" for providing defaults in case of NULL values."]],
 							],
 
 						TR.languageFeatures(
