@@ -2,7 +2,7 @@
 The schema and XML namespaces for OAI/VOR documents.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -25,32 +25,37 @@ registerPrefix("oai_dc", "http://www.openarchives.org/OAI/2.0/oai_dc/",
 	schemaURL("oai_dc.xsd"))
 registerPrefix("ri",
 	"http://www.ivoa.net/xml/RegistryInterface/v1.0",
-	schemaURL("RegistryInterface-v1.0.xsd"))
+	schemaURL("RegistryInterface.xsd"))
 registerPrefix("vg", "http://www.ivoa.net/xml/VORegistry/v1.0",
-	schemaURL("VORegistry-v1.0.xsd"))
+	schemaURL("VORegistry.xsd"))
 registerPrefix("vr", "http://www.ivoa.net/xml/VOResource/v1.0",
-	schemaURL("VOResource-v1.1.xsd"))
+	schemaURL("VOResource.xsd"))
 registerPrefix("dc", "http://purl.org/dc/elements/1.1/",
 	schemaURL("simpledc20021212.xsd"))
 registerPrefix("vs", "http://www.ivoa.net/xml/VODataService/v1.1",
-	schemaURL("VODataService-v1.2.xsd"))
+	schemaURL("VODataService.xsd"))
 registerPrefix("cs", "http://www.ivoa.net/xml/ConeSearch/v1.0",
-	schemaURL("ConeSearch-v1.1.xsd"))
+	schemaURL("ConeSearch.xsd"))
 registerPrefix("sia", "http://www.ivoa.net/xml/SIA/v1.1",
-	schemaURL("SIA-v1.2.xsd"))
+	schemaURL("SIA.xsd"))
 registerPrefix("slap", "http://www.ivoa.net/xml/SLAP/v1.0",
-	schemaURL("SLAP-v1.1.xsd"))
+	schemaURL("SLAP.xsd"))
 registerPrefix("ssap", "http://www.ivoa.net/xml/SSA/v1.1",
-	schemaURL("SSA-v1.3.xsd"))
+	schemaURL("SSA.xsd"))
 registerPrefix("tr", "http://www.ivoa.net/xml/TAPRegExt/v1.0",
-	schemaURL("TAPRegExt-v1.0.xsd"))
+	schemaURL("TAPRegExt.xsd"))
 registerPrefix("vstd", "http://www.ivoa.net/xml/StandardsRegExt/v1.0",
-	schemaURL("StandardsRegExt-1.0.xsd"))
+	schemaURL("StandardsRegExt.xsd"))
 registerPrefix("doc", "http://www.ivoa.net/xml/DocRegExt/v1",
-	schemaURL("DocRegExt-v1.0.xsd"))
+	schemaURL("DocRegExt.xsd"))
 registerPrefix("voe", "http://www.ivoa.net/xml/VOEventRegExt/v2",
-	schemaURL("VOEventRegExt-v2.0.xsd"))
-
+	schemaURL("VOEventRegExt.xsd"))
+registerPrefix("g-colstat", "http://dc.g-vo.org/ColStats-1",
+	schemaURL("Colstats.xsd"))
+registerPrefix("b2f", "http://b2find.eudat.eu/schema/b2f/2.0",
+	schemaURL("b2find.xsd"))
+registerPrefix("dachs", "http://docs.g-vo.org/schemata/DaCHS.xsd",
+	schemaURL("DaCHS.xsd"))
 
 
 class _ResourceNameMixin(object):
@@ -425,11 +430,12 @@ class VS:
 	
 	class title(VSElement): pass
 	class utype(VSElement): pass
+	class nrows(VSElement): pass
 	
 	class table(VSElement):
 		_a_type = None
 		_childSequence = ["name", "title", "description", "utype",
-			"column", "foreignKey"]
+			"nrows", "column", "foreignKey"]
 
 	class foreignKey(VSElement):
 		_childSequence = ["targetTable", "fkColumn", "description", "utype"]
@@ -506,7 +512,23 @@ class VS:
 		_a_ivoId = None
 		_name_a_ivoId = "ivo-id"
 
-	class column(VSElement): pass
+	# column for now contains our prototype g-colstat attributes;
+	# these ought to be removed as we leave the prototyping state.
+	# In particular, the _additionalPrefixes should go again then
+	class column(VSElement):
+		_additionalPrefixes = frozenset(["g-colstat"])
+		_a_minValue = None
+		_name_a_minValue = "g-colstat:min-value"
+		_a_maxValue = None
+		_name_a_maxValue = "g-colstat:max-value"
+		_a_median = None
+		_name_a_median = "g-colstat:median"
+		_a_percentile03 = None
+		_name_a_percentile03 = "g-colstat:percentile03"
+		_a_percentile97 = None
+		_name_a_percentile97 = "g-colstat:percentile97"
+		_a_fillFactor = None
+		_name_a_fillFactor = "g-colstat:fillFactor"
 
 	class dataType(VSElement):
 		name_ = "dataType"
@@ -515,6 +537,7 @@ class VS:
 		_a_delim = None
 		_a_extendedSchema = None
 		_a_extendedType = None
+		_a_xsi_type = None
 
 	class dataType(dataType):
 		name_ = "dataType"
@@ -635,7 +658,6 @@ class SSAP(object):
 	class supportedFrame(SSAElement): pass
 	class testQuery(SSAElement): pass
 	class queryDataCmd(SSAElement): pass
-	class productType(SSAElement): pass
 
 
 class SLAP(object):
@@ -673,11 +695,25 @@ class TR(object):
 		_a_xsi_type = "vs:ParamHTTP"
 		_additionalPrefixes = frozenset(["xsi", "vs"])
 
+	class daliInterface(VOR.interface):
+		name_ = "interface"
+		_a_role = "std"
+		_a_xsi_type = "tr:DALIInterface"
+		_additionalPrefixes = frozenset(["xsi"])
+
 	class capability(VOR.capability):
 		_a_standardID = 	"ivo://ivoa.net/std/TAP"
 		_a_xsi_type = "tr:TableAccess"
 		_additionalPrefixes = frozenset(["tr", "xsi"])
 
+	class endpoint(TRElement):
+		pass
+	
+	class meta(TRElement):
+		_a_about = None
+		_a_property = None
+		_a_resource = None
+
 	class dataModel(TRElement):
 		_a_ivoId = None
 		_name_a_ivoId = "ivo-id"
@@ -766,3 +802,26 @@ class DOC(object):
 
 	class languageCode(DOCElement): pass
 	class locTitle(DOCElement): pass
+
+
+class DaFut(object):
+	"""A container for XSD elements we want to push into other schemas
+	one of these days ("DaCHS Future").
+	"""
+	class DaFutElement(Element):
+		_prefix = "dachs"
+		_local = True
+
+	class daliInterface(VOR.interface):
+		name_ = "interface"
+		_a_role = "std"
+		_a_xsi_type = "dachs:DALIInterface"
+		_additionalPrefixes = frozenset(["xsi"])
+
+	class endpoint(DaFutElement):
+		pass
+	
+	class meta(DaFutElement):
+		_a_about = None
+		_a_property = None
+		_a_resource = None
