@@ -12,7 +12,7 @@ the content of the service table is not actually used to build resource
 records.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -67,13 +67,17 @@ def makeBaseRecord(res, keepTimestamp=False):
 	ivoid = base.getMetaText(res, "identifier")
 	auth = urllib.parse.urlparse(ivoid).netloc
 
-	if auth=="x-unregistred":
-		raise base.ReportableError("Resource from %s uses the authority"
-			" x-unregistered; this is not permitted."%res.rd.sourceId,
-			hint="This in all likelihood means that you are not setting"
-			" the [ivoa]authority config item in your gavo.rc.  Read up"
-			" on 'Choosing your authority' in the tutorial to see how"
-			" to fix this.")
+	if (auth=="x-unregistred" 
+			and not "yes"==base.getMetaText(res, "i-am-sure", "")):
+		# allow this if people have no ivo_managed publications
+		if list(res.getPublicationsForSet(
+				{"ivo_managed"}, includeDatalink=False)):
+			raise base.ReportableError("Resource {} uses the authority"
+				" x-unregistered; this is not permitted.".format(res.getFullId()),
+				hint="This in all likelihood means that you are not setting"
+				" the [ivoa]authority config item in your gavo.rc.  Read up"
+				" on 'Choosing your authority' in the tutorial to see how"
+				" to fix this.")
 
 	if not auth in getManagedAuthorities():
 		base.ui.notifyWarning("Skipping publication of resource"
@@ -366,6 +370,7 @@ def updateServiceList(rds,
 				tap.publishToTAP(rd, connection)
 
 			deletedUpdater()
+			connection.commit()
 
 		except base.MetaValidationError as ex:
 			msg = ("Aborting publication of rd '%s' since meta structure of"
