@@ -2,7 +2,7 @@
 Making data out of descriptors and sources.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -455,7 +455,7 @@ def processSource(data, source, feeder, opts, connection=None):
 			raise base.ReportableError("Can only ignore source errors"
 				" when filling database tables.", 
 				hint="The -c flag on dachs imp and its friends builds on database"
-				" savepoints.  You can thus only meaninfully use it when your"
+				" savepoints.  You can thus only meaningfully use it when your"
 				" table has onDisk='True'.")
 		try:
 			with connection.savepoint():
