@@ -2,7 +2,7 @@
 Tables on disk
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -596,20 +596,68 @@ class DBTable(DBMethodsMixin, table.BaseTable, MetaTableMixin):
 		if not self.exists():
 			self.create()
 		return self
-	
-	def drop(self):
+
+	def _iterDerivedViews(self):
+		"""yields view that contain the current table.
+
+		This is in particular used when dropping the table (rather than rely on
+		cascade) in order to give view's dropping scripts a chance to run.
+		"""
+		depNames = set(r[0] for r in self.connection.query("""
+				SELECT DISTINCT v.oid::regclass as table_name
+				FROM pg_depend AS d
+					JOIN pg_rewrite AS r ON r.oid = d.objid
+					JOIN pg_class AS v ON v.oid = r.ev_class
+				WHERE v.relkind = 'v'
+					AND d.classid = 'pg_rewrite'::regclass
+					AND d.refclassid = 'pg_class'::regclass
+					AND d.deptype = 'n'
+					AND refobjid=%(qname)s::regclass""",
+				{"qname": self.tableDef.getQName()}))
+
+		if not depNames:
+			return
+
+		myId = self.tableDef.getFullId()
+		for depId in [r[1]+"#"+r[0].split(".")[-1]
+				for r in self.connection.query("""SELECT tablename, sourcerd
+					FROM dc.tablemeta
+					WHERE tablename IN %(depNames)s""",
+					locals())]:
+			if depId==myId:
+				# can't be bothered to figure out why a view apparently
+				# depends on itself by the logic above
+				continue
+
+			try:
+				yield base.resolveCrossId(depId)
+			except base.NotFoundError:
+				base.ui.notifyError("Would have liked to drop %s before"
+					" dropping %s, but found its definition gone."%(
+					depId, self.tableDef.getQName()))
+
+	def drop(self, seenIds=()):
 		"""drops the table.
+
+		This will recurse into dependent objects.
 		"""
 		self.runScripts("beforeDrop")
 		if self.exists():
+			if seenIds==():
+				seenIds = set()
+
 			# if there are any dependent views (known), drop them in a controlled
 			# fashion becore CASCADE hits
-			for viewId in self.tableDef.derivedViews:
-				View(
-					base.resolveCrossId(viewId),
-					connection=self.connection,
-					create=False).drop()
+			for viewDef in self._iterDerivedViews():
+				if id(viewDef) in seenIds:
+					continue
+				seenIds.add(id(viewDef))
 
+				View(
+					viewDef,
+					connection=self.connection,
+					create=False).drop(seenIds)
+				
 			self.dropTable(self.tableDef.getQName(), cascade=True)
 			if not self.nometa:
 				self.cleanFromMeta()
@@ -755,9 +803,10 @@ class View(DBTable):
 	construct a DBTable or a View).  You can get a feeder for them,
 	but trying to actually feed anything will raise a DataError.
 
-	On import, views only run postCreation scripts; since there are no indices,
-	no preIndex scripts are run, since no import takes place, there's no
-	preImport or newSource.
+	On import, views only run postCreation scripts; we assume everything
+	else (preIndex, postIndex, preImport, newSource, etc) has run
+	in the contributing tables.  Materialised views *will* create indices,
+	however.
 	"""
 
 	def __init__(self, *args, **kwargs):
@@ -794,10 +843,8 @@ class View(DBTable):
 					self.tableDef.getSourcePosition(),
 					msg))
 
-	def makeIndices(self):
-		return self  # no indices or primary keys on views.
-
 	def importFinished(self, nImported):
-		# don't do anything but run postCreation scripts
+		if self.getTableType(self.tableDef.getQName())=="MATERIALIZED VIEW":
+			self.makeIndices()
 		self.runScripts("postCreation")
 		return self
