@@ -2,7 +2,7 @@
 Common items used by resource definition objects.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -96,11 +96,12 @@ class ProfileListAttribute(base.AtomicAttribute):
 	There's the special role name "defaults" for whatever default this 
 	profile list was constructed with.
 	"""
-	typeDesc_ = "Comma separated list of profile names."
+	typeDesc_ = "Comma separated list of profile names"
 
-	def __init__(self, name, default, description):
+	def __init__(self, name, default, description, defaultSource):
 		base.AtomicAttribute.__init__(self, name, base.Computed, description)
 		self.realDefault = default
+		self.defaultSource_ = defaultSource
 	
 	@property
 	def default_(self):
@@ -123,6 +124,10 @@ class ProfileListAttribute(base.AtomicAttribute):
 # certainly not worth the effort.
 		return ", ".join(value)
 
+	def makeUserDoc(self):
+		return (f"**{self.name_}** ({self.typeDesc_}, defaults to"
+			f" {self.defaultSource_} from gavorc) -- {self.description_}")
+
 
 class PrivilegesMixin(object):
 	"""A mixin for structures declaring access to database objects (tables,
@@ -137,12 +142,13 @@ class PrivilegesMixin(object):
 	_readProfiles = ProfileListAttribute("readProfiles", 
 		default=base.getConfig("db", "queryProfiles"),
 		description="A (comma separated) list of profile names through"
-			" which the object can be read.")
+			" which the object can be read.",
+		defaultSource="[db]queryProfiles")
 	_allProfiles = ProfileListAttribute("allProfiles", 
 		default=base.getConfig("db", "maintainers"),
 		description="A (comma separated) list of profile names through"
-			" which the object can be written or administred (oh, and the"
-			" default is not admin, msdemlei but is the value of [db]maintainers)")
+			" which the object can be written or administred.",
+		defaultSource="[db]maintainers")
 
 
 class IVOMetaMixin(object):
@@ -194,7 +200,7 @@ class Registration(base.Structure, base.MetaMixin):
 	_servedThrough = base.ReferenceListAttribute("services",
 		description="A DC-internal reference to a service that lets users"
 			" query that within the data collection; tables with adql=True"
-			" are automatically declared to be servedBy the TAP service.")
+			" are automatically declared as isServiceFor the TAP service.")
 
 	# the following attribute is for compatibility with service.Publication
 	# in case someone manages to pass such a publication to the capability
@@ -237,7 +243,7 @@ class Registration(base.Structure, base.MetaMixin):
 		return False
 
 	def register(self):
-		"""adds servedBy and serviceFrom metadata to data, service pairs
+		"""adds isServiceFor and isServedBy metadata to data, service pairs
 		in this registration.
 		"""
 		if self.publishedForADQL():
