@@ -2,7 +2,7 @@
 Resource mixins.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -198,13 +198,8 @@ class MixinDef(activetags.ReplayBase):
 	def execMacro(self, macName, args):
 		if macName in self.macroExpansions:
 			return self.macroExpansions[macName]
-		try:
-			if self.parentMacroPackage:
-				return self.parentMacroPackage.execMacro(macName, args)
-		except base.MacroError:
-			raise base.MacroError(
-				"No macro \\%s available in this mixin or substrate."%(macName), 
-				macName)
+		if self.parentMacroPackage:
+			return self.parentMacroPackage.execMacro(macName, args)
 
 	def applyTo(self, destination, ctx, fillers={}):
 		"""replays the stored events on destination and arranges for processEarly
