@@ -3,10 +3,10 @@ A framework for regression tests within RDs.
 
 The basic idea is that there's small pieces of python almost-declaratively
 defining tests for a given piece of data.	These things can then be
-run while (or rather, after) executing gavo val.
+run while (or rather, after) executing dachs val.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -28,7 +28,6 @@ import time
 import threading
 import traceback
 import unittest
-import urllib.request, urllib.parse, urllib.error
 import urllib.parse
 
 try:
@@ -83,11 +82,19 @@ def getAuthFor(authKey):
 		base64.b64encode(b"%s:%s"%(user, pw))).strip()}
 
 
-def doHTTPRequest(method, host, path, query, 
+def doHTTPRequest(scheme, method, host, path, query, 
 		payload, headers, timeout):
 	"""creates the HTTP request and retrieves the result.
 	"""
-	conn = http.client.HTTPConnection(host, timeout=timeout)
+	try:
+		connClass = {
+			"http": http.client.HTTPConnection,
+			"https": http.client.HTTPSConnection}[scheme]
+	except KeyError:
+		raise base.ReportableError(
+			f"Unsupported scheme for regTest URL: {scheme}")
+
+	conn = connClass(host, timeout=timeout)
 	conn.connect()
 	try:
 		if query:
@@ -422,17 +429,17 @@ class DataURL(base.Structure):
 				headers["Content-Type"] = "application/x-www-form-urlencoded"
 
 		scheme, host, path, _, query, _ = urllib.parse.urlparse(str(self.httpURL))
-		assert scheme=="http"
 
 		if self.httpAuthKey is not base.NotGiven:
 			headers.update(getAuthFor(self.httpAuthKey))
-		status, respHeaders, content = doHTTPRequest(str(self.httpMethod),
+		status, respHeaders, content = doHTTPRequest(
+			scheme, str(self.httpMethod),
 			host, path, query, payload, headers, timeout)
 		
 		while self.httpHonorRedirects and status in [301, 302, 303]:
 			scheme, host, path, _, query, _ = urllib.parse.urlparse(
 				getHeaderValue(respHeaders, "location"))
-			status, respHeaders, content = doHTTPRequest("GET",
+			status, respHeaders, content = doHTTPRequest(scheme, "GET",
 				host, path, query, None, {}, timeout)
 
 		return status, respHeaders, content
@@ -648,7 +655,12 @@ class RegTest(procdef.ProcApp, unittest.TestCase):
 		el = res[0]
 		for key, val in assertions.items():
 			if key is None:
-				foundVal = el.text
+				try:
+					foundVal = el.text
+				except AttributeError:
+					# assume the expression was for an attribute and just use teh
+					# value
+					foundVal = el
 			else:
 				foundVal = el.attrib[key]
 			assert val==foundVal, "Trouble with %s: %s (%s, %s)"%(
