@@ -5,7 +5,7 @@ rowmakers are objects that take a dictionary of some kind and emit
 a row suitable for inclusion into a table.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -20,6 +20,7 @@ import traceback
 from gavo import base
 from gavo import utils
 from gavo.rscdef import common
+from gavo.rscdef import column
 from gavo.rscdef import procdef
 from gavo.rscdef import rmkfuncs
 from gavo.rscdef import rowtriggers
@@ -41,7 +42,7 @@ class MappedExpression(base.Structure):
 	destDict = None
 	restrictedMode = False
 
-	_dest = base.UnicodeAttribute("key", 
+	_dest = column.ColumnNameAttribute("key",
 		default=base.Undefined, 
 		description="Name of the column the value is to end up in.",
 		copyable=True, 
@@ -98,11 +99,6 @@ class MappedExpression(base.Structure):
 			raise base.StructureError("Map must have exactly one of source attribute"
 				" or element content")
 
-		if not utils.identifierPattern.match(self.key):
-			raise base.LiteralParseError("name", self.key,
-				hint="Var keys must be valid python"
-				" identifiers, and '%s' is not"%self.key)
-
 		if self.nullExpr is not base.NotGiven:
 			utils.ensureExpression(self.nullExpr)
 
@@ -112,40 +108,46 @@ class MappedExpression(base.Structure):
 		if self.nullExcs is not base.NotGiven:
 			utils.ensureExpression(self.nullExcs, "%s.nullExcs"%(self.name_))
 
+	def getAutoMapper(self):
+		"""returns an expression literal for turning what is self.source to a 
+		value suitable for self.key.
+
+		This must be defined for concrete classes derived from this.
+		"""
+		raise NotImplementedError("No automatic mappers defined here")
+
 	def getCode(self, columns):
 		"""returns python source code for this map.
 		"""
 		code = []
 
-		if self.content_:
-			code.append('%s["%s"] = %s'%(self.destDict, self.key, self.content_))
+		if isinstance(self.key, utils.QuotedName):
+			destIndex = '"%s"'%(self.key.name.replace('"', '\\"'))
 		else:
-			colDef = columns.getColumnByName(self.key)
-			try:
-				code.append('%s["%s"] = %s'%(self.destDict,
-					self.key, 
-					base.sqltypeToPythonCode(colDef.type)%'vars["%s"]'%
-						self.source.replace("\\", r"\\").replace('"', '\\"')))
-			except base.ConversionError:
-				raise base.ui.logOldExc(base.LiteralParseError("map", colDef.type,
-					hint="Auto-mapping to %s is impossible since"
-					" no default map for %s is known"%(self.key, colDef.type)))
+			destIndex = '"%s"'%self.key
+
+		if self.content_:
+			code.append('%s[%s] = %s'%(self.destDict, destIndex, self.content_))
+		else:
+			code.append('%s[%s] = %s'%(self.destDict,
+				destIndex,
+				self.getAutoMapper(columns)))
 
 		if self.nullExpr is not base.NotGiven:
-			code.append('\nif %s["%s"]==%s: %s["%s"] = None'%(
+			code.append('\nif %s[%s]==%s: %s[%s] = None'%(
 				self.destDict,
-				self.key, 
+				destIndex,
 				self.nullExpr, 
 				self.destDict,
-				self.key))
+				destIndex))
 		code = "".join(code)
 
 		if self.nullExcs is not base.NotGiven:
-			code = 'try:\n%s\nexcept (%s): %s["%s"] = None'%(
+			code = 'try:\n%s\nexcept (%s): %s[%s] = None'%(
 				re.sub("(?m)^", "  ", code), 
 				self.nullExcs, 
 				self.destDict,
-				self.key)
+				destIndex)
 		return code
 
 
@@ -168,6 +170,20 @@ class MapRule(MappedExpression):
 	name_ = "map"
 	destDict = "result"
 
+	def getAutoMapper(self, columns):
+		"""returns an expression to automatically map self.source to
+		a column in the destination table.
+		"""
+		colDef = columns.getColumnByName(self.key)
+		try:
+			return base.sqltypeToPythonCode(colDef.type)%(
+				'vars["{}"]'.format(
+					self.source.replace("\\", r"\\").replace('"', '\\"')))
+		except base.ConversionError:
+			raise base.ui.logOldExc(base.LiteralParseError("map", colDef.type,
+				hint="Auto-mapping to %s is impossible since"
+				" no default map for %s is known"%(self.key, colDef.type)))
+
 
 class VarDef(MappedExpression):
 	"""A definition of a rowmaker variable.
@@ -183,6 +199,16 @@ class VarDef(MappedExpression):
 	name_ = "var"
 	destDict = "vars"
 
+	def getAutoMapper(self, columns):
+		"""returns var[self.source].
+
+		Having source with var doesn't make a lot of sense, but it's
+		a nifty way to introduce None-s for missing keys if one wants.
+		And it should do *something*.
+		"""
+		return 'vars["{}"]'.format(
+	 		self.source.replace("\\", r"\\").replace('"', '\\"'))
+
 
 class ApplyDef(procdef.ProcApp):
 	"""A code fragment to manipulate the result row (and possibly more).
@@ -434,13 +460,13 @@ class RowmakerDef(base.Structure, RowmakerMacroMixin):
 				"  raise IgnoreThisRow(vars)",
 				line, "Checking ignore")
 		for v in self.vars:
-			line = appendToSource(v.getCode(columns), line, "assigning "+v.key)
+			line = appendToSource(v.getCode(columns), line, "assigning "+str(v.key))
 		for a in self.apps:
 			line = appendToSource(
 				"%s(vars, result, targetTable, _self)"%a.name, 
 				line, "executing "+a.name)
 		for m in self.maps:
-			line = appendToSource(m.getCode(columns), line, "building "+m.key)
+			line = appendToSource(m.getCode(columns), line, "building "+str(m.key))
 		return "\n".join(source), lineMap
 
 	def _getSource(self, tableDef):
