@@ -10,7 +10,7 @@ In the DaCHS, a resource descriptor typically sets up a schema in
 the database.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -550,7 +550,7 @@ class _UserConfigFakeRD(object):
 				pass
 			except Exception as msg:
 				base.ui.notifyError("Bad userconfig: (%s), ignoring it.  Run"
-					" 'gavo val %%' to see actual errors."%repr(msg))
+					" 'dachs val %%' to see actual errors."%repr(msg))
 
 			return base.caches.getRD("//userconfig"
 				).getById(id, forceType=forceType)
@@ -559,6 +559,17 @@ class _UserConfigFakeRD(object):
 				"etc/userconfig.rd")
 
 
+def refuseBlacklisted(srcId, rdInputPath):
+	"""raises an exception if rdInputPath ends with or srcId is equal to
+	one of the strings in [general]rdblacklist.
+	"""
+	for blacklistedName in base.getConfig("rdblacklist"):
+		if (rdInputPath.endswith(blacklistedName)
+				or blacklistedName==srcId):
+			raise utils.Error(
+				f"RD {srcId} is blacklisted in gavo.rc.  Not loading.")
+
+
 def getRD(srcId, doQueries=True, restricted=False, useRD=None):
 	"""returns a ResourceDescriptor for srcId.
 
@@ -580,6 +591,8 @@ def getRD(srcId, doQueries=True, restricted=False, useRD=None):
 		rd = useRD
 	
 	srcPath, inputFile = getRDInputStream(rd.sourceId)
+	refuseBlacklisted(srcId, srcPath)
+
 	# look for a context upstack and get the default parameters from there,
 	# overriding the parameters.
 	try:
@@ -599,11 +612,13 @@ def getRD(srcId, doQueries=True, restricted=False, useRD=None):
 
 	try:
 		rd = base.parseFromStream(rd, inputFile, context=getRD_context)
+		setRDDateTimes(getRD_context, rd, inputFile)
 	except Exception as ex:
 		ex.inFile = srcPath
 		ex.cacheable = getRD_context.failuresAreCacheable
 		raise
-	setRDDateTimes(getRD_context, rd, inputFile)
+	finally:
+		inputFile.close()
 	return rd
 
 
