@@ -21,24 +21,26 @@ Extensions to what the note says:
 		ephemeris used; currently, only JPL-DE200 and JPL-DE405 are allowed.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
-
-from pyparsing import (
-	Word, Literal, Optional, alphas, CaselessKeyword,
-		ZeroOrMore, OneOrMore, StringEnd,
-		Suppress, Forward, 
-		Regex, alphanums,
-		ParseException, ParseResults, 
-		ParseSyntaxException)
+import itertools
 
 from gavo import utils
 from gavo.stc import common
 from gavo.stc import stcsdefaults
 from gavo.stc import times
+from gavo.utils.parsetricks import (
+	Word, Literal, Optional, alphas, CaselessKeyword,
+		ZeroOrMore, OneOrMore, StringEnd,
+		Suppress, Forward, 
+		Regex, alphanums,
+		ParseException, ParseResults, 
+		ParseSyntaxException,
+		pyparsingWhitechars)
+
 
 class AComputedDefault(object):
 	"""A sentinel for computed default values.
@@ -144,6 +146,7 @@ def makeTree(parseResult):
 	"""
 	if not len(parseResult):  # empty parse results become Nones
 		res = None
+
 	elif list(parseResult.keys()):  # named children, generate a dict
 		res = {}
 		for k in list(parseResult.keys()):
@@ -153,11 +156,13 @@ def makeTree(parseResult):
 				v = makeTree(v)
 			if v is not None:  # discard empty branches
 				res[k] = v
+
 	else:                     # no named children, generate a list
 		if isinstance(parseResult[0], ParseResults):
 			res = [makeTree(child) for child in parseResult]
 		else:
 			res = list(parseResult)
+
 	return res
 
 
@@ -201,6 +206,22 @@ def _makeSingle(s, p, t):
 	return t[0]
 
 
+def _unpackTime(s, p, t):
+	"""unravels on level of list from t["coos"] or t["pos"].
+
+	We need this because of the way setResultsName works since pyparsing 2.4.
+	"""
+	if "coos" in t:
+		coos = t.pop("coos")
+		for r in itertools.chain(*coos):
+			t["coos"] = r
+	if "pos" in t:
+		coos = t.pop("pos")
+		for r in itertools.chain(*coos):
+			t["pos"] = r
+	return None # (changes in place)
+
+
 def _getSTCSGrammar(numberLiteral, 
 		timeLiteral, 
 		_exportAll=False,
@@ -215,7 +236,7 @@ def _getSTCSGrammar(numberLiteral,
 	_addGeoReferences lets you write quoted references to vectors
 	(like Circle "center" 20.).
 	"""
-	with utils.pyparsingWhitechars("\n\t\r "):
+	with pyparsingWhitechars("\n\t\r "):
 	
 		number = numberLiteral
 		del numberLiteral
@@ -230,7 +251,7 @@ def _getSTCSGrammar(numberLiteral,
 		spectralUnit = _unitOpener - Regex(_reFromKeys(spectralUnits))("unit")
 		redshiftUnit = _unitOpener - ( 
 			(_spaceUnitWord + "/" + _timeUnitWord).addParseAction(_stringify) 
-			| CaselessKeyword("nil") )("unit")
+			| CaselessKeyword("nil"))("unit")
 		velocityUnit = _unitOpener - (OneOrMore( 
 			(_spaceUnitWord + "/" + _timeUnitWord).addParseAction(_stringify) 
 			).addParseAction(_stringifyBlank))("unit")
@@ -274,10 +295,10 @@ def _getSTCSGrammar(numberLiteral,
 			) + OneOrMore( number )
 		size = Suppress( CaselessKeyword("Size") ) + OneOrMore(number)
 		pixSize = Suppress( CaselessKeyword("PixSize") ) + OneOrMore(number)
-		cooProps = (Optional( error("error") ) 
-			+ Optional( resolution("resolution") ) 
-			+ Optional( size("size") ) 
-			+ Optional( pixSize("pixSize") ))
+		cooProps = (Optional(error("error")) 
+			+ Optional(resolution("resolution")) 
+			+ Optional(size("size")) 
+			+ Optional(pixSize("pixSize")))
 
 # properties of most spatial specs
 		_coos = ZeroOrMore( number )("coos")
@@ -289,7 +310,7 @@ def _getSTCSGrammar(numberLiteral,
 			_pos = complexColRef("pos") | _pos
 		positionSpec = Suppress( CaselessKeyword("Position") ) + _pos
 		epochSpec = Suppress( CaselessKeyword("Epoch") ) - astroYear
-		_spatialProps = Optional( spaceUnit ) + cooProps
+		_spatialProps = Optional(spaceUnit) + cooProps
 		velocitySpec = (CaselessKeyword("Velocity")("type")
 			 + OneOrMore( number )("pos"))
 		velocityInterval = (
@@ -297,8 +318,8 @@ def _getSTCSGrammar(numberLiteral,
 				CaselessKeyword("VelocityInterval")("type") 
 				+ Optional( fillfactor ) 
 				+ _coos )
-			+ Optional( velocitySpec ) 
-			+ Optional( velocityUnit ) 
+			+ Optional(velocitySpec) 
+			+ Optional(velocityUnit) 
 			+ cooProps).addParseAction(makeTree)
 		_spatialTail = (_spatialProps + 
 			Optional( velocityInterval)("velocity"))
@@ -311,22 +332,27 @@ def _getSTCSGrammar(numberLiteral,
 # times and time intervals
 		timescale = (Regex("|".join(common.stcTimeScales)))("timescale")
 		timephrase = Suppress( CaselessKeyword("Time") ) + timeLiteral
-		_commonTimeItems = Optional( timeUnit ) + cooProps
+		_commonTimeItems = Optional(timeUnit) + cooProps
 		_intervalOpener = ( Optional( fillfactor ) + 
 			Optional( timescale("timescale") ) +
 			Optional( refpos ) )
-		_intervalCloser = Optional( timephrase("pos") ) + _commonTimeItems
+		_intervalCloser = (Optional(timephrase.setResultsName("pos", True)) 
+			+ _commonTimeItems)
 
 		timeInterval =  (CaselessKeyword("TimeInterval")("type") + 
-			_intervalOpener + ZeroOrMore( timeLiteral )("coos") + 
-			_intervalCloser)
+			_intervalOpener + ZeroOrMore(timeLiteral.setResultsName("coos", True))
+			+ _intervalCloser).addParseAction(_unpackTime)
 		startTime = (CaselessKeyword("StartTime")("type") + _intervalOpener + 
-			timeLiteral.setResultsName("coos", True) + _intervalCloser)
+			timeLiteral.setResultsName("coos", True) + _intervalCloser
+			).addParseAction(_unpackTime)
 		stopTime = (CaselessKeyword("StopTime")("type") + _intervalOpener + 
-			timeLiteral.setResultsName("coos", True) + _intervalCloser)
-		time = (CaselessKeyword("Time")("type")  + Optional( timescale("timescale") ) + 
-			Optional( refpos ) + Optional(
-				timeLiteral.setResultsName("pos", True) ) + _commonTimeItems)
+			timeLiteral.setResultsName("coos", True) + _intervalCloser
+			).addParseAction(_unpackTime)
+		time = (CaselessKeyword("Time")("type")
+			+ Optional(timescale("timescale")) 
+			+ Optional(refpos) + Optional(
+				timeLiteral.setResultsName("pos", True).addParseAction(_unpackTime)) 
+			+ _commonTimeItems)
 		timeSubPhrase = (timeInterval 
 			| startTime 
 			| stopTime 
@@ -402,7 +428,7 @@ def _getSTCSGrammar(numberLiteral,
 # spectral subphrase
 		spectralSpec = (Suppress( CaselessKeyword("Spectral") ) 
 			+ number)("pos")
-		_spectralTail = Optional( spectralUnit ) + cooProps
+		_spectralTail = Optional(spectralUnit) + cooProps
 		spectralInterval = (CaselessKeyword("SpectralInterval")("type") 
 			+ Optional( fillfactor ) 
 			+ Optional( refpos ) 
@@ -420,19 +446,19 @@ def _getSTCSGrammar(numberLiteral,
 		redshiftType = Regex("VELOCITY|REDSHIFT")("redshiftType")
 		redshiftSpec = (Suppress( CaselessKeyword("Redshift") ) + number)("pos")
 		dopplerdef = Regex("OPTICAL|RADIO|RELATIVISTIC")("dopplerdef")
-		_redshiftTail = Optional( redshiftUnit ) + cooProps
+		_redshiftTail = Optional(redshiftUnit) + cooProps
 		redshiftInterval = (CaselessKeyword("RedshiftInterval")("type") 
-			+ Optional( fillfactor ) 
-			+ Optional( refpos ) 
-			+ Optional( redshiftType ) 
-			+ Optional( dopplerdef ) 
+			+ Optional(fillfactor) 
+			+ Optional(refpos) 
+			+ Optional(redshiftType) 
+			+ Optional(dopplerdef) 
 			+ _coos 
-			+ Optional( redshiftSpec ) 
+			+ Optional(redshiftSpec) 
 			+ _redshiftTail)
 		redshift = (CaselessKeyword("Redshift")("type") 
-			+ Optional( refpos ) 
-			+ Optional( redshiftType ) 
-			+ Optional( dopplerdef ) 
+			+ Optional(refpos) 
+			+ Optional(redshiftType) 
+			+ Optional(dopplerdef) 
 			+ _pos 
 			+ _redshiftTail)
 		redshiftSubPhrase = (redshiftInterval | redshift).addParseAction(
@@ -460,7 +486,7 @@ def getSymbols(_exportAll=False, _colrefLiteral=None,
 		_addGeoReferences=False):
 	"""returns an STC-S grammar with terminal values.
 	"""
-	with utils.pyparsingWhitechars("\n\t\r "):
+	with pyparsingWhitechars("\n\t\r "):
 		_exactNumericRE = r"[+-]?\d+(\.(\d+)?)?|[+-]?\.\d+"
 		exactNumericLiteral = Regex(_exactNumericRE)
 		numberLiteral = Regex(r"(?i)(%s)(E[+-]?\d+)?"%_exactNumericRE
@@ -498,7 +524,7 @@ def getColrefSymbols():
 	"""
 	def makeColRef(s, p, toks):
 		return common.ColRef(toks[0][1:-1])
-	with utils.pyparsingWhitechars("\n\t\r "):
+	with pyparsingWhitechars("\n\t\r "):
 		atomicColRef = Regex('"[A-Za-z_][A-Za-z_0-9]*"').addParseAction(
 			makeColRef)
 	return getSymbols(_colrefLiteral=atomicColRef, _addGeoReferences=True)
