@@ -22,7 +22,7 @@ dates), parsing is done by the specific PQLPar types (fromLiteral).  See
 the PQLPar docstring for further info.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -653,10 +653,10 @@ def _getShPatGrammar():
 	This is different from fnmatch.translate in that it handles escaping
 	correctly.
 	"""
-	from pyparsing import (
-		Literal, Regex, CharsNotIn, ZeroOrMore, QuotedString)
+	from gavo.utils.parsetricks import (
+		Literal, Regex, CharsNotIn, ZeroOrMore, QuotedString, pyparsingWhitechars)
 
-	with utils.pyparsingWhitechars(""):
+	with pyparsingWhitechars(""):
 		enumChars = QuotedString(quoteChar="[", endQuoteChar="]", escChar="\\"
 			).addParseAction(_mungeEnumSequence)
 		noEnum = Literal("[").addParseAction(lambda s, p, t: "\\[")
