@@ -5,7 +5,7 @@ See the __init__.py docstring for a bit more on the general
 architecture.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -70,7 +70,8 @@ class Publication(base.Structure, base.ComputedMetaMixin):
 			" capability corresponding to this publication.  This is"
 			" mainly when there is a vs:WebBrowser service accompanying a VO"
 			" protocol service, and this other service should be published"
-			" in the same resource record.  See also the operator's guide.",
+			" in the same resource record; you would say something like"
+			' ``<publish render="form" sets="..." service="web"/>``.',
 			copyable="True")
 	_auxiliary = base.BooleanAttribute("auxiliary", default=False,
 		description="Auxiliary publications are for capabilities"
@@ -318,9 +319,11 @@ class Service(base.Structure, base.ComputedMetaMixin,
 	  all columns from the underlying database table with low enough
 	  verbLevel (essentially).  When this property is "True" (case-sensitive),
 		that's not done and only the service's output table is evaluated.
-	* superseded-url -- if you leave a "carcass" service around, you can
 	  set this property to the URL of its successor.  This will produce
 	  a 404 page with a link to the new service.
+	* fixedFormat -- if this is set to "True", the form renderer will
+	  not produce an output format selector (and we shouldn't produce
+	  DALI RESPONSEFORMAT metadata).
 	"""
 	name_ = "service"
 
@@ -452,10 +455,12 @@ class Service(base.Structure, base.ComputedMetaMixin,
 						" in the logs."))
 			self.customPageCode = page, (os.path.basename(self.customPage),moddesc)
 
-	def getTemplate(self, key):
+	def getTemplate(self, key, reload=False):
 		"""returns the nevow template for the function key on this service.
+
+		Pass reload to force a reload of templates already parsed.
 		"""
-		if key not in self._loadedTemplates:
+		if reload or key not in self._loadedTemplates:
 			tp = self.templates[key]
 			if tp.startswith("//"):
 				self._loadedTemplates[key] = common.loadSystemTemplate(tp[2:])
@@ -465,13 +470,18 @@ class Service(base.Structure, base.ComputedMetaMixin,
 		return self._loadedTemplates[key]
 
 	def getUWS(self):
-		"""returns a user UWS instance for this service.
+		"""returns the UWS worker system for this service.
 
-		This is a service for the UWSAsyncRenderer.
+		This is a service for the DALIAsyncRenderer.
 		"""
 		if not hasattr(self, "uws"):
-			from gavo.protocols import useruws
-			self.uws = useruws.makeUWSForService(self)
+			if hasattr(self.core, "workerSystem"):
+				self.uws = self.core.workerSystem
+			else:
+				# todo: we don't want that any more.  People wanting user UWS
+				# should attach their worker systems to their cores in the future.
+				from gavo.protocols import useruws
+				self.uws = useruws.makeUWSForService(self)
 		return self.uws
 
 	################### Registry and related methods.
@@ -501,7 +511,7 @@ class Service(base.Structure, base.ComputedMetaMixin,
 		the resType meta item.
 		"""
 		if (self.core.outputTable.columns 
-			or "tap" in self.allowed):
+			or "dali" in self.allowed):
 			self.resType = "catalogService"
 		else: # no output table defined, we're a plain service
 			self.resType = "nonTabularService"
@@ -535,7 +545,7 @@ class Service(base.Structure, base.ComputedMetaMixin,
 			if self.core.queriedTable.adql:
 				tapService = base.resolveCrossId("//tap#run") 
 				yield base.makeStruct(Publication, 
-						render="tap", 
+						render="dali",
 						sets=vosiSet,
 						auxiliary=True, 
 						service=tapService,
@@ -577,7 +587,7 @@ class Service(base.Structure, base.ComputedMetaMixin,
 			self._publications.feedObject(self, pub)
 
 	  # This is no longer a good place to do this, but TAP-published
-		# tables need a servedBy, and we since it's automatic, let's
+		# tables need a isServedBy, and we since it's automatic, let's
 		# do it here.
 		# According to the "discovering dependent" note, we don't
 		# do the reverse relationship lest the TAP service
@@ -585,7 +595,7 @@ class Service(base.Structure, base.ComputedMetaMixin,
 		if isinstance(self.core, core.getCore("dbCore")):
 			if self.core.queriedTable.adql:
 				tapService = base.resolveCrossId("//tap#run")
-				self.addMeta("servedBy", 
+				self.addMeta("isServedBy", 
 					base.getMetaText(tapService, "title"),
 					ivoId=base.getMetaText(tapService, "identifier"))
 
@@ -615,7 +625,7 @@ class Service(base.Structure, base.ComputedMetaMixin,
 			if svcRef:
 				yield self.rd.getById(svcRef)
 
-	def getPublicationsForSet(self, names):
+	def getPublicationsForSet(self, names, includeDatalink=True):
 		"""returns publications for a set of set names (the names argument).
 
 		In the special case names=None, all allowed renderers are treated
@@ -634,29 +644,35 @@ class Service(base.Structure, base.ComputedMetaMixin,
 
 		# for ivo_managed, also return a datalink endpoints if they're
 		# there; the specs imply that might be useful some day.
-		for dlSvc in self._iterAssociatedDatalinkServices():
-			if "dlget" in dlSvc.allowed:
-				result.append(base.makeStruct(Publication,
-					render="dlget",
-					sets=set(["ivo_managed"]),
-					service=dlSvc))
+		# (btw., I think we shouldn't do that but instead extend
+		# publish so people can ask for other services to be included
+		# as capabilities).
+		if includeDatalink and "ivo_managed" in names:
+			for dlSvc in self._iterAssociatedDatalinkServices():
+				if "dlget" in dlSvc.allowed:
+					result.append(base.makeStruct(Publication,
+						render="dlget",
+						sets=set(["ivo_managed"]),
+						service=dlSvc))
 
-			if "dlasync" in dlSvc.allowed:
-				result.append(base.makeStruct(Publication,
-					render="dlasync",
-					sets=set(["ivo_managed"]),
-					service=dlSvc))
+				if "dlasync" in dlSvc.allowed:
+					result.append(base.makeStruct(Publication,
+						render="dlasync",
+						sets=set(["ivo_managed"]),
+						service=dlSvc))
 
-			if "dlmeta" in dlSvc.allowed:
-				result.append(base.makeStruct(Publication,
-					render="dlmeta",
-					sets=set(["ivo_managed"]),
-					service=dlSvc))
+				if "dlmeta" in dlSvc.allowed:
+					result.append(base.makeStruct(Publication,
+						render="dlmeta",
+						sets=set(["ivo_managed"]),
+						service=dlSvc))
 
 		if addAllAllowed: # name=None was passed in,
 			# add publications for everything allowed but not yet marked
 			# as published (for VOSI)
-			alreadyPublished = set(p.render for p in result)
+			alreadyPublished = set(p.render for p in result
+				) | frozenset(["sync", "async"]) # these never get capabilities
+
 			for rendName in self.allowed:
 				if not rendName in alreadyPublished:
 					result.append(base.makeStruct(Publication,
@@ -727,43 +743,15 @@ class Service(base.Structure, base.ComputedMetaMixin,
 		"""returns a list of table definitions that have something to do with
 		this service.
 
-		This is for VOSI-type requests.  Usually, that's just the core's
-		queried table or an output table, except when there is a TAP renderer on
-		the service.
+		This is for VOSI-type requests.
 
-		All this is a bit heuristic; but then again, there's no rigorous 
-		definition for what's to be in a tables endpoint either.
+		Basically, we're leaving the decision to the core, except when
+		we have an output Table of our own.
 		"""
-		tables = []
+		if self.outputTable and self.outputTable.columns:
+			return [self.outputTable]
 
-		qt = getattr(self.core, "queriedTable", None)
-		if qt is None or not qt.columns:
-			qt = self.core.outputTable
-			if qt is None or not qt.columns:
-				if self.outputTable and self.outputTable.columns:
-					qt = self.outputTable
-
-		if qt is not None:
-			tables.append(qt)
-
-		# XXX TODO: This stinks big time.  It's because we got TAP factorization
-		# wrong.  Sync and async should be renderers, and there should
-		# be a core that then could say this kind of thing.  That's not
-		# yet the case, so:
-		if "tap" in self.allowed:
-			# tap never has "native" tables, so start afresh
-			tables = []
-
-			with base.getTableConn() as conn:
-				for tableName, in conn.query("SELECT table_name"
-						" FROM TAP_SCHEMA.tables"):
-					try:
-						tables.append(base.getTableDefForTable(conn, tableName))
-					except:
-						base.ui.notifyError("Failure trying to retrieve table definition"
-							" for table %s.  Please fix the corresponding RD."%tableName)
-
-		return [t for t in tables if t is not None and t.rd is not None]
+		return self.core.getRelevantTables()
 
 	def declareServes(self, data):
 		"""adds meta to self and data indicating that data is served by
@@ -772,10 +760,10 @@ class Service(base.Structure, base.ComputedMetaMixin,
 		This is used by table/@adql and the publish element on data.
 		"""
 		if data.registration:
-			self.addMeta("serviceFor", 
+			self.addMeta("isServiceFor", 
 				base.getMetaText(data, "title", default="Anonymous"),
 				ivoId=base.getMetaText(data, "identifier"))
-			data.addMeta("servedBy", 
+			data.addMeta("isServedBy", 
 				base.getMetaText(self, "title"),
 				ivoId=base.getMetaText(self, "identifier"))
 
@@ -834,7 +822,6 @@ class Service(base.Structure, base.ComputedMetaMixin,
 
 		return rscdef.ColumnList(fields)
 
-
 	def getHTMLOutputFields(self, queryMeta, ignoreAdditionals=False,
 			raiseOnUnknown=True):
 		"""returns a list of OutputFields suitable for an HTML response described
@@ -880,7 +867,17 @@ class Service(base.Structure, base.ComputedMetaMixin,
 		structure of the tables passed.
 		"""
 		queryMeta = queryMeta or common.emptyQueryMeta
-		format = queryMeta.get("format", "HTML")
+
+		# "renderer" in queryMeta needs to be set in the run method below;
+		# the net effect is that format will come from RESPONSEFORMAT, _FORMAT,
+		# and the renderer default in sequence.  If none of that is available,
+		# blindly go for default HTML (which in retrospect perhaps wasn't the
+		# best choice for DaCHS, but changing this now to VOTable would introduce
+		# needless breakage).
+		renderer = queryMeta.get("renderer")
+		format = queryMeta.get("format", 
+			getattr(renderer, "defaultOutputFormat", None)) or "HTML"
+
 		if format in self.htmlLikeFormats:
 			return self.getHTMLOutputFields(queryMeta, raiseOnUnknown=raiseOnUnknown)
 		else:
@@ -1046,7 +1043,8 @@ class Service(base.Structure, base.ComputedMetaMixin,
 		coreArgs = inputdef.CoreArgs.fromRawArgs(
 			core.inputTable, args, self.getContextGrammarFor(
 				renderer, queryMeta, core))
-
+		queryMeta["renderer"] = renderer
+	
 		return self._runWithInputTable(core, coreArgs, queryMeta)
 
 
