@@ -6,7 +6,7 @@ parameters in the doc rec) and returns an output thing, usually a table
 (but see the src package docstring).
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -416,6 +416,9 @@ class TableBasedCore(core.Core):
 		else:
 			return core.Core.adaptForRenderer(self, renderer, queryMeta)
 
+	def getRelevantTables(self):
+		return [self.queriedTable]
+
 
 class FancyQueryCore(TableBasedCore, base.RestrictionMixin):
 	"""A core executing a pre-specified query with fancy conditions.
