@@ -2,7 +2,7 @@
 Streaming out large computed things using twisted and threads.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -50,12 +50,6 @@ class DataStreamer(threading.Thread):
 
 
 	def __init__(self, writeStreamTo, consumer, queryMeta=None):
-		# we play the renderer for whatever we're running.  Hence, we need
-		# to have the HANDLING_HTTPS attribute that used code looks for
-		# upstack to determine whether to make https URLs (yes, I'd use
-		# a different basic design if I had to do this again).
-		self.HANDLING_HTTPS = consumer.isSecure()
-	
 		threading.Thread.__init__(self)
 		self.writeStreamTo, self.consumer = writeStreamTo, consumer
 		self.queryMeta = queryMeta
@@ -66,6 +60,12 @@ class DataStreamer(threading.Thread):
 		self.setDaemon(True) # kill transfers on server restart
 		self.buffer = utils.StreamBuffer()
 
+	def isSecure(self):
+		"""we're standing in as a request for the streaming code, so
+		we have to tell it whether or not to produce https.
+		"""
+		return self.consumer.isSecure()
+
 	def _abortProducing(self, res):
 		# this errback is called when the request is finishing after an error.  The
 		# request, here, still is there as self.consumer; this code runs in the
@@ -142,7 +142,7 @@ class DataStreamer(threading.Thread):
 		# Must be callFromThread'ed
 		self.exceptionToRaise = None
 		self.join(0.01)
-		if self.isAlive():
+		if self.is_alive():
 			base.ui.notifyError("Streaming thread couldn't be joined?")
 
 		if self.connectionLive:
