@@ -2,7 +2,7 @@
 Classes and methods to support vizier-type specifications on fields.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -11,9 +11,6 @@ Classes and methods to support vizier-type specifications on fields.
 import datetime
 import re
 
-from pyparsing import (Word, Literal, Optional, Forward,
-	ZeroOrMore, Suppress, ParseException, StringEnd, Regex,
-	OneOrMore, CharsNotIn)
 
 from gavo import base
 from gavo import stc
@@ -21,7 +18,9 @@ from gavo import utils
 from gavo.base import literals
 from gavo.base import sqlmunge
 from gavo.base import typesystems
-
+from gavo.utils.parsetricks import (Word, Literal, Optional, Forward,
+	ZeroOrMore, Suppress, ParseException, StringEnd, Regex,
+	OneOrMore, CharsNotIn, pyparsingWhitechars)
 
 class ParseNode(object):
 	"""is a parse node, consisting of an operator and children.
@@ -338,7 +337,7 @@ def getComplexGrammar(baseLiteral, pmBuilder, errorLiteral=None,
 	if errorLiteral is None:
 		errorLiteral = baseLiteral
 
-	with utils.pyparsingWhitechars(" \t"):
+	with pyparsingWhitechars(" \t"):
 		preOp = Literal("=") |  Literal(">=") | Literal(">"
 			) | Literal("<=") | Literal("<")
 		rangeOp = Literal("..")
@@ -416,7 +415,7 @@ def parseDateTime(s, pos, tok):
 
 floatLiteral = Regex(utils.floatRE).addParseAction(parseFloat)
 
-_DATE_REGEX = r"\d\d\d\d-\d\d-\d\d(T\d\d:\d\d:\d\d)?|(\d+(\.\d*)?)"
+_DATE_REGEX = r"\d\d\d\d-\d\d-\d\d(T\d\d:\d\d:\d\d(\.\d*)?)?|(\d+(\.\d*)?)"
 _DATE_LITERAL_DT = Regex(_DATE_REGEX).addParseAction(parseDateTime)
 
 
@@ -453,7 +452,7 @@ def getStringGrammar():
 	"""returns a grammar for parsing vizier-like string expressions.
 	"""
 # XXX TODO: should we cut at =| (which is currently parsed as = |)?
-	with utils.pyparsingWhitechars(" \t"):
+	with pyparsingWhitechars(" \t"):
 		simpleOperator = Literal("==") | Literal("!=") | Literal(">=") |\
 			Literal(">") | Literal("<=") | Literal("<") | Literal("=~") |\
 			Literal("=,")
@@ -615,17 +614,17 @@ def getPlaceholderFor(inputKey, values):
 
 	if inputKey.type=="vexpr-float":
 		scaling = inputKey.scaling or 1
-		return format_placeholder(values.min, values.max,
+		return format_placeholder(values.min_typed, values.max_typed,
 			lambda val: "%s"%(val/scaling))
 
 	elif inputKey.type=="vexpr-mjd":
 		# date with underlying MJD column
-		return format_placeholder(values.min, values.max,
+		return format_placeholder(values.min_typed, values.max_typed,
 			lambda val: "%s"%utils.formatISODT(stc.mjdToDateTime(val)))
 
 	elif inputKey.type=="vexpr-date":
 		# date with underlying timestamp column
-		return format_placeholder(values.min, values.max,
+		return format_placeholder(values.min_typed, values.max_typed,
 			lambda val: "%s"%utils.formatISODT(val))
 
 	# fall through to None if no placeholder can be make
