@@ -2,7 +2,7 @@
 DC administration interface.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -184,7 +184,10 @@ def suggestucds(querier, args):
 	def getMatches(description):
 		res = utils.urlopenRemote(apiURL, {"description": description})
 		data, metadata = votable.load(res)
-		return list(metadata.iterDicts(data))
+		if metadata:
+			return list(metadata.iterDicts(data))
+		else:
+			return []
 	
 	td = api.getReferencedElement(args.tableId, forceType=api.TableDef)
 	for col in td:
@@ -241,18 +244,27 @@ def dumpDF(querier, args):
 		raise base.ReportableError("No such distribution file: %s\n"%args.path)
 
 
-@exposedFunction([Arg(help="XML file", dest="path")],
+@exposedFunction([Arg(help="XML file", dest="path", nargs='+')],
 	help="Validate a file against built-in VO schemas and with built-in"
 		" schema validator.")
 def xsdValidate(querier, args):
 	from gavo.helpers import testtricks
-	msgs = testtricks.getXSDErrors(open(args.path, "rb").read())
-	if not msgs:
-		print("-- valid")
-		return 0
-	else:
-		print(msgs)
-		return 1
+	rtval = 0
+
+	for path in args.path:
+		print(path, end=" -- ")
+		try:
+			with open(path, "rb") as f:
+				msgs = testtricks.getXSDErrors(f.read())
+		except Exception as ex:
+			msgs = [str(ex)]
+		if not msgs:
+			print("valid")
+		else:
+			print(msgs)
+			rtval = 1
+
+	return rtval
 
 
 @exposedFunction([Arg(help="IVOID to mark as deleted", dest="ivoid")],
