@@ -6,7 +6,7 @@ as possible.  Take care to give sensible error messages -- much can go wrong
 here, and it's nice if the user has a way to figure out what's wrong.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -441,7 +441,8 @@ def parseCommandLine():
 		" the future DaCHS database.  The DSN must let DaCHS connect"
 		" to the DB as an administrator.  dbname, host, and port"
 		" get copied to the profile, if given.  The DSN looks roughly like"
-		' "host=foo.bar user=admin password=secret". If you followed the'
+		' "host=foo.bar user=admin password=secret dbname=gavo".'
+		' If you followed the'
 		" installation instructions, you don't need this option.",
 		action="store", type=str, dest="dsn", default="gavo")
 	parser.add_argument("--nodb", help="Inhibit initialization of the"
