@@ -12,7 +12,7 @@ commented RD templates, fills out a thing or two and leaves the rest to
 the operator.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -40,6 +40,18 @@ from gavo.utils import fitstools
 MS = base.makeStruct
 
 
+FT_TYPE_MAP = {
+	"L": "boolean",
+	"B": "bytea",
+	"A": "text",
+	"I": "smallint",
+	"J": "integer",
+	"K": "bigint",
+	"E": "real",
+	"D": "double precision",
+}
+
+
 # ======================= Begin deprecated ui and implementation ===========
 
 ignoredFITSHeaders = set(["COMMENT", "SIMPLE", "BITPIX", "EXTEND", 
@@ -91,26 +103,16 @@ def structToETree(aStruct):
 	return nodeStack[-1]
 
 
-FT_TYPE_MAP = {
-	"B": "bytea",
-	"A": "text",
-	"I": "smallint",
-	"J": "integer",
-	"K": "bigint",
-	"E": "real",
-	"D": "double precision",
-}
-
-def getTypeForFTColumn(fitsCol):
+def getTypeForFTFormat(format, name):
 	"""returns a DaCHS type for FITS table column.
 
 	This currently ignores array sizes and such.  Well, people can always
 	fix things manually.
 	"""
-	mat = re.match("(\d*)(.)$", fitsCol.format)
+	mat = re.match("(\d*)(.)$", format)
 	if not mat or not mat.group(2) in FT_TYPE_MAP:
 		raise base.ReportableError("FITS type code '%s' of %s not handled"
-			" by gavo mkrd; add handling if you can."%(fitsCol.format, fitsCol.name))
+			" by gavo mkrd; add handling if you can."%(format, name))
 	return FT_TYPE_MAP[mat.group(2)]
 
 
@@ -119,12 +121,13 @@ def makeTableFromFT(rd, srcName, opts):
 	cols, nameMaps = [], {}
 	nameMaker = base.VOTNameMaker()
 
-	for fitsCol in pyfits.open(srcName)[1].columns:
+	hdus = pyfits.open(srcName)
+	for fitsCol in hdus[1].columns:
 		destName = nameMaker.makeName(fitsCol)
 		if destName!=fitsCol.name:
 			nameMaps[destName] = fitsCol.name
 		cols.append(MS(rscdef.Column,
-			type=getTypeForFTColumn(fitsCol),
+			type=getTypeForFTFormat(fitsCol.format, fitsCol.name),
 			name=destName,
 			unit=fitsCol.unit,
 			ucd="",
@@ -132,6 +135,7 @@ def makeTableFromFT(rd, srcName, opts):
 	table = rscdef.TableDef(rd, id=opts.tableName, onDisk=True,
 		columns=cols)
 	table.nameMaps = nameMaps
+	hdus.close()
 	return table
 
 
@@ -160,7 +164,8 @@ def makeDataForFT(rd, srcName, opts):
 def makeTableFromFITS(rd, srcName, opts):
 	keyMappings = []
 	table = rscdef.TableDef(rd, id=opts.tableName, onDisk=True)
-	headerCards = fitstools.openFits(srcName)[0].header.cards
+	hdus = fitstools.openFits(srcName)
+	headerCards = hdus[0].header.cards
 	for index, card in enumerate(headerCards):
 		if isIgnoredKeyword(card.keyword):
 			continue
@@ -180,6 +185,7 @@ def makeTableFromFITS(rd, srcName, opts):
 			description=card.comment))
 		keyMappings.append((colName, card.keyword))
 	rd.setProperty("mapKeys", ", ".join("%s:%s"%(v,k) for k,v in keyMappings))
+	hdus.close()
 	return table.finishElement()
 
 
@@ -335,6 +341,284 @@ def main():
 # ======================= End deprecated ui and implementation ===========
 
 
+def iterColAttrsFITS(table):
+	"""yields dicts of column attributes from a FITS binary table HDU.
+	"""
+	hdr = table.header
+	for colInd in range(1, hdr["TFIELDS"]+1):
+		colMeta = [
+			("name", hdr[f"TTYPE{colInd}"]),
+			("type", getTypeForFTFormat(
+				hdr[f"TFORM{colInd}"], hdr[f"TTYPE{colInd}"])),
+			("ucd", hdr.get(f"TUCD{colInd}", None)),
+			("utype", hdr.get(f"TUTYP{colInd}", None)),
+			("unit", hdr.get(f"TUNIT{colInd}", None)),
+			("description", hdr.get(f"TCOMM{colInd}", None
+				) or hdr.cards[f"TTYPE{colInd}"].comment or ""),
+			("verbLevel", 1),]
+
+		yield dict((k,v) for k,v in colMeta if v is not None)
+
+
+def iterColAttrsVOTable(inFile):
+	"""tries to parse (binary) inFile as a VOTable and yields colAttrs 
+	dictionaries for all FIELDs in it.
+	"""
+	for el in votable.parse(inFile, watchset=[votable.V.FIELD]):
+		if isinstance(el, votable.V.FIELD):
+			colMeta = [
+				("name", el.name),
+				("unit", el.unit),
+				("ucd", el.ucd or ""),
+				("utype", el.utype),
+				("type", base.voTableToSQLType(el.datatype, el.arraysize, el.xtype)),
+				("description", el.getDescription()),
+				("verbLevel", 1),]
+			yield dict((k,v) for k,v in colMeta if v is not None)
+
+
+class VizBBBParser(object):
+	"""A simple and naive parser for VizieR Byte-by-Byte descriptions.
+
+	The results are available in the colAttrs and bytesForCols attributes.
+	"""
+	bbbLinePat = re.compile(
+		r"\s*(?P<from>\d+)\s*-\s*(?P<to>\d+)"
+		r"\s+(?P<format>[A-Z0-9.]+)"
+		r"\s+(?P<units>[^\s]*)"
+		r"\s+(?P<label>[^\s]*)"
+		r"\s+(?P<description>.*)")
+	tableBorderPat = re.compile("--------+")
+
+	def __init__(self):
+		self.state = "scanning"
+		self.infoSoFar = {}
+		self.colAttrs = []
+		self.bytesForCols = []
+
+	def _parse_finished(self, ln):
+		"""we've found and parsed the BBB table.
+		"""
+		pass
+	
+	def _parse_scanning(self, ln):
+		"""we're waiting for the table header of the first BBB block.
+		"""
+		if ln.startswith("Byte-by-byte Description"):
+			self.state = "skip_header"
+	
+	def _parse_skip_header(self, ln):
+		"""we're skipping over the table header of the first BBB block.
+		"""
+		mat = self.bbbLinePat.match(ln)
+		if mat:
+			self.state = "bbb_colhead"
+			self._parse_bbb_colhead(ln, mat)
+
+		elif self.tableBorderPat.match(ln) or re.match("\s*Bytes", ln):
+			pass
+
+		else:
+			raise Exception(f"Expected BBB table, got '{ln}'")
+
+	def _parse_bbb_colhead(self, ln, mat=None):
+		"""we're expecting either a BBB column or the end of the table.
+		"""
+		if mat is None:
+			mat = self.bbbLinePat.match(ln)
+
+		if not mat:
+			if self.tableBorderPat.match(ln):
+				self.state = "finished"
+			else:
+				raise Exception("Expected BBB column def, got '{ln}'")
+
+		self.infoSoFar = mat.groupdict()
+		self.state = "bbb_moreexpl"
+	
+	def _parse_bbb_moreexpl(self, ln):
+		"""we're expecting either a BBB column, a continuation line for the
+		explanation, or the end of the table.
+
+		In this state, there's non-shipped column data in infoSoFar.
+		"""
+		mat = self.bbbLinePat.match(ln)
+		if mat:
+			self._shipout()
+			self.state = "bbb_colhead"
+			self._parse_bbb_colhead(ln, mat)
+
+		elif self.tableBorderPat.match(ln):
+			self._shipout()
+			self.state = "finished"
+	
+		else:
+			self.infoSoFar["description"] += (" "+ln.strip())
+
+	def _getTypeForTypeCode(self, typeCode):
+		"""returns one of our SQL types for a VizieR BBB type code.
+		"""
+		baseType = typeCode[0]
+
+		if baseType=="A":
+			return "text"
+			
+		elif baseType=="I":
+			return {
+				"I1": "smallint",
+				"I2": "smallint",
+				"I4": "integer"}.get(typeCode, "bigint")
+
+		elif baseType in "FE":
+			if int(typeCode[1:].split(".")[0])>7:
+				return "double precision"
+			else:
+				return "real"
+
+	def _shipout(self):
+		"""takes infoSoFar and makes bytesForCols and colAttrs entries from
+		it.
+		"""
+		unit = self.infoSoFar["units"]
+		rec = [
+			("name", self.infoSoFar["label"]),
+			("unit", None if unit=="---" else unit),
+			("ucd", ""),
+			("verbLevel", "1"),
+			("type", self._getTypeForTypeCode(self.infoSoFar["format"])),
+			("description", self.infoSoFar["description"].strip())]
+		self.colAttrs.append(dict((k,v) for k,v in rec if v is not None))
+
+		self.bytesForCols.append("{label}: {from}-{to}".format(**self.infoSoFar))
+
+	def feed(self, line):
+		getattr(self, "_parse_"+self.state)(line)
+
+
+def iterColAttrsViz(inFile, dumpRanges=False):
+	"""tries to parse (text) inFile as a VizieR-Style bytes-by-bytes
+	description and yields colAttrs for the first described table.
+
+	dumpRanges is for interactiv use, where the function additionally
+	dumps a colDefs content to stdout.
+	"""
+	parser = VizBBBParser()
+	for ln in inFile:
+		parser.feed(ln)
+	
+	if parser.state!="finished":
+		base.ui.notifyWarning("Probably invalid VizieR byte-by-byte: parser"
+			f" ended up in {parser.state}")
+
+	if dumpRanges:
+		print("\n".join(parser.bytesForCols)+"\n")
+
+	return parser.colAttrs
+
+
+COLUMN_ELEMENT_TEMPLATES = ['<column', 
+	' name="{name}"', ' type="{type}"',
+	'\n  ', 'unit="{unit}" ', 'ucd="{ucd}"', 
+	'\n  utype="{utype}"',
+	'\n  tablehead="{tablehead}"',
+	'\n  description="{description}"',
+	'\n  displayHint="{displayHint}"',
+	'\n  verbLevel="{verbLevel}"',
+	"/>"]
+
+def formatColumnElement(colAttrs):
+	"""returns a column element XML literal for colAttrs in Markus'
+	preferred formatting.
+	"""
+	accum = []
+	for template in COLUMN_ELEMENT_TEMPLATES:
+		try:
+			accum.append(template.format(**colAttrs))
+		except KeyError:
+			# skip this template element as there's no value for it
+			pass
+	return "".join(accum)
+
+
+def getColumnXML(colAttrs):
+	"""returns formatted XML for a sequence for column attributes.
+
+	colAttrs is a sequence of dicts with fillers into COLUMN_ELEMENT_TEMPLATES; 
+	the must already be escaped for use in XML attributes.
+	"""
+	accum = []
+	for fillers in colAttrs:
+		accum.append(formatColumnElement(fillers))
+	return "\n".join(accum)
+
+
+def parseGenColCommandLine():
+	import argparse
+	parser = argparse.ArgumentParser(description="Write column definitions"
+		" for representing a table")
+	parser.add_argument("table", metavar="FILE", action="store",
+		type=str, help="Source file to generate the columns from."
+		" This can be a FITS, a VOTable, or Vizier Column-by-column"
+		" description, and DaCHS will guess based on the extension.  Use"
+		" the format option if it guesses wrong")
+
+	parser.add_argument("-f", "--format", dest="format", action="store",
+		choices=["fits", "vot", "viz"],
+		help="Format of the input table: FITS binary, VOTable, or Vizier"
+			" byte-by-byte.")
+		
+	return parser.parse_args()
+
+
+def gencol():
+	"""a UI function generating columns element from FITS binary tables.
+	"""
+	from gavo.utils import pyfits
+
+	args = parseGenColCommandLine()
+
+	if args.table.endswith(".fits") or args.format=="fits":
+		try:
+			table = pyfits.open(args.table)[1]
+			colIterator = iterColAttrsFITS(table)
+		except Exception as exc:
+			base.ui.notifyError(f"Cannot open {args.table} as a FITS binary table: "
+				+str(exc))
+			sys.exit(1)
+	
+	elif (args.table.endswith(".vot") 
+			or args.table.endswith(".xml")
+			or args.format=="vot"):
+		try:
+			with open(args.table, "rb") as f:
+				colIterator = list(iterColAttrsVOTable(f))
+		except Exception as exc:
+			base.ui.notifyError(f"Cannot open {args.table} as a VOTable: "
+				+str(exc))
+	
+	elif (args.table.endswith(".txt")
+			or args.table=="README" 
+			or args.format=="viz"):
+		try:
+			with open(args.table, "rb") as f:
+				colIterator = iterColAttrsViz(f, True)
+		except Exception as exc:
+			base.ui.notifyError(f"Cannot open {args.table} as a VizieR"
+				" Byte-by-Byte file: "+str(exc))
+	
+	else:
+		base.ui.notifyError(f"Do not know how to read {args.table}.  Please"
+			" use --format")
+		sys.exit(1)
+
+	print(utils.fixIndentation(
+		getColumnXML(
+			colIterator), "    "))
+
+
+# ====================== Begin templating =========================
+
+
 def _listProtocols():
 	"""writes to stdout a list of protocols we can generate RD templates
 	for.
@@ -360,9 +644,9 @@ def _listProtocols():
 		print("%s -- %s"%(protName, protDesc))
 
 
-def parseNewCommandLine():
+def parseStartCommandLine():
 	import argparse
-	parser = argparse.ArgumentParser(description = "Write a template"
+	parser = argparse.ArgumentParser(description="Write a template"
 		" q.rd for a certain data type")
 	parser.add_argument("protocol", metavar="PROTO", action="store",
 		type=str, help="Generate an RD template for PROTO; use list to"
@@ -410,7 +694,7 @@ class MkrdMacroProcessor(macros.MacroExpander):
 
 
 def start():
-	args = parseNewCommandLine()
+	args = parseStartCommandLine()
 	outputName = "q.rd"
 
 	try:
@@ -429,3 +713,5 @@ def start():
 		rdSource = proc.expand(source.read().decode("utf-8"))
 		with open(outputName, "wb") as dest:
 			dest.write(rdSource.encode("utf-8"))
+	
+	source.close()
