@@ -2,7 +2,7 @@
 A wrapper script suitable for starting the server.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -40,12 +40,21 @@ from gavo.utils import exposedFunction, makeCLIParser, Arg
 from gavo.web import root
 
 
-def setupServer(rootPage):
+def setupServer(rootPage, runIntervalJobs=True):
+	"""sets the process up for running a server.
+
+	This involves managing server resource limits, doing a notification,
+	and, in particular, starting up the scheduler.
+	
+	runIntervalJobs can be set to False to suppress running jobs
+	that are probably maintenance jobs on a running instance and
+	which would be unwelcome on debug instances.
+	"""
 	manageResourceLimits()
 	config.setMeta("upSince", utils.formatISODT(datetime.datetime.utcnow()))
 	base.ui.notifyWebServerUp()
 
-	if base.DEBUG:
+	if not runIntervalJobs:
 		# we don't want most periodic stuff to happen when in debug mode, since
 		# it usually will involve fetching or importing things, and it's at
 		# best going to be confusing.  However, stuff that is supposed to
@@ -143,6 +152,13 @@ def manageResourceLimits():
 			(configuredLimit, configuredLimit))
 		_, hard = resource.getrlimit(resource.RLIMIT_CORE)
 		resource.setrlimit(resource.RLIMIT_CORE, (hard, hard))
+
+		# for an actual coredump in production systems (which setuid)
+		# you will usually also need 
+		# echo 2 >/proc/sys/fs/suid_dumpable
+		# and this (needs python3-prctl installed):
+		# import prctl; prctl.set_dumpable(1)
+
 	except (ValueError, resource.error):
 		# it's perfectly legal to call this as an unprivileged user, so
 		# we won't complain here.
@@ -363,7 +379,7 @@ def _perhapsEnableSSL(factory, sslPort=443):
 		log.err("Not turning on TLS because: "+str(msg))
 
 
-def _startServer():
+def _startServer(managePID=True):
 	"""runs a detached server, dropping privileges and all.
 	"""
 	try:
@@ -383,8 +399,9 @@ def _startServer():
 	os.chdir(base.getConfig("statedir"))
 	_dropPrivileges()
 	root.site.webLog = _configureTwistedLog()
-	
-	PIDManager.setPID()
+
+	if managePID:
+		PIDManager.setPID()
 	try:
 		setupServer(root)
 		signal.signal(signal.SIGHUP, lambda sig, stack: 
@@ -395,7 +412,8 @@ def _startServer():
 				1, threads.deferToThread, _preloadPublishedRDs)
 		reactor.run()
 	finally:
-		PIDManager.clearPID()
+		if managePID:
+			PIDManager.clearPID()
 
 
 @exposedFunction([
@@ -407,21 +425,23 @@ def _startServer():
 	" background by default).",
 	)
 def start(args):
-	oldPID = PIDManager.getPID()
-	if oldPID is not None:  # Server could already be running,.. .
-		if os.path.exists("/proc/%s"%oldPID):
-			# ...if the PID is active, give up right away
-			sys.exit("It seems there's already a server (pid %s) running."
-				" Try 'dachs serve stop'."%(PIDManager.getPID()))
-		else:
-			warnings.warn("Unclean server shutdown suspected, trying to clean up...")
-			_stopServer()
+	if not getattr(args, "foreground", False):
+		oldPID = PIDManager.getPID()
+		if oldPID is not None:  # Server could already be running,.. .
+			if os.path.exists(f"/proc/{oldPID}"):
+				# ...if the PID is active, give up right away
+				sys.exit("It seems there's already a server (pid %s) running."
+					" Try 'dachs serve stop'."%(PIDManager.getPID()))
+			else:
+				warnings.warn(
+					"Unclean server shutdown suspected, trying to clean up...")
+				_stopServer()
 
 	# this is called here because we might still be root here and
 	# can raise our hard limits.
 	manageResourceLimits()
 	if getattr(args, "foreground", False):
-		_startServer()
+		_startServer(managePID=False)
 	else:
 		daemonize(
 			getLogFile("server.stderr"),
@@ -533,7 +553,7 @@ def debug(args):
 	# since we don't support non-443 https, really, fudge this to enable
 	# debugging
 	base.getHTTPSBase = lambda:"https://localhost:40443"
-	setupServer(root)
+	setupServer(root, runIntervalJobs=False)
 	reactor.run()
 
 
