@@ -13,12 +13,14 @@ upgrade does everything required to bring the what's in the database in sync
 with the code (or so I hope).
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
 
+import os
+import re
 import sys
 
 from psycopg2 import extensions
@@ -35,7 +37,7 @@ class _COMMIT(object):
 	"""
 
 
-CURRENT_SCHEMAVERSION = 25
+CURRENT_SCHEMAVERSION = 29
 
 
 def getColumnNamesFor(tableId, connection):
@@ -75,6 +77,22 @@ def _updateTAP_SCHEMA(connection):
 			continue
 
 
+def iterEPNTAPTables(connection):
+	"""yields table descriptors for all tables that claim to conform to EPN-TAP.
+	"""
+	mth = base.caches.getMTH(None)
+
+	for tableName, in connection.query("select table_name"
+			" from tap_schema.tables"
+			" where utype in ("
+			"   'ivo://vopdc.obspm/std/epncore#schema-2.0',"
+			"   'ivo://ivoa.net/std/epntap#table-2.0')"):
+		try:
+			yield mth.getTableDefForTable(tableName)
+		except:
+			base.ui.notifyWarning(f"Published EPN-TAP table {tableName}"
+				" has no definition any more")
+
 class AnnotatedString(str):
 	"""a string with an annotation.
 	
@@ -735,18 +753,110 @@ class To25Upgrader(Upgrader):
 
 	@classmethod
 	def u_000_hash_passwords(cls, connection):
-		"""scrypt-hash existing passwords"""
+		"""scrypt-hashing existing passwords"""
 		from gavo.protocols import creds
 		updates = []
 		for username, password in connection.query(
 				"SELECT username, password FROM dc.users"):
-			updates.append((username, creds.hashPassword(password)))
+			if not re.match("[a-z]+:", password):
+				updates.append((username, creds.hashPassword(password)))
 
 		for username, hash in updates:
 			connection.execute("UPDATE dc.users SET password=%(hash)s"
 				" WHERE username=%(username)s", locals())
 
 
+class To26Upgrader(Upgrader):
+	version = 25
+
+	@classmethod
+	def u_000_give_extra_float_digits(cls, connection):
+		"""ensuring postgres doesn't lose bits in floats"""
+		with base.getWritableAdminConn() as suconn:
+			for profileName in ["untrustedquery", "trustedquery", "admin"]:
+				profile = base.getDBProfile(profileName)
+				suconn.execute("alter role %s set extra_float_digits=3"%profile.user)
+
+
+class To27Upgrader(Upgrader):
+	version = 26
+
+	@classmethod
+	def s_010_create_col_stats_table(cls, connection):
+		"""Creating the dc.simple_col_stats table"""
+		if not relationExists("dc.simple_col_stats", connection):
+			# this will first make tableName alone a primary key on dc.tablemeta
+			tablemeta = rsc.TableForDef(
+				base.resolveCrossId("//dc_tables#tablemeta"), 
+				connection=connection)
+			tablemeta.updateMeta()
+			tablemeta._dropPrimaryKey()
+			tablemeta.makeIndices()
+
+			rsc.makeData(base.resolveCrossId("//dc_tables#import"),
+				connection=connection, runCommit=False)
+
+	@classmethod
+	def s_020_add_coverage_to_tablemeta(cls, connection):
+		"""Adding a coverage column to rdmeta"""
+		cols = getColumnNamesFor("//rds#rdmeta", connection)
+		if "spatial" not in cols:
+			connection.execute(
+				"ALTER TABLE dc.rdmeta ADD COLUMN spatial TEXT")
+			connection.execute(
+				"ALTER TABLE dc.rdmeta ADD COLUMN temporal double precision[]")
+			connection.execute(
+				"ALTER TABLE dc.rdmeta ADD COLUMN spectral double precision[]")
+
+	@classmethod
+	def u_30_update_obscore_meta(cls, connection):
+		"""Re-import obscore metadata to update for UCD erratum"""
+		if list(connection.query(
+				"select * from dc.tablemeta where tablename ILIKE 'ivoa.ObsCore'")):
+			from gavo.protocols import tap
+			rd = base.caches.getRD("//obscore")
+			tap.publishToTAP(rd, connection)
+		else:
+			showProgress(" (not present)")
+
+
+class To28Upgrader(Upgrader):
+	version = 27
+
+	@classmethod
+	def u_10_update_TAP_SCHEMA_META(cls, connection):
+		"""Declaring TAP_SCHEMA descriptions as unicode"""
+		from gavo.protocols import tap
+		tap._insertRDIntoTAP_SCHEMA(rscdesc.getRD("//tap"), connection)
+
+
+class To29Upgrade(Upgrader):
+	version = 28
+
+	@classmethod
+	def u_10_update_EPNTAP_meta(cls, connection):
+		"""Updating EPNTAP2 metadata where necessary and possible"""
+		rds = set()
+		for td in iterEPNTAPTables(connection):
+			try:
+				rds.add(td.rd.sourceId)
+				table = rsc.DBTable(td, connection=connection)
+				table.updateMeta()
+			except Exception as msg:
+				base.ui.notifyWarning("Could not update EPN-TAP table"
+					f" {td.getQName()}: {msg}.  Please fix manually.")
+
+		from gavo.protocols import tap
+
+		for rdId in rds:
+			try:
+				tap.publishToTAP(
+					base.caches.getRD(rdId), connection)
+			except Exception as msg:
+				base.ui.notifyWarning("Could not update TAP metadata for"
+					f" {rdId}: {msg}.  Please fix manually.")
+
+
 def iterStatements(startVersion, endVersion=CURRENT_SCHEMAVERSION, 
 		upgraders=None):
 	"""yields all upgraders from startVersion to endVersion in sequence.
@@ -807,6 +917,22 @@ def upgrade(forceDBVersion=None):
 				conn.execute(statement)
 			showProgress(" ok\n")
 	
+	# other chores; add them below for the time being.
+	clearThirdPartyCache()
+
+
+################## misc chores
+# if we get more of those, I guess we ought to have
+# a module for those and report on them
+def clearThirdPartyCache():
+	from gavo.web import ifpages
+	tpcDir = ifpages.ThirdPartyCachePage.cacheDir
+	if os.path.isdir(tpcDir):
+		for fName in os.listdir(tpcDir):
+			path = os.path.join(tpcDir)
+			if os.path.isfile(path):
+				os.unlink(path)
+
 
 def parseCommandLine():
 	import argparse
@@ -820,6 +946,11 @@ def parseCommandLine():
 		" print nothing if no extensions need updating).  This will return"
 		" 0 if material was written, 1 otherwise.",
 		dest="dumpExtScript", action="store_true")
+	parser.add_argument("-t", "--update-tap-schema",
+		help="Force the re-generation of the TAP schema.  This is basic"
+		" housekeeping that's perhaps smart to do on the occasion of"
+		" an upgrade.", 
+		dest="updateTAPSchema", action="store_true")
 	return parser.parse_args()
 
 
@@ -830,3 +961,7 @@ def main():
 			sys.exit(dumpExtensionUpdater(conn))
 	else:
 		upgrade(args.forceDBVersion)
+	
+	if args.updateTAPSchema:
+		with base.getWritableAdminConn() as conn:
+			_updateTAP_SCHEMA(conn)
