@@ -5,7 +5,7 @@ This comprises helpers and wrappers that do not need gavo.base but for some
 reason or another should be within the dc package.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -16,6 +16,9 @@ reason or another should be within the dc package.
 
 import os
 
+# do any necessary monkeypatching before doing anything else
+import gavo.utils.monkeypatch
+
 from gavo.utils.algotricks import (
 	chunk, identity, topoSort, commonPrefixLength, uniqueItems)
 
@@ -51,19 +54,21 @@ from gavo.utils.mathtricks import *
 
 from gavo.utils.misctricks import (Undefined, QuotedName, getfirst,
 	logOldExc, sendUIEvent, 
-	pyparsingWhitechars, pyparsing,
 	getWithCache,
 	rstxToHTML, rstxToHTMLWithWarning, 
 	couldBeABibcode,
-	pyparseString, pyparseTransform, parseKVLine, makeKVLine,
+	parseKVLine, makeKVLine,
 	StreamBuffer, CaseSemisensitiveDict,
 	NotInstalledModuleStub, grouped,
 	getCleanBytes, getDirtyBytes)
 
-from gavo.utils.ostricks import (safeclose, urlopenRemote, 
+from gavo.utils.ostricks import (safeclose, 
+	urlopenRemote, setUserAgent,
 	fgetmtime, cat, ensureDir, safeReplaced,
 	Arg, exposedFunction, makeCLIParser, StatusDisplay)
 
+from gavo.utils.parsetricks import pyparseString, pyparseTransform
+
 from gavo.utils.plainxml import StartEndHandler, iterparse, traverseETree
 
 from gavo.utils.serializers import (defaultMFRegistry, registerDefaultMF)
