@@ -18,7 +18,7 @@ become an even worse nightmare than it already is.
 """
 
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -250,5 +250,5 @@ class SkipThis(ExecutiveAction):
 	"""
 
 
-from pyparsing import ( #noflake: exported name
+from gavo.utils.parsetricks import ( #noflake: exported name
 	ParseBaseException as ParseException)
