@@ -42,7 +42,7 @@ For examples of how this is used, see pftf (http://www.tfiu.de/pftf)
 or DaCHS (http://soft.g-vo.org/dachs)
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -59,7 +59,7 @@ import weakref
 defaultSection = "general"  # must be all lowercase
 
 # Set to true if avoid making a bad config item bomb out
-BAD_CONFIG_ITEM_JUST_WARNS = False
+BAD_CONFIG_ITEM_JUST_WARNS = True
 
 
 class ConfigError(Exception):
@@ -107,18 +107,18 @@ class ConfigItem(object):
 	used  by clients), and a description.  The origin is important
 	for distinguishing what to save.
 
-	You need to define the _parse and _unparse methods in deriving
-	classes.  The _parse methods must take a string and return anything or 
+	You need to define the _parse and _unparse methods when deriving
+	subclasses.  The _parse methods must take a string and return anything or 
 	raise ParseErrors (with
 	a sensible description of the problem) if there is a problem
-	with the input; the must not raise other exceptions when passed a
-	string (but may do anything when passed something else.  _unparse
+	with the input; they must not raise other exceptions when passed a
+	string (but may do anything when passed something else).  _unparse
 	methods must not raise exceptions, take a value as returned
 	by parse (nothing else must be passed in) and return a
 	string that _parse would parse into this value.
 
 	Thus, the set method *only* takes strings as values.  To set
-	parsed values, assign to value directly.  However, _unparse
+	parsed values, assign to the value attribute directly.  However, _unparse
 	methods are not required to cope with any crazy stuff you enter
 	in this way, and thus you suddenly may receive all kinds of
 	funny exceptions when serializing a Configuration.
@@ -202,7 +202,7 @@ class IntConfigItem(ConfigItem):
 
 	It supports a Null value through the special None literal.
 
-	>>> ci = IntConfigItem("foo"); print ci.value
+	>>> ci = IntConfigItem("foo"); print(ci.value)
 	None
 	>>> ci = IntConfigItem("foo", default="23"); ci.value
 	23
@@ -221,7 +221,8 @@ class IntConfigItem(ConfigItem):
 		try:
 			return int(value)
 		except ValueError:
-			raise ParseError("%s is not an integer literal"%value)
+			raise ParseError(f"{value} is not an integer literal"
+				)
 
 	def _unparse(self, value):
 		return str(value)
@@ -232,7 +233,7 @@ class FloatConfigItem(ConfigItem):
 
 	It supports a Null value through the special None literal.
 
-	>>> ci = FloatConfigItem("foo"); print ci.value
+	>>> ci = FloatConfigItem("foo"); print(ci.value)
 	None
 	>>> ci = FloatConfigItem("foo", default="23"); ci.value
 	23.0
@@ -251,7 +252,8 @@ class FloatConfigItem(ConfigItem):
 		try:
 			return float(value)
 		except ValueError:
-			raise ParseError("%s is not a floating point literal"%value)
+			raise ParseError("%s is not a floating point literal"%value
+				)
 
 	def _unparse(self, value):
 		return repr(value)
@@ -271,12 +273,12 @@ class ListConfigItem(StringConfigItem):
 	([], '')
 	>>> ci.set(ci.getAsString());ci.value
 	[]
-	>>> ci.set("3, 2, 1, Z\xc3\xbcndung"); ci.value, ci.getAsString()
-	([u'3', u'2', u'1', u'Z\xfcndung'], '3, 2, 1, Z\xc3\xbcndung, ')
+	>>> ci.set("3, 2, 1, Zündung"); ci.value, ci.getAsString()
+	(['3', '2', '1', 'Zündung'], '3, 2, 1, Zündung, ')
 	>>> ci.set(",");ci.value
-	[u'']
+	['']
 	>>> ci.set(ci.getAsString());ci.value
-	[u'']
+	['']
 	"""
 
 	typedesc = "list of strings"
@@ -301,6 +303,9 @@ class SetConfigItem(ListConfigItem):
 	def _parse(self, value):
 		return set(ListConfigItem._parse(self, value))
 
+	def _unparse(self, value):
+		return StringConfigItem._unparse(self, ", ".join(value))
+		
 
 class IntListConfigItem(ListConfigItem):
 	"""A ConfigItem containing a comma separated list of ints.
@@ -315,7 +320,8 @@ class IntListConfigItem(ListConfigItem):
 	[3, 2, 1]
 	>>> ci.set("1, 2, 3, rubbish")
 	Traceback (most recent call last):
-	ParseError: Non-integer in integer list
+		...
+	fancyconfig.ParseError: Non-integer in integer list
 	"""
 
 	typedesc = "list of integers"
@@ -325,7 +331,7 @@ class IntListConfigItem(ListConfigItem):
 		try:
 			return [int(s) for s in ListConfigItem._parse(self, value)]
 		except ValueError:
-			raise ParseError("Non-integer in integer list")
+			raise ParseError("Non-integer in integer list") from None
 	
 	def _unparse(self, value):
 		return ListConfigItem._unparse(self, [str(n) for n in value])
@@ -352,14 +358,15 @@ class DictConfigItem(ListConfigItem):
 
 	>>> ci = DictConfigItem("foo"); ci.value
 	{}
-	>>> ci.set("ab:cd, foo:Fu\xc3\x9f"); ci.value
-	{u'ab': u'cd', u'foo': u'Fu\xdf'}
+	>>> ci.set("ab:cd, foo:Fuß"); ci.value
+	{'ab': 'cd', 'foo': 'Fuß'}
 	>>> ci.getAsString();ci.set(ci.getAsString()); ci.value
-	'ab:cd, foo:Fu\xc3\x9f, '
-	{u'ab': u'cd', u'foo': u'Fu\xdf'}
+	'ab:cd, foo:Fuß, '
+	{'ab': 'cd', 'foo': 'Fuß'}
 	>>> ci.set("ab:cd, rubbish")
 	Traceback (most recent call last):
-	ParseError: 'rubbish' is not a valid mapping literal element
+		...
+	fancyconfig.ParseError: 'rubbish' is not a valid mapping literal element
 	"""
 
 	typedesc = "mapping"
@@ -372,8 +379,8 @@ class DictConfigItem(ListConfigItem):
 				k, v = item.split(":")
 				res[k.strip()] = v.strip()
 			except ValueError:
-				raise ParseError("'%s' is not a valid mapping literal element"%
-					item)
+				raise ParseError(f"'{item}' is not a valid mapping literal element"
+					) from None
 		return res
 	
 	def _unparse(self, value):
@@ -398,7 +405,8 @@ class BooleanConfigItem(ConfigItem):
 		elif value in self.falseLiterals:
 			return False
 		else:
-			raise ParseError("'%s' is no recognized boolean literal."%value)
+			raise ParseError(
+				f"'{value}' is no recognized boolean literal.")
 	
 	def _unparse(self, value):
 		return {True: "True", False: "False"}[value]
@@ -419,7 +427,8 @@ class EnumeratedConfigItem(StringConfigItem):
 		if default is None:
 			default = options[0]
 		self.options = set(options)
-		self.typedesc = "value from the list %s"%(", ".join(self.options))
+		self.typedesc = "value from the list %s"%(", ".join(
+			sorted(self.options)))
 		StringConfigItem.__init__(self, name, default, description)
 	
 	def _parse(self, value):
@@ -972,9 +981,16 @@ def _getTestSuite():
 	return suite, tests
 
 
-if __name__=="__main__": # pragma: no cover
-	import doctest, unittest
-	from . import fancyconfig
+def load_tests(loader, tests, ignore):
+	import doctest
+	import fancyconfig
 	suite, _ = _getTestSuite()
-	suite.addTest(doctest.DocTestSuite(fancyconfig))
-	unittest.TextTestRunner().run(suite)
+	tests.addTest(suite)
+	tests.addTest(doctest.DocTestSuite(fancyconfig))
+	return tests
+
+
+if __name__=="__main__": # pragma: no-cover
+	import unittest
+	suite = unittest.TestSuite()
+	unittest.TextTestRunner().run(load_tests(None, suite, None))
