@@ -2,7 +2,7 @@
 Various helpers that didn't fit into any other xTricks.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -10,7 +10,6 @@ Various helpers that didn't fit into any other xTricks.
 
 import base64
 import collections
-import contextlib
 import functools
 import io
 import os
@@ -617,6 +616,9 @@ class CaseSemisensitiveDict(dict):
 	def __contains__(self, key):
 		return dict.__contains__(self, key) or key.upper() in self._normCased
 
+	def __delitem__(self, key):
+		self.pop(key, None)
+
 	def get(self, key, default=None):
 		try:
 			return self[key]
@@ -634,7 +636,10 @@ class CaseSemisensitiveDict(dict):
 			if default is not KeyError:
 				return default
 			raise
-		
+
+	def copy(self):
+		return CaseSemisensitiveDict(dict.copy(self))
+
 	@property
 	def _normCased(self):
 		if self._normCasedCache is None:
@@ -672,94 +677,13 @@ def getDirtyBytes(b):
 		base64.b64decode(b))
 
 
-####################### Pyparsing hacks
-# This may not be the best place to put this, but I don't really have a
-# better one at this point.  We need some configuration of pyparsing, and
-# this is probably imported by all modules doing pyparsing.
-#
-# (1) When building grammars, always do so using the pyparsingWhitechars
-# context manager.  Building grammars is thread-safe, but different
-# grammars here use different whitespace conventions, so without
-# the c.m., you might get those messed up.
-#
-# (2) When parsing strings, *always* go through pyparseString(grammar,
-# string) and fellow functions whenever your code could run from within
-# the server (i.e., basically always outside of tests).
-# pyparsing is not thread-safe, and thus we'll need to shoehorn some
-# locking on top of it; I don't want to change the pyparsing methods
-# themselves since they may be called very frequently.
-#
-# TODO: make all pyparsing users import pyparsing from utils so we're
-# sure it's always properly initialised.
-
-import pyparsing
-from pyparsing import ParserElement, ParseExpression
-ParserElement.enablePackrat()
-
-_PYPARSE_LOCK = threading.RLock()
-
-@contextlib.contextmanager
-def pyparsingWhitechars(whiteChars):
-	"""a context manager that serializes pyparsing grammar compilation
-	and manages its whitespace chars.
-
-	We need different whitespace definitions in some parts of DaCHS.
-	(The default used to be " \\t" for a while, so this is what things
-	get reset to).
-
-	Since whitespace apparently can only be set globally for pyparsing,
-	we provide this c.m.  Since it is possible that grammars will be
-	compiled in threads (e.g., as a side effect of getRD), this is
-	protected by a lock.  This, in turn, means that this can 
-	potentially block for a fairly long time.
-
-	Bottom line: When compiling pyparsing grammars, *always* set
-	the whitespace chars explicitely, and do it through this c.m.
-	"""
-	_PYPARSE_LOCK.acquire()
-	ParserElement.setDefaultWhitespaceChars(whiteChars)
-	try:
-		yield
-	finally:
-		ParserElement.setDefaultWhitespaceChars(" \t")
-		_PYPARSE_LOCK.release()
-
-def pyparseString(grammar, string, **kwargs):
-	"""parses a string using a pyparsing grammar thread-safely.
-	"""
-	with _PYPARSE_LOCK:
-		res = grammar.parseString(string, **kwargs)
-		ParserElement.resetCache()
-		return res
-
-def pyparseTransform(grammar, string, **kwargs):
-	"""calls grammar's transformString method thread-safely.
-	"""
-	with _PYPARSE_LOCK:
-		return grammar.transformString(string, **kwargs)
-
-
-# we monkeypatch pyparsing newer than 1.5.2 to not deep copy
-# expressions; the way newer pyparsings have been doing this
-# is incompatible with the way we bind actions to parser elements
-# in ADQL (perhaps we should change this?).
-def _parse_expression_copy(self):
-	ret = super(ParseExpression,self).copy()
-	ret.exprs = self.exprs[:]  # CHANGE wrt upstream
-	return ret
-
-if pyparsing.__version__>"1.5.2":
-	ParseExpression.copy = _parse_expression_copy
-
-del _parse_expression_copy
-
-
 ######################### pyparsing-based key-value lines.  
 
-def _makeKVLGrammar():
-	from pyparsing import (
-		Word,alphas, QuotedString, Regex, OneOrMore)
+from gavo.utils.parsetricks import (
+	Word,alphas, QuotedString, Regex, OneOrMore, 
+	pyparsingWhitechars, pyparseString)
 
+def _makeKVLGrammar():
 	with pyparsingWhitechars(" \t"):
 		keyword = Word(alphas+"_")("key")
 		keyword.setName("Keyword")
