@@ -14,7 +14,7 @@ To specify the command line arguments to the function, use Args.  See
 admin.py for an example.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -135,6 +135,16 @@ _restrictedURLOpener.add_handler(urllib.request.UnknownHandler())
 _restrictedURLOpener.addheaders = [("user-agent", 
 	"GAVO DaCHS HTTP client")]
 
+def setUserAgent(userAgent):
+	"""sets the user agent string for requests through urlopenRemote.
+
+	This is a global setting and thus, in particular, nowhere near
+	thread-safe.
+	"""
+	assert len(_restrictedURLOpener.addheaders)==1
+	_restrictedURLOpener.addheaders = [("user-agent", userAgent)]
+
+
 def urlopenRemote(url, data=None, creds=(None, None), timeout=100):
 	"""works like urllib.urlopen, except only http, https, and ftp URLs
 	are handled.
