@@ -7,7 +7,7 @@ and you're done.
 All native representation is in rad.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -353,6 +353,22 @@ class SPoly(PgSAdapter):
 			nside=2**order, nest=True, inclusive=inclusive)
 		return SMoc.fromCells(order, pixels)
 
+	def getCenter(self):
+		"""returns an estimate for some sort of center of this polygon.
+
+		This is computed as the mean of the vertices (in unit sphere
+		representation), which, depending on the polygon, may be very far from the
+		polygon's center of mass.  Also, you can kill this by having all points
+		sit on a great circle, etc.
+
+		So: handle with care.  Doing this properly is hard.
+		"""
+		vertices = [numpy.array(mathtricks.spherToCart(p.x, p.y))
+			for p in self.points]
+		center = numpy.average(vertices, axis=0)
+		center = mathtricks.normalize3(center)
+		return mathtricks.cartToSpher(center)
+
 
 class SBox(PgSAdapter):
 	"""A spherical box from pgSphere.
@@ -481,7 +497,7 @@ class SMoc(PgSAdapter):
 		curPos = 0
 		cells = []
 
-		for item in re.split("[, ]", cellLiteral):
+		for item in re.split("[, \n\t]", cellLiteral):
 			if not item.strip():
 				continue
 
@@ -564,7 +580,7 @@ class SMoc(PgSAdapter):
 					rmin = rmax = cell
 
 			ranges.append(self._formatASCIIRange(rmin, rmax))
-			parts.append("%d/%s"%(order, ",".join(ranges)))
+			parts.append("%d/%s"%(order, " ".join(ranges)))
 
 		return " ".join(parts)
 
