@@ -2,11 +2,12 @@
 Formatting, text manipulation, string constants, and such.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
+
 import datetime
 import functools
 import math
@@ -524,14 +525,14 @@ def parseDefaultDatetime(literal):
 		return literal
 	if literal.endswith("Z"):
 		literal = literal[:-1]
-	# just nuke fractional seconds, they're trouble with strptime.
-	literal = literal.split(".")[0]
+
 	for format in _SUPPORTED_DT_FORMATS:
 		try:
 			return datetime.datetime(
 				*time.strptime(literal, format)[:6])
 		except ValueError:
 			pass
+
 	return parseISODT(literal)
 
 
