@@ -26,7 +26,7 @@ when code can rely on gavo.base; this module should be considered an
 implementation detail.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -107,7 +107,7 @@ class ToVOTableConverter(FromSQLConverter):
 		"scircle": ("double", "3", "circle"),
 		"spoly": ("double", "*", "polygon"),
 		"smoc": ("char", "*", "moc"),
-		"sbox": ("double", "4", "x:box"),
+		"sbox": ("double", "4", "x-box"),
 		"unicode": ("unicodeChar", "*", None),
 		"int4range": ("int", "2", "interval"),
 	}
@@ -122,6 +122,12 @@ class ToVOTableConverter(FromSQLConverter):
 		# consequence of TOPCAT workaround as of 2018
 		if type=="char" and length=='1':
 			length = None
+		
+		# timestamp and interval both need xtype, and that's not really defined.
+		# Oh, madness.  Do a secret handshake with the encoder for custom
+		# behaviour.
+		if type=="timestamp" and length=="2":
+			return "char", "19x2", "timestamp-interval"
 
 		if length is not None:
 			# Special handling for arrays of something
@@ -167,7 +173,7 @@ class FromVOTableConverter(object):
 		"circle": "scircle",
 		"polygon": "spoly",
 		"moc": "smoc",
-		"x:box": "sbox",
+		"x-box": "sbox",
 	}
 
 	def convert(self, type, arraysize, xtype=None):
