@@ -2,7 +2,7 @@
 Common definitions for the GAVO VOTable modules.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -157,12 +157,17 @@ def getXtypeEncoderCode(field):
 	if loopify is None:
 		return []
 
-	if (field.xtype=="adql:TIMESTAMP" # legacy, delete ~ 2020
+	if (field.xtype=="adql:TIMESTAMP" # legacy, delete ~ 2024
 			or field.xtype=="timestamp"):
 		return loopify([
 			"if isinstance(val, datetime.datetime):",
 			"  val = utils.formatISODT(val)"])
 
+	if field.xtype=="timestamp-interval": # local addition
+		return loopify([
+			"if isinstance(val, datetime.datetime):",
+			"  val = utils.formatISODT(val)"])
+
 	elif field.xtype=="dachs:DATE":
 		return loopify([
 			"if isinstance(val, datetime.date):",
@@ -239,7 +244,8 @@ def getXtypeDecoderCode(field):
 			"  val = pgsphere.SBox.fromDALI(val)"])
 
 	elif (field.xtype=="adql:TIMESTAMP"
-			or field.xtype=="timestamp"):
+			or field.xtype=="timestamp"
+			or field.xtype=="timestamp-interval"):
 		src.extend([
 			"  val = parseDefaultDatetime(val)"])
 
