@@ -4,7 +4,7 @@ An interface to querying TAP servers (i.e., a TAP client).
 This is deprecated.  Use pyvo.dal.tap instead.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -36,6 +36,10 @@ ABORTED = "ABORTED"
 UNKNOWN = "UNKNOWN"
 
 
+# override this as you see fit
+USER_AGENT = "Python TAP library http://soft.g-vo.org/subpkgs"
+
+
 debug = False
 
 
@@ -442,6 +446,8 @@ def _canUseFormEncoding(params):
 			return False
 	return True
 
+ACCEPTED_REDIRECTS = {301, 302, 303, 307}
+
 
 def request(scheme, host, path, data="", customHeaders={}, method="GET",
 		expectedStatus=None, followRedirects=False, setResponse=None,
@@ -475,7 +481,7 @@ def request(scheme, host, path, data="", customHeaders={}, method="GET",
 		assert False
 
 	headers = {"connection": "close",
-		"user-agent": "Python TAP library http://soft.g-vo.org/subpkgs"}
+		"user-agent": USER_AGENT}
 
 	if not isinstance(data, str):
 		if _canUseFormEncoding(data):
@@ -506,9 +512,7 @@ def request(scheme, host, path, data="", customHeaders={}, method="GET",
 		setResponse(resp.data)
 	conn.close()
 
-	if ((followRedirects and resp.status==303)
-			or resp.status==301
-			or resp.status==302):
+	if followRedirects and resp.status in ACCEPTED_REDIRECTS:
 		parts = urllib.parse.urlparse(resp.getheader("location"))
 		return request(parts.scheme, parts.netloc, parts.path,
 			method="GET", expectedStatus=expectedStatus, 
@@ -821,7 +825,7 @@ class ADQLSyncJob(_WithEndpoint):
 	def postToService(self, params):
 		return request(self.destScheme, self.destHost, self.destPath+"/sync", 
 			params,
-			method="POST", followRedirects=3, expectedStatus=200,
+			method="POST", followRedirects=5, expectedStatus=200,
 			setResponse=self._setErrorFromServer, timeout=self.timeout)
 
 	def delete(self, usePOST=None):
