@@ -2,7 +2,7 @@
 Common code for the nevow based interface.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -34,6 +34,7 @@ from gavo import svcs
 from gavo import utils
 from gavo.base import meta
 from gavo.protocols import creds
+from gavo.protocols import dali
 
 # a magic sentinel for the FieldStorage hack (see below)
 _FSHACK_MAGIC = "__FSHACK_MAGIC_KEY__"
@@ -94,7 +95,7 @@ class HTMLMetaBuilder(meta.MetaBuilder):
 
 		# for meta items rendering their children themselves (which return
 		# in IncludesChildren object), do not fold in rendered children
-		# (the None sentinel is handed in endKey)
+		# (the None sentinel is handled in endKey)
 		if isinstance(val, meta.IncludesChildren):
 			self.resultTree[-1].append(None)
 
@@ -361,11 +362,6 @@ class _UploadLimiter(object):
 	"""A wrapper class for upload destinations that limits writes to
 	limit bytes and sends a 413 if the limit is reached.
 	"""
-	message = (b"If you receive this error using sync, try again\r\n"
-		b"with async (the limits are more generous there).  If you\r\n"
-		b"receive this message with async, see if the upload can be\r\n"
-		b"reduced by transmitting only columns absolutely necessary\r\n")
-
 	def __init__(self, original, request, limit):
 		self.original, self.request, self.limit = original, request, limit
 		self.writtenSoFar = 0
@@ -377,18 +373,21 @@ class _UploadLimiter(object):
 		# ...about the same for the client attribute
 		if not hasattr(request, "client"):
 			request.client = getattr(request.channel, "client", b"(unknown client)")
+		self.discard = False
 	
 	def __getattr__(self, attrName):
 		return getattr(self.original, attrName)
 
 	def write(self, bytes):
+		if self.discard:
+			return
+
 		self.writtenSoFar += len(bytes)
 		if self.writtenSoFar>=self.limit:
-			self.request.setResponseCode(413, b"Request Entity Too Large")
-			self.request.setHeader("content-length", str(len(self.message)))
-			self.request.setHeader("content-type", "text/plain")
-			self.request.write(self.message)
-			self.request.finish()
+			# I wonder if there's a way to stop the client without
+			# brutally closing the connection (which would kill our
+			# error messages).
+			self.discard = True
 
 		else:
 			self.original.write(bytes)
@@ -396,9 +395,12 @@ class _UploadLimiter(object):
 	def finish(self):
 		"""break cycles and return our original.
 
-		This is mainly a curtesy to the garbage collector.
+		This is mainly a courtesy to the garbage collector.
 		"""
 		del self.request
+		if self.discard:
+			raise svcs.RequestEntityTooLarge(
+				f"Would only accept {self.limit} bytes of upload here.")
 		return self.original
 
 
@@ -449,7 +451,8 @@ class Request(server.Request):
 			maxUploadSize = 0
 
 		if self.content:
-			self.content = _UploadLimiter(self.content, self, maxUploadSize)
+			self.content = _UploadLimiter(
+				self.content, self, maxUploadSize)
 
 	def write(self, payload):
 		if getattr(self, "channel", None) is None:
@@ -642,7 +645,7 @@ class Request(server.Request):
 
 	def process(self):
 		# overridden because of the FieldStorage hack (see below);
-		# the magic is been bytified by twisted's requestReceived
+		# the _FSHACK_MAGIC has been bytified by twisted's requestReceived
 		# Also, only do this if fields hasn't been set by the chunked
 		# upload hack (which needs to do that early, before twisted
 		# butchers the value).
@@ -676,11 +679,26 @@ class Request(server.Request):
 		#   butchers it.
 		# * I'm reading in the whole request body into a BytesIO because
     #   python cgi really can't live without clength.
+		#
+		# * There's an URL mapping hack in there for legacy TAP for now.
+		#
 		self.content.seek(0,0)
 		self.args = {}
 
+		# Brute-force TAP compatibility redirect, 2021
+		# As long as old registry records may still contain the legacy
+		# TAP URL, just brute-force in the new one.  For now, DaCHS doesn't
+		# have a mechanism for replacing multiple segments of a path with
+		# multiple other segments (vanity can just do one).
+		# I suppose this ugly stuff can go ~2025; or we'll need an alternative
+		# way of doing this kind of thing.
+		path = path.replace(
+			b"__system__/tap/run/tap/", 
+			b"__system__/tap/run/")
+
 		self.method, self.uri = command, path
 		self.clientproto = version
+
 		x = self.uri.split(b'?', 1)
 
 		if len(x) == 1:
@@ -743,8 +761,11 @@ class Request(server.Request):
 				self.content.seek(0,0)
 
 		# take the upload limiter out of content; it's done its work
-		if hasattr(self.content, "finish"):
-			self.content = self.content.finish()
+		try:
+			if hasattr(self.content, "finish"):
+				self.content = self.content.finish()
+		except svcs.RequestEntityTooLarge as ex:
+			return dali.serveDALIError(self, ex, httpStatus=413)
 
 		self.process()
 
