@@ -2,7 +2,7 @@
 The form renderer is the standard renderer for web-facing services.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -392,7 +392,8 @@ class FormMixin(object):
 		self._addMetaFields(form)
 		self._addDefaults(request, form, self._defaultsForForm)
 
-		if self.name=="form":
+		if (self.name=="form" 
+				and self.service.getProperty("fixedFormat", "")!="True"):
 			form.addField("_OUTPUT", formal.String, 
 				formal.widgetFactory(serviceresults.OutputFormat, 
 					self.service, self.queryMeta),
@@ -449,7 +450,7 @@ class FormMixin(object):
 
 		It is here so we can add an error handler in submitAction.
 		"""
-		if self.queryMeta["format"] in ("HTML", ""):
+		if self.queryMeta.get("format") in ("HTML", None):
 			resultWriter = self
 		else:
 			resultWriter = serviceresults.getFormat(self.queryMeta["format"])
@@ -490,6 +491,7 @@ class FormRenderer(formal.ResourceWithForm,
 	runOnEmptyInputs = False
 	processOnGET = True
 	compute = True
+	defaultOutputFormat = "HTML"
 
 	def __init__(self, request, service):
 		grend.ServiceBasedPage.__init__(self, request, service)
