@@ -6,7 +6,7 @@ somehow acquire input (typically, nevow contexts) and then format
 the result for the user.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -554,6 +554,9 @@ class ResourceBasedPage(GavoPage):
 	attaching ?__nevow_form__=genForm&, and the soap renderer does
 	nontrivial things there).
 
+	Renderers capable of producing multiple output formats should
+	give a format key (suitable for formats.getKeyFor) in defaultOutputFormat.
+
 	Within DaCHS, this class is mainly used as a base for ServiceBasedRenderer,
 	since almost always only services talk to the world.  However,
 	we try to fudge render and data functions such that the sidebar works.
@@ -561,11 +564,13 @@ class ResourceBasedPage(GavoPage):
 	preferredMethod = "GET"
 	urlUse = "full"
 	resultType = None
+	defaultOutputFormat = "votable"
 	# parameterStyle is a hint for inputKeys how to transform themselves
 	# "clear" keeps types, "form" gives vizier-like expressions
 	# "vo" gives parameter-like expressions.
 	parameterStyle = "clear"
 	name = None
+	aliases = frozenset()
 
 	def __init__(self, request, rd):
 		nevowc.TemplatedPage.__init__(self)
@@ -673,6 +678,10 @@ class ServiceBasedPage(ResourceBasedPage):
 	def __init__(self, request, service):
 		ResourceBasedPage.__init__(self, request, service.rd)
 
+		if service.hasMeta("superseded"):
+			raise svcs.UnknownURI("This service is superseded by something else.",
+				htmlMessage=T.xml(service.getMeta("superseded").getContent("html")))
+
 		self.service = service
 		if not self.openRenderer and service.limitTo:
 			if not creds.hasCredentials(request.getUser(), request.getPassword(),
@@ -684,12 +693,6 @@ class ServiceBasedPage(ResourceBasedPage):
 				"The renderer %s is not allowed on this service."%self.name,
 				rd=self.service.rd)
 	
-		# the following attribute is checked for upstack by
-		# base.getCurrentServerURL
-		# NOTE: analogous code in svc.streaming.  If you change the logic here,
-		# you'll have to change the logic there, too.
-		self.HANDLING_HTTPS = request.isSecure()
-
 		self.metaCarrier = self.service
 		self.macroPackage = self.service
 
