@@ -2,7 +2,7 @@
 Infrastructure pages.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -15,10 +15,13 @@ import time
 
 import pkg_resources
 from twisted.internet import reactor
+from twisted.web import client
 from twisted.web import http
 from twisted.web import resource
+from twisted.web import server
 from twisted.web import static
 from twisted.web import template
+from twisted.web.template import tags as T
 
 from gavo import base
 from gavo import registry
@@ -27,10 +30,12 @@ from gavo import utils
 from gavo.base import meta
 from gavo.formal import nevowc
 from gavo.protocols import vocabularies
+from gavo.utils import stanxml
 from gavo.votable import V
 from gavo.web import common
 from gavo.web import caching
 from gavo.web import grend
+from gavo.web import weberrors
 
 # prefer globally installed version to what we deliver
 try:
@@ -366,6 +371,61 @@ class StaticServer(resource.Resource):
 			" neither built-in nor user-provided")
 
 
+class SchemaServer(nevowc.TemplatedPage, common.CommonRenderers):
+	"""a resource returning our built-in schema files.
+
+	This exposes stanxml's NSRegistry, i.e., everything that's been
+	introduced to DaCHS using registryPrefix at any point.
+
+	This both reacts to prefixes and to the last segments of schema locations.
+
+	This needs to look a bit deeper into stanxml.NSRegistry.  As long
+	as it's not operationally relevant, I'll accept using underscored
+	names here.
+
+	TODO: We actually have essentially all of these schema locally
+	(except we don't necessarily know their canonical prefixes), but
+	the NSRegistry so far only records locations on alnilam.  We
+	should probably return them from the local server.
+	"""
+	def data_schemadata(self, request, tag):
+		return list(
+			sorted(item for item in
+				stanxml.NSRegistry._schemaLocations.items()
+				if item[0] and item[1]))
+
+	@template.renderer
+	def schemalink(self, request, tag):
+		# data is an item from schemadata
+		return tag(href=tag.slotData[1])[tag.slotData[0]]
+
+	def getChild(self, name, request):
+		name = utils.debytify(name)
+		try:
+			_, loc = stanxml.NSRegistry.getPrefixInfo(name)
+			raise svcs.Found(loc)
+		except KeyError:
+			# it's not a prefix, try to get a schema for it
+			for loc in stanxml.NSRegistry._schemaLocations.values():
+				if loc and loc.split("/")[-1]==name:
+					raise svcs.Found(loc)
+
+		raise svcs.UnknownURI(f"'{name}' is neither a prefix nor a"
+			" schema name known here")
+
+	loader = common.doctypedStan(
+		T.html[
+			T.head(render="commonhead")[
+				T.title["Schema files known here"]],
+			T.body() [
+				T.h1["XML Schema Namespaces and Schemata known to DaCHS"],
+				nevowc.addNevowAttributes(
+						T.ul(render="sequence"),
+						data="schemadata") [
+					nevowc.addNevowAttributes(T.li, pattern="item")[
+						T.a(render="schemalink")]]]])
+							
+
 class RobotsTxt(resource.Resource):
 	"""A page combining some built-in robots.txt material with etc/robots.txt
 	if it exists.
@@ -475,6 +535,115 @@ class Teapot(resource.Resource):
 		return self
 
 
+class _ThirdPartyCacher(resource.Resource):
+	"""A page used by ThirdPartyCachePage in case of cache misses.
+
+	It is constructed with the key to ThirdPartyCachePage.urlMapping
+	and the parent third party cache.
+	"""
+	def __init__(self, keyToGet, parent):
+		self.keyToGet, self.parent = keyToGet, parent
+	
+	def render(self, request):
+		retriever = client.Agent(reactor)
+		d = retriever.request(b"GET", 
+			self.parent.urlMapping[self.keyToGet])
+		d.addCallback(self.startReadingRemote, request)
+		d.addErrback(self.stopAndForget, request)
+		return server.NOT_DONE_YET
+	
+	def startReadingRemote(self, response, request):
+		"""arranges for the agent response to be cached and then served to
+		request.
+		"""
+		reading = client.readBody(response)
+		reading.addErrback(self.acceptPartialDownloads, request)
+		reading.addCallback(self.saveAndServe, request)
+		reading.addErrback(self.stopAndForget, request)
+		return server.NOT_DONE_YET
+
+	def acceptPartialDownloads(self, failure, request):
+		"""An errback that lets us accept twisted's PartialDownloadError
+		as a successful download.
+
+		This needs to be added before startReadingRemote.
+		"""
+		failure.trap(client.PartialDownloadError)
+		if failure.value.status==b'200':
+			return failure.value.response
+		return failure
+
+	def saveAndServe(self, data, request):
+		"""writes data to the cache for the current document and then
+		serves that cache.
+		"""
+		with open(self.parent.getCacheName(self.keyToGet), "wb") as f:
+			f.write(data)
+		return self.serveFromCache(self.keyToGet, request)
+
+	def stopAndForget(self, failure, request):
+		return weberrors.renderDCErrorPage(failure, request)
+
+	def serveFromCache(self, name, request):
+		"""writes the cached resource name to the request and finished it.
+
+		If name isn't cached yet, this will raise a KeyError.
+		"""
+		res = self.parent.getFromCache(name)
+		return res.render(request)
+
+
+class ThirdPartyCachePage(resource.Resource):
+	"""A page returning third-party resources I don't want to distribute
+	with DaCHS for a variety of reason.
+
+	I still want them to come from the service itself, if only to
+	keep the number of third-party requests down.
+
+	These things are retrieved and cached on first access.  They are
+	removed on upgrades.
+	"""
+	#	NOTE: this maps bytes to bytes!
+	urlMapping = {
+		b"aladin.min.js": 
+			b"https://aladin.u-strasbg.fr/AladinLite/api/v2/latest/aladin.min.js",
+		b"aladin.min.css":
+			b"https://aladin.u-strasbg.fr/AladinLite/api/v2/latest/aladin.min.css",
+	}
+
+	cacheDir = os.path.join(base.getConfig("cacheDir"), "3rdparty")
+
+	def render(self, request):
+		raise svcs.UnknownURI("No representation here")
+
+	def getCacheName(self, name):
+		"""returns the name of a file the resource name would be
+		cached under.
+		"""
+		return os.path.join(self.cacheDir, utils.debytify(name))
+
+	def getFromCache(self, name):
+		"""returns a resource serving name if it's alreading in the cache,
+		raises a KeyError if not.
+		"""
+		fn = self.getCacheName(name)
+		if os.path.exists(fn):
+			return static.File(fn)
+		else:
+			os.makedirs(self.cacheDir, exist_ok=True)
+			raise KeyError(name)
+	
+	def getChild(self, name, request):
+		if name not in self.urlMapping:
+			raise svcs.UnknownURI(f"No cache rule for {name}")
+		try:
+			return self.getFromCache(name)
+		except KeyError:
+			# We'll have to retrieve the stuff yourselves.  That'll be done
+			# from the the render method
+			return _ThirdPartyCacher(name, self)
+
+
 ############### resources for the javascript-enabled root page
 # These once had a module of their own, which I didn't like.
 # Really, we'd like them in some RD, as they're really linked to
@@ -568,7 +737,8 @@ class ByAuthor(common.JSONQuery):
 class ServiceInfo(common.JSONQuery):
 	query = (
 		"SELECT title, description, authors,"
-		"    to_char(dateUpdated, 'YYYY-MM-DD') as lastupdate,"
+		"    case when dateUpdated is null then 'unknown'"
+		"       else to_char(dateUpdated, 'YYYY-MM-DD') end as lastupdate,"
 		"    referenceURL, accessURL"
 		"  FROM dc.interfaces"
 		"    NATURAL JOIN dc.sets"
