@@ -3,7 +3,7 @@
 Renderers that take services "as arguments".
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -187,13 +187,13 @@ class RendExplainer(object):
 			# Slightly obfuscated -- I need to get ", " in between the items.
 			fieldIter = iter(service.getInputKeysFor("soap"))
 			try:
-				next = next(fieldIter)
+				nextItem = next(fieldIter)
 				while True:
-					desc = "%s/%s"%(next.name, next.type)
-					if next.required:
+					desc = "%s/%s"%(nextItem.name, nextItem.type)
+					if nextItem.required:
 						desc = T.strong[desc]
 					yield desc
-					next = next(fieldIter)
+					nextItem = next(fieldIter)
 					yield ', '
 			except StopIteration:
 				pass
@@ -293,18 +293,20 @@ class RendExplainer(object):
 				"Validate"]]]
 
 	@classmethod
-	def _explain_tap(cls, service):
-		return T.transparent["the interface to this site's Table Access Protocol"
-			" service.  This protocol is best accessed using specialized clients"
-			" or libraries. In such clients, you can find this service by its"
+	def _explain_dali(cls, service):
+		return T.transparent["a complex interface consisting of multiple"
+			" endpoints (sync, async, perhaps tables or examples, and possibly"
+			" more).  This is used, in particular, for TAP.  You will usually need"
+			" a client programme to use this."
+			"  In such clients, you can find this service by its"
 			" IVOID, ",
 			T.code(render="meta")["identifier"], 
 			", or access it by entering its base URL ",
-			T.code[service.getURL("tap")],
+			T.code[service.getURL("dali")],
 			" directly.  Using an XSL-enabled web browser you can, in a pinch,"
 			" also operate ",
-			T.a(href=service.getURL("tap")+"/async")["the service"],
-			" without a specialized client."]
+			T.a(href=service.getURL("async"))["the service"],
+			" asynchronously without a specialized client."]
 
 	@classmethod
 	def _explain_uws_xml(cls, service):
@@ -698,8 +700,10 @@ class CoverageRenderer(MetaRenderer):
 class EditionRenderer(MetaRenderer):
 	"""A renderer representing a (tutorial-like) text document.
 
-	Not sure yet what I'll do when people actually call this; for now,
-	the access URL must be given as metadata.
+	Not sure yet what I'll do when people actually retrieve these.
+
+	This must have a meta accessURL with the document URI.  It may
+	have a sourceURL meta giving the VCS URI.
 	"""
 	name = "edition"
 
