@@ -2,7 +2,7 @@
 The root resource of the data center.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -133,7 +133,8 @@ def _upgradeInsecureRequests(request):
 	Otherwise, it returns None and the request needs to processed in the
 	normal way.
 	"""
-	if not base.LISTENING_TO_HTTPS:
+	if (not base.LISTENING_TO_HTTPS 
+			or base.getConfig("web", "ignore-uir-header")):
 		return
 
 	if request.method!=b"GET":
@@ -248,22 +249,22 @@ class ArchiveService(nevowc.TemplatedPage):
 				break
 		else:
 			raise UnknownURI("No matching RD")
-		
+
 		try:
 			subId, rendName = segments[srvInd], segments[srvInd+1]
 		except IndexError:
 			# a URL requesting a default renderer
 			subId, rendName = segments[srvInd], None
-			
+		
 		service = rd.getService(subId)
-		if service is None:
-			if rd.hasProperty("superseded-url"):
+		if not service:
+			if rd.hasMeta("superseded"):
 				return weberrors.NotFoundPageWithFancyMessage([
-					"This resource is stale and has been superseded; you should really"
-					" not be directed here.  Anyway, you might find what you"
-					" were looking for at this ",
-					T.a(href=rd.getProperty("superseded-url"))["new location"],
-					"."]), []
+					"This resource is stale and has been superseded.",
+					T.div(class_="superseded-message")[
+						T.xml(rd.getMeta("superseded").getContent("html"))]
+					]), []
+	
 			raise UnknownURI("No such service: %s"%subId, rd=rd)
 
 		if not rendName:
@@ -351,9 +352,11 @@ class ArchiveService(nevowc.TemplatedPage):
 
 ArchiveService.addStatic("login", ifpages.LoginPage())
 ArchiveService.addStatic("static", ifpages.StaticServer())
+ArchiveService.addStatic("schemata", ifpages.SchemaServer())
 ArchiveService.addStatic("robots.txt", ifpages.RobotsTxt())
 ArchiveService.addStatic("clientcount", ifpages.CurReaders())
 ArchiveService.addStatic("teapot", ifpages.Teapot())
+ArchiveService.addStatic("3rdparty", ifpages.ThirdPartyCachePage())
 
 # make these self-registering?  Or write them out somewhere?
 ArchiveService.addStatic("getRR", metarender.ResourceRecordMaker())
@@ -379,7 +382,6 @@ ArchiveService.installVanityMap()
 root = ArchiveService()
 
 
-
 site = server.Site(root, 
 	timeout=300,
 	logFormatter=getLogFormatter())
