@@ -1,8 +1,10 @@
 """
 Support for IVOA DAL and registry protocols.
+
+DALI-specified protocols are in dalirender, UWS/async is in asyncrender.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -23,7 +25,6 @@ from gavo import svcs
 from gavo import utils
 from gavo import votable
 from gavo.protocols import dali
-from gavo.protocols import dlasync
 from gavo.protocols import soda
 from gavo.protocols import ssap
 from gavo.svcs import streaming
@@ -35,11 +36,8 @@ from gavo.web import weberrors
 MS = base.makeStruct
 
 
-__docformat__ = "restructuredtext en"
-
-
 class DALRenderer(grend.ServiceBasedPage):
-	"""is a base class for renderers for the usual IVOA DAL protocols.
+	"""A base class for renderers for the usual IVOA DAL protocols.
 
 	This is for simple, GET-based DAL renderers (where we allow POST as 
 	well).  They work using nevow forms, but with standard-compliant error
@@ -50,12 +48,9 @@ class DALRenderer(grend.ServiceBasedPage):
 	resultType = base.votableType
 	urlUse = "base"
 	standardId = None
+	defaultOutputFormat = "votable"
 
 	def __init__(self, request, *args, **kwargs):
-		reqArgs = request.strargs
-		# XXX TODO: Do away with _FORMAT in general, move to RESPONSEFORMAT
-		reqArgs["_FORMAT"] = ["VOTable"]
-
 		grend.ServiceBasedPage.__init__(self, request, *args, **kwargs)
 		self.defaultLimit = base.getConfig("ivoa", "dalDefaultLimit")
 
@@ -79,7 +74,7 @@ class DALRenderer(grend.ServiceBasedPage):
 			runner = self.runSync
 		else:
 			runner = self.runAsync
-
+		
 		# overwrite previous query meta for new limit
 		self.queryMeta = svcs.QueryMeta.fromRequest(request, 
 			defaultLimit=self.defaultLimit)
@@ -120,13 +115,13 @@ class DALRenderer(grend.ServiceBasedPage):
 		"""
 		inputFields = []
 		for param in self.service.getInputKeysFor(self):
-			# Caution: UPLOAD mangling is a *renderer* thing -- the core
+			# Caution: UPLOAD mangling is a *renderer* thing -- the service
 			# doesn't know anything about it.  Hence, parameter adaption
-			# is *not* done by adapting the core.  Instead:
+			# is *not* done by adapting the real service metadata.  Instead:
 			if param.type=="file":
-				inputFields.append(dali.getUploadKeyFor(param))
-			else:
-				inputFields.append(param.change(name="INPUT:"+param.name))
+				param = dali.getUploadKeyFor(param)
+			inputFields.append(param.change(name="INPUT:"+param.name))
+
 		outputTD = self.service.core.outputTable.change(id="results")
 		for param in outputTD.params:
 			param.name = "OUTPUT:"+param.name
@@ -139,7 +134,7 @@ class DALRenderer(grend.ServiceBasedPage):
 
 		data = rsc.makeData(dataDesc)
 		data.tables["results"].votCasts = self._outputTableCasts
-		data.setMeta("_type", "results")
+		data.setMeta("_type", "meta")
 		data.addMeta("info", 
 			base.getMetaText(self.service, "title") or "Unnamed",
 			infoName="serviceInfo", 
@@ -199,12 +194,12 @@ class DALRenderer(grend.ServiceBasedPage):
 				infoName="standardID",
 				infoValue=self.standardId)
 
-		destFormat = base.votableType
-		if "RESPONSEFORMAT" in self.queryMeta.ctxArgs:
+		destFormat = self.defaultOutputFormat
+		if "responseformat" in self.queryMeta.ctxArgs:
 			# This is our DALI RESPONSEFORMAT implementation; the corresponding
 			# parameter is declared automatically in Service.completeElement
 			# from pql#DALIPars.
-			destFormat = self.queryMeta.ctxArgs["RESPONSEFORMAT"]
+			destFormat = self.queryMeta.ctxArgs["responseformat"]
 			requestedType = formats.getMIMEFor(destFormat, destFormat)
 		else:
 			requestedType = self.resultType
@@ -265,16 +260,14 @@ class SCSRenderer(DALRenderer):
 	version = "1.0"
 	parameterStyle = "dali"
 	standardId = "ivo://ivoa.net/std/ConeSearch"
+	defaultOutputFormat = "votable1.1"
 
 	# move the ucdCasts from formatOutput here.
 	_outputTableCasts = {}
 
 	def __init__(self, request, *args, **kwargs):
-		reqArgs = request.strargs
-		if "RESPONSEFORMAT" not in reqArgs:
-			reqArgs["RESPONSEFORMAT"] = ["votable1.1"]
-		self.defaultLimit = base.getConfig("ivoa", "dalDefaultLimit")*10
 		DALRenderer.__init__(self, request, *args, **kwargs)
+		self.defaultLimit = base.getConfig("ivoa", "dalDefaultLimit")*10
 
 	def _writeErrorTable(self, request, msg, httpStatus=200, queryStatus="ERROR"):
 		if not request.client:
@@ -295,6 +288,9 @@ class SCSRenderer(DALRenderer):
 		This comprises mapping meta.id;meta.main to ID_MAIN and
 		pos.eq* to POS_EQ*.
 		"""
+		if isinstance(data, tuple):
+			return DALRenderer._formatOutput(self, data, request, stream)
+
 		ucdCasts = {
 			"meta.id;meta.main": {"ucd": "ID_MAIN", "datatype": "char", 
 				"arraysize": "*"},
@@ -356,17 +352,50 @@ class SIAPRenderer(DALRenderer):
 	standardId = "ivo://ivoa.net/std/sia"
 
 	_outputTableCasts = {
-		"pixelScale": {"datatype": "double", "arraysize": "*"},
-		"wcs_cdmatrix": {"datatype": "double", "arraysize": "*"},
-		"wcs_refValues": {"datatype": "double", "arraysize": "*"},
-		"bandpassHi": {"datatype": "double"},
-		"bandpassLo": {"datatype": "double"},
-		"bandpassRefval": {"datatype": "double"},
-		"wcs_refPixel": {"datatype": "double", "arraysize": "*"},
-		"wcs_projection": {"arraysize": "3", "castFunction": lambda s: s[:3]},
+		"pixelScale": {
+			"datatype": "double", 
+			"arraysize": "*",
+			"ucd": "VOX:Image_Scale"},
+		"wcs_cdmatrix": {
+			"datatype": "double", 
+			"arraysize": "*",
+			"ucd": "VOX:WCS_CDMatrix"},
+		"wcs_refValues": {
+			"datatype": "double", 
+			"arraysize": "*",
+			"ucd": "VOX:WCS_CoordRefValue"},
+		"bandpassHi": {
+			"datatype": "double",
+			"ucd": "VOX:BandPass_HiLimit"},
+		"bandpassLo": {
+			"datatype": "double",
+			"ucd": "VOX:BandPass_LoLimit"},
+		"bandpassRefval": {
+			"datatype": "double",
+			"ucd": "VOX:BandPass_RefValue"},
+		"wcs_refPixel": {
+			"datatype": "double", 
+			"arraysize": "*",
+			"ucd": "VOX:WCS_CoordRefPixel"},
+		"wcs_projection": {
+			"arraysize": "3", 
+			"castFunction": lambda s: s[:3],
+			"ucd": "VOX:WCS_CoordProjection"},
 		"mime": {"ucd": "VOX:Image_Format"},
 		"accref": {"ucd": "VOX:Image_AccessReference"},
 		"accsize": {"datatype": "int"},
+		"centerAlpha": {"ucd": "POS_EQ_RA_MAIN"},
+		"centerDelta": {"ucd": "POS_EQ_DEC_MAIN"},
+		"imageTitle": {"ucd": "VOX:Image_Title"},
+		"instId": {"ucd": "INST_ID"},
+		"dateObs": {"ucd": "VOX:Image_MJDateObs"},
+		"nAxes": {"ucd": "VOX:Image_Naxes"},
+		"pixelSize": {"ucd": "VOX:Image_Naxis"},
+		"refFrame": {"ucd": "VOX:STC_CoordRefFrame"},
+		"wcs_equinox": {"ucd": "VOX:STC_CoordEquinox"},
+		"bandpassId": {"ucd": "VOX:BandPass_ID"},
+		"bandpassUnit": {"ucd": "VOX:BandPass_Unit"},
+		"pixflags": {"ucd": "VOX:Image_PixFlags"},
 	}
 
 	def _formatOutput(self, data, request, stream=True):
@@ -465,7 +494,6 @@ class SSAPRenderer(UnifiedDALRenderer):
 	 - ssap.dataSource -- survey, pointed, custom, theory, artificial
 	 - ssap.testQuery -- a query string that returns some data; REQUEST=queryData
 	   is added automatically
-	 - productType -- the term(s) from http://www.ivoa.net/rdf/product-type
 	   that describe the type of data served through the service.  Will
 	   usually by ``spectrum``, but ``timeseries`` is a realistic option.
 	
@@ -495,12 +523,7 @@ class SSAPRenderer(UnifiedDALRenderer):
 	name = "ssap.xml"
 	parameterStyle = "pql"
 	standardId = "ivo://ivoa.net/std/ssap"
-
-	def __init__(self, request, *args, **kwargs):
-		reqArgs = request.strargs
-		if "RESPONSEFORMAT" not in reqArgs:
-			reqArgs["RESPONSEFORMAT"] = ["votabletd"]
-		UnifiedDALRenderer.__init__(self, request, *args, **kwargs)
+	defaultOutputFormat = "votabletd"
 
 	def _getMetadataData(self):
 		data = UnifiedDALRenderer._getMetadataData(self)
@@ -615,13 +638,13 @@ class RegistryRenderer(grend.ServiceBasedPage):
 	def render(self, request):
 		# Make a robust (unchecked) pars dict for error rendering; real
 		# parameter checking happens in getPMHResponse
-		inData = {"args": [request.strargs]}
+		inData = {"args": request.strargs}
 
 		streamResponse = request.strargs.get("verb")==["ListRecords"]
 
 		self.runAsync(inData, 
 			).addCallback(self._renderXML, request, streamResponse
-			).addErrback(self._renderError, request, inData["args"][0])
+			).addErrback(self._renderError, request, inData["args"])
 		return server.NOT_DONE_YET
 
 	def _renderXML(self, resultTree, request, streamResponse=False):
@@ -851,6 +874,7 @@ class DatalinkGetMetaRenderer(_DatalinkRendererBase):
 		#
 		# remove this method once we've figured out how to placate these browsers.
 		mime, data = svcResult
+		# acceptDict = utils.parseAccept(request.getHeader("accept"))
 		if "Mozilla" in (request.getHeader("user-agent") or ""):
 			# it's a browser, do server-side XSLT
 			request.setHeader("content-type", "text/html;charset=utf-8")
@@ -862,54 +886,3 @@ class DatalinkGetMetaRenderer(_DatalinkRendererBase):
 			request.write(data)
 	
 		request.finish()
-
-
-class AsyncRendererBase(grend.ServiceBasedPage):
-	"""An abstract renderer for things running in a UWS.
-
-	To make these concrete, they need a name and a workerSystem attribute.
-	"""
-	parameterStyle = "pql"
-
-	def render(self, request):
-		# we don't distinguish between .../uws.xml and .../uws.xml/ -- should we?
-		res = self.getChild(None, request)
-		return res.render(request)
-
-	def getChild(self, name, request):
-		from gavo.web import asyncrender
-		
-		# TODO: We want to get the core and use that to parse request.strargs.
-		# But that'll also need the UWS service args, and so I'll have to
-		# reconsider this.  And hack it for now.
-		request.uwsArgs = {key.lower(): " ".join(value) 
-			for key, value in request.strargs.items()}
-		return asyncrender.getAsyncResource(request, 
-			self.workerSystem,
-			self.name, 
-			self.service,
-			name)
-
-
-class DatalinkAsyncRenderer(AsyncRendererBase):
-	"""A renderer for asynchronous datalink.
-	"""
-# TODO: I suspect this should go somewhere else, presumably together
-# with the stripped-down TAP renderer.
-	name = "dlasync"
-	workerSystem = dlasync.DL_WORKER
-
-
-class UWSAsyncRenderer(AsyncRendererBase):
-	"""A renderer speaking UWS.
-
-	This is for asynchronous exection of larger jobs.  Operators will normally
-	use this together with a custom core or a python core.
-
-	See `Custom UWSes`_ for details.
-	"""
-	name = "uws.xml"
-
-	@property
-	def workerSystem(self):
-		return self.service.getUWS()
